if (AjxPackage.define("ContactsCore")) {
/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite Web Client
 * Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011, 2013 Zimbra Software, LLC.
 * 
 * The contents of this file are subject to the Zimbra Public License
 * Version 1.4 ("License"); you may not use this file except in
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.zimbra.com/license.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * ***** END LICENSE BLOCK *****
 */
/*
 * Package: ContactsCore
 * 
 * Supports: Loading of contacts and address books
 * 
 * Loaded:
 * 	- When user contacts are loaded during startup
 * 	- If the <refresh> block has address books
 * 	- If a search for contacts returns results
 */
if (AjxPackage.define("ajax.dwt.widgets.DwtChooser")) {
/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite Web Client
 * Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011, 2013 Zimbra Software, LLC.
 * 
 * The contents of this file are subject to the Zimbra Public License
 * Version 1.4 ("License"); you may not use this file except in
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.zimbra.com/license.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * ***** END LICENSE BLOCK *****
 */

/**
 * Creates a control that allows the user to select items from a list, and
 * places the selected items in another list.
 * @constructor
 * @class
 * This class creates and manages a control that lets the user
 * select items from a list. Two lists are maintained, one with items to select 
 * from, and one that contains the selected items. Between them are buttons 
 * to shuffle items back and forth between the two lists.
 * <p>
 * There are two types of buttons: one or more transfer buttons move items from
 * the source list to the target list, and the remove button moves items from the
 * target list to the source list. The client can specify its transfer buttons.
 * If no specification is given, there will be a single transfer button called 
 * "Add".</p>
 * <p>
 * The parent must implement search(columnItem, ascending) if column sorting
 * is supported. It should also create a subclass of {@link DwtChooser} which returns
 * the appropriate source and target list views, themselves subclasses of
 * {@link DwtChooserListView}. Those subclasses must implement _getHeaderList() and
 * _createItemHtml(item).</p>
 * <p>
 * There are two different layout styles, horizontal (with the list views at the
 * left and right) and vertical (with the list views at the top and bottom). There
 * are two different selection styles, single and multiple, which control how many
 * items may appear in the target list view. There are two different transfer modes:
 * one where items are copied between lists, and one where they're moved.</p>
 *
 * @author Conrad Damon
 *
 * @param	{hash}		params		a hash of parameters
 * @param {DwtComposite}	params.parent			the containing widget
 * @param {string}	params.className		the CSS class
 * @param {string}	params.slvClassName	the CSS class for source list view
 * @param {string}	params.tlvClassName	the CSS class for target list view
 * @param {array}	params.buttonInfo		the id/label pairs for transfer buttons
 * @param {DwtChooser.HORIZ_STYLE|DwtChooser.VERT_STYLE}	params.layoutStyle	the layout style (vertical or horizontal)
 * @param {DwtChooser.SINGLE_SELECT|DwtChooser.MULTI_SELECT}	params.selectStyle	the multi-select (default) or single-select
 * @param {constant}	params.mode			the items are moved or copied
 * @param {boolean}	params.noDuplicates	if <code>true</code>, prevent duplicates in target list
 * @param {number}	params.singleHeight	the height of list view for single select style
 * @param {number}	params.listSize		the list width (if {@link DwtChooser.HORIZ_STYLE}) or height (if {@link DwtChooser.VERT_STYLE})
 * @param {boolean}	params.sourceEmptyOk	if <code>true</code>, do not show "No Results" in source list view
 * @param {boolean}	params.allButtons		if <code>true</code>, offer "Add All" and "Remove All" buttons
 * @param {boolean}	params.hasTextField	if <code>true</code>, create a text field for user input
 * 
 * @extends		DwtComposite
 */
DwtChooser = function(params) {

	if (arguments.length == 0) return;
	DwtComposite.call(this, params.parent, params.className);

	this._slvClassName = params.slvClassName;
	this._tlvClassName = params.tlvClassName;
	this._layoutStyle = params.layoutStyle ? params.layoutStyle : DwtChooser.HORIZ_STYLE;
	this._selectStyle = params.selectStyle ? params.selectStyle : DwtChooser.MULTI_SELECT;
	this._mode = params.listStyle ? params.listStyle : DwtChooser.MODE_MOVE;
	this._noDuplicates = (params.noDuplicates !== false);
	this._singleHeight = params.singleHeight ? params.singleHeight : 45; // 45 = header row + row with icon
	this._listSize = params.listSize;
	this._sourceEmptyOk = params.sourceEmptyOk;
	this._allButtons = params.allButtons;
	this._hasTextField = params.hasTextField;

	this._handleButtonInfo(params.buttonInfo);
	this._mode = params.mode ? params.mode :
						this._hasMultiButtons ? DwtChooser.MODE_COPY : DwtChooser.MODE_MOVE;

	this._createHtml();
	this._initialize();
	
	var parentSz = params.parent.getSize();
	var listWidth = params.listWidth || parentSz.x;
	var listHeight = params.listHeight || parentSz.y;
	if (listWidth && listHeight) {
		this.resize(listWidth, listHeight);
	}

};

DwtChooser.prototype = new DwtComposite;
DwtChooser.prototype.constructor = DwtChooser;

// Consts

// layout style
/**
 * Defines a "horizontal" layout style.
 */
DwtChooser.HORIZ_STYLE	= 1;
/**
 * Defines a "vertical" layout style.
 */
DwtChooser.VERT_STYLE	= 2;

// number of items target list can hold
/**
 * Defines a "single" select.
 */
DwtChooser.SINGLE_SELECT	= 1;
/**
 * Defines a "multi" select.
 */
DwtChooser.MULTI_SELECT		= 2;

// what happens to source items during transfer
DwtChooser.MODE_COPY	= 1;
DwtChooser.MODE_MOVE	= 2;

DwtChooser.REMOVE_BTN_ID = "__remove__";
DwtChooser.ADD_ALL_BTN_ID = "__addAll__";
DwtChooser.REMOVE_ALL_BTN_ID = "__removeAll__";

DwtChooser.prototype.toString = 
function() {
	return "DwtChooser";
};

/**
 * Sets the given list view with the given list. Defaults to source view.
 *
 * @param {AjxVector|Array|Object|Hash}	items			a list of items or hash of lists
 * @param {DwtChooserListView.SOURCE|DwtChooserListView.TARGET}	view			the view to set
 * @param {boolean}	clearOtherView	if <code>true</code>, clear out other view
 */
DwtChooser.prototype.setItems =
function(items, view, clearOtherView) {
	view = view ? view : DwtChooserListView.SOURCE;
	this._reset(view);
	this.addItems(items, view, true);
	this._selectFirst(view);
	if (clearOtherView) {
		this._reset((view == DwtChooserListView.SOURCE) ? DwtChooserListView.TARGET : DwtChooserListView.SOURCE);
	}
};	
	
/**
 * Gets a copy of the items in the given list. If that's the target list and 
 * there are multiple transfer buttons, then a hash with a vector for each one
 * is returned. Otherwise, a single vector is returned. Defaults to target view.
 *
 * @param {DwtChooserListView.SOURCE|DwtChooserListView.TARGET}	view			the view to set
 * @return	{AjxVector|array|Object|hash}		the item(s)
 */
DwtChooser.prototype.getItems =
function(view) {
	view = view ? view : DwtChooserListView.TARGET;
	if (view == DwtChooserListView.SOURCE) {
		return this.sourceListView.getList().clone();
	} else {	
		if (this._hasMultiButtons) {
			var data = {};
			for (var i in this._data) {
				data[i] = this._data[i].clone();
			}
			return data;
		} else {
			return this._data[this._buttonInfo[0].id].clone();
		}
	}
};

/**
 * Adds items to the given list view.
 *
 * @param {AjxVector|array|Object|hash}	items			a list of items or hash of lists
 * @param {DwtChooserListView.SOURCE|DwtChooserListView.TARGET}	view			the view to set
 * @param {boolean}	skipNotify	if <code>true</code>, do not notify listeners
 * @param {string}	id			the button ID
 */
DwtChooser.prototype.addItems =
function(items, view, skipNotify, id) {
	view = view ? view : DwtChooserListView.SOURCE;
	var list = (items instanceof AjxVector) ? items.getArray() : (items instanceof Array) ? items : [items];
	if (view == DwtChooserListView.SOURCE) {
		for (var i = 0; i < list.length; i++) {
			this._addToSource(list[i], null, skipNotify);
		}
	} else {
		var data;
		if (this._selectStyle == DwtChooser.SINGLE_SELECT) {
			this.targetListView._resetList();
			list = (list.length > 0) ? [list[0]] : list;
		}
		for (var i = 0; i < list.length; i++) {
			this._addToTarget(list[i], id, skipNotify);
//			if (this._selectStyle == DwtChooser.SINGLE_SELECT) {
//				return;
//			}
		}
	}
	if (view == DwtChooserListView.SOURCE) {
		var list = this.sourceListView.getList();
		this._sourceSize = list ? list.size() : 0;
	}
};

/**
 * Removes items from the given list view.
 *
 * @param {AjxVector|array|Object|hash}	list			a list of items or hash of lists
 * @param {DwtChooserListView.SOURCE|DwtChooserListView.TARGET}	view			the view to set
 * @param {boolean}	skipNotify	if <code>true</code>, do not notify listeners
 */
DwtChooser.prototype.removeItems =
function(list, view, skipNotify) {
	list = (list instanceof AjxVector) ? list.getArray() : (list instanceof Array) ? list : [list];
	for (var i = 0; i < list.length; i++) {
		(view == DwtChooserListView.SOURCE) ? this._removeFromSource(list[i], skipNotify) : this._removeFromTarget(list[i], skipNotify);
	}
};

/**
 * Moves or copies items from the source list to the target list, paying attention
 * to current mode.
 *
 * @param {AjxVector|array|Object|hash}	list			a list of items or hash of lists
 * @param {string}	id			the ID of the transfer button that was used
 * @param {boolean}	skipNotify	if <code>true</code>, do not notify listeners
 */
DwtChooser.prototype.transfer =
function(list, id, skipNotify) {
	id = id ? id : this._activeButtonId;
	this._setActiveButton(id);
	if (this._mode == DwtChooser.MODE_MOVE) {
		if (this._selectStyle == DwtChooser.SINGLE_SELECT) {
			var tlist = this.targetListView.getList();
			if (tlist && tlist.size()) {
				this.remove(tlist, true);
			}
		}
		this.removeItems(list, DwtChooserListView.SOURCE, true);
	}
	this.addItems(list, DwtChooserListView.TARGET, skipNotify);
	this.sourceListView.deselectAll();
};

/**
 * Removes items from target list, paying attention to current mode. Also handles button state.
 *
 * @param {AjxVector|array|Object|hash}	list			a list of items or hash of lists
 * @param {boolean}	skipNotify	if <code>true</code>, do not notify listeners
 */
DwtChooser.prototype.remove =
function(list, skipNotify) {
	list = (list instanceof AjxVector) ? list.getArray() : (list instanceof Array) ? list : [list];
	if (this._mode == DwtChooser.MODE_MOVE) {
		for (var i = 0; i < list.length; i++) {
			var index = this._getInsertionIndex(this.sourceListView, list[i]);
			this.sourceListView.addItem(list[i], index, true);
		}
		this._sourceSize = list ? list.length : 0;
	}
	this.removeItems(list, DwtChooserListView.TARGET);
};

/**
 * Sets the select style to the given style. Performs a resize
 * in order to adjust the layout, and changes the label on the transfer button if it's
 * the default one.
 *
 * @param {DwtChooser.SINGLE_SELECT|DwtChooser.MULTI_SELECT}	style		the style single or multiple select
 * @param {boolean}	noResize	if <code>true</code>, do not perform resize
 */
DwtChooser.prototype.setSelectStyle =
function(style, noResize) {
	if (style == this._selectStyle) return;
	
	this._selectStyle = style;
	if (this._defLabel) {
		var button = this._button[this._buttonInfo[0].id];
		button.setText((style == DwtChooser.SINGLE_SELECT) ? AjxMsg.select : AjxMsg.add);
	}
	if (!noResize) {
		var curSz = this.getSize();
		this.resize(curSz.x, curSz.y);
	}
	
	// "Add All" and "Remove All" buttons only shown if MULTI_SELECT
	if (this._allButtons) {
		this._addAllButton.setVisible(style == DwtChooser.MULTI_SELECT);
		this._removeAllButton.setVisible(style == DwtChooser.MULTI_SELECT);
		this._enableButtons();
	}

	// if we're going from multi to single, preserve only the first target item
	if (style == DwtChooser.SINGLE_SELECT) {
		var list = this.targetListView.getList();
		var a = list ? list.clone().getArray() : null;
		if (a && a.length) {
			this._reset(DwtChooserListView.TARGET);
			this.addItems(a[0], DwtChooserListView.TARGET, true);
			this.targetListView.deselectAll();
			if (a.length > 1 && this._mode == DwtChooser.MODE_MOVE) {
				this.addItems(a.slice(1), DwtChooserListView.SOURCE, true);
			}
			this._enableButtons();
		}
	}
	
	this.sourceListView.multiSelectEnabled = (style == DwtChooser.MULTI_SELECT);
	this.targetListView.multiSelectEnabled = (style == DwtChooser.MULTI_SELECT);
};

/**
 * Resets one or both list views, and the buttons. Defaults to resetting both list views.
 *
 * @param {DwtChooser.SINGLE_SELECT|DwtChooser.MULTI_SELECT}	style		the style single or multiple select
 */
DwtChooser.prototype.reset =
function(view) {
	this._reset(view);
	this._setActiveButton(this._buttonInfo[0].id); // make first button active by default
	this._enableButtons();
	if (this._hasTextField) {
		this._textField.setValue("");
	}
};

/**
 * Resets one or both list views. Defaults to resetting both list views.
 *
 * @param {DwtChooser.SINGLE_SELECT|DwtChooser.MULTI_SELECT}	style		the style single or multiple select
 * 
 * @private
 */
DwtChooser.prototype._reset =
function(view) {
	// clear out source list view and related data
	if (!view || view == DwtChooserListView.SOURCE) {
		this.sourceListView._resetList();
	}

	// clear out target list view and related data
	if (!view || view == DwtChooserListView.TARGET) {
		this.targetListView._resetList();
		for (var i in this._data) {
			this._data[i].removeAll();
		}
	}
};

/**
 * Adds a state change listener.
 *
 * @param {AjxListener}		listener	a listener
 */
DwtChooser.prototype.addStateChangeListener = 
function(listener) {
	this.targetListView.addStateChangeListener(listener);
};

/**
 * Removes a state change listener.
 *
 * @param {AjxListener}		listener	a listener
 */
DwtChooser.prototype.removeStateChangeListener = 
function(listener) {
	this.targetListView.removeStateChangeListener(listener);
};

/**
 * Gets the source <code>&lt;divgt;</code> that contains the source list view.
 * 
 * @return	{Element}		the element
 */
DwtChooser.prototype.getSourceListView = 
function() {
	return document.getElementById(this._sourceListViewDivId);
};

/**
 * Gets the source <code>&lt;div&gt;</code> that contains the buttons
 * 
 * @return	{Element}		the element
 */
DwtChooser.prototype.getButtons = 
function() {
	return document.getElementById(this._buttonsDivId);
};

/**
 * Gets the source <code>&lt;div&gt;</code> that contains the target list view.
 * 
 * @return	{Element}		the element
 */
DwtChooser.prototype.getTargetListView = 
function() {
	return document.getElementById(this._targetListViewDivId);
};

/**
 * Gets the text input field.
 * 
 * @return	{DwtInputField}		the text input field
 */
DwtChooser.prototype.getTextField =
function() {
	return this._textField;
};

/**
 * Creates the HTML framework, with placeholders for elements which are created later.
 * 
 * @private
 */
DwtChooser.prototype._createHtml = 
function() {

	this._sourceListViewDivId	= Dwt.getNextId();
	this._targetListViewDivId	= Dwt.getNextId();
	this._buttonsDivId			= Dwt.getNextId();
	this._removeButtonDivId		= Dwt.getNextId();
	if (this._allButtons) {
		this._addAllButtonDivId		= Dwt.getNextId();
		this._removeAllButtonDivId	= Dwt.getNextId();
	}
	if (this._hasTextField) {
		this._textFieldTdId = Dwt.getNextId();
	}

	var html = [];
	var idx = 0;
	
	if (this._layoutStyle == DwtChooser.HORIZ_STYLE) {
		// start new table for list views
		html[idx++] = "<table cellspacing=0 cellpadding=0 border=0>";
		html[idx++] = "<tr>";

		// source list
		html[idx++] = "<td id='";
		html[idx++] = this._sourceListViewDivId;
		html[idx++] = "'></td>";

		// transfer buttons
		html[idx++] = "<td valign='middle' id='";
		html[idx++] = this._buttonsDivId;
		html[idx++] = "'>";
		if (this._allButtons) {
			html[idx++] = "<div id='";
			html[idx++] = this._addAllButtonDivId;
			html[idx++] = "'></div><br>";
		}
		for (var i = 0; i < this._buttonInfo.length; i++) {
			var id = this._buttonInfo[i].id;
			html[idx++] = "<div id='";
			html[idx++] = this._buttonDivId[id];
			html[idx++] = "'></div><br>";
		}
		// remove button
		html[idx++] = "<br><div id='";
		html[idx++] = this._removeButtonDivId;
		html[idx++] = "'></div>";
		if (this._allButtons) {
			html[idx++] = "<br><div id='";
			html[idx++] = this._removeAllButtonDivId;
			html[idx++] = "'></div><br>";
		}
		html[idx++] = "</td>";

		// target list
		html[idx++] = "<td id='";
		html[idx++] = this._targetListViewDivId;
		html[idx++] = "'></td>";

		html[idx++] = "</tr>";
		
		if (this._hasTextField) {
			html[idx++] = "<tr><td>";
			html[idx++] = "<table border=0 width=100%><tr><td style='white-space:nowrap; width:1%'>";
			html[idx++] = AjxMsg.add;
			html[idx++] = ":</td><td id='";
			html[idx++] = this._textFieldTdId;
			html[idx++] = "'></td></tr></table>";
			html[idx++] = "</td><td>&nbsp;</td><td>&nbsp;</td></tr>";
		}

		html[idx++] = "</table>";
	} else {
		// source list
		html[idx++] = "<div id='";
		html[idx++] = this._sourceListViewDivId;
		html[idx++] = "'></div>";

		// transfer buttons
		html[idx++] = "<div align='center' id='";
		html[idx++] = this._buttonsDivId;
		html[idx++] = "'>";
		html[idx++] = "<table><tr>";
		if (this._allButtons) {
			html[idx++] = "<td id='";
			html[idx++] = this._addAllButtonDivId;
			html[idx++] = "'></td>";
		}
		for (var i = 0; i < this._buttonInfo.length; i++) {
			var id = this._buttonInfo[i].id;
			html[idx++] = "<td id='";
			html[idx++] = this._buttonDivId[id];
			html[idx++] = "'></td>";
		}
		// remove button
		html[idx++] = "<td id='";
		html[idx++] = this._removeButtonDivId;
		html[idx++] = "'></td>";
		if (this._allButtons) {
			html[idx++] = "<td id='";
			html[idx++] = this._removeAllButtonDivId;
			html[idx++] = "'></td>";
		}
		html[idx++] = "</tr></table></div>";

		// target list
		html[idx++] = "<div id='";
		html[idx++] = this._targetListViewDivId;
		html[idx++] = "'></div>";
	}

	this.getHtmlElement().innerHTML = html.join("");
};

/*
* Takes button info and sets up various bits of internal data for later use.
*/
DwtChooser.prototype._handleButtonInfo =
function(buttonInfo) {

	if (!buttonInfo) {
		this._defLabel = (this._selectStyle == DwtChooser.SINGLE_SELECT) ? AjxMsg.select : AjxMsg.add;
		buttonInfo = [ { label: this._defLabel } ];
	}
	this._buttonInfo = buttonInfo;

	// create IDs for button elements and their containers
	this._buttonDivId = {};
	this._buttonId = {};
	if (this._buttonInfo.length == 1) {
		if (!this._buttonInfo[0].id) {
			this._buttonInfo[0].id = Dwt.getNextId("DwtChooserButtonInfo_");
		}
		this._activeButtonId = this._buttonInfo[0].id;
	}
	for (var i = 0; i < this._buttonInfo.length; i++) {
		var id = this._buttonInfo[i].id;
		this._buttonDivId[id] = Dwt.getNextId("DwtChooserButtonDiv_");
		this._buttonId[id] = Dwt.getNextId("DwtChooserButton_");
	}
	this._hasMultiButtons = (this._buttonInfo.length > 1);
};

/**
 * Creates and places elements into the DOM.
 * 
 * @private
 */
DwtChooser.prototype._initialize =
function() {

	// create and add transfer buttons
	var buttonListener = new AjxListener(this, this._transferButtonListener);
	this._button = {};
	this._buttonIndex = {};
	this._data = {};
	for (var i = 0; i < this._buttonInfo.length; i++) {
		var id = this._buttonInfo[i].id;
		this._button[id] = this._setupButton(id, this._buttonId[id], this._buttonDivId[id], this._buttonInfo[i].label);
		this._button[id].addSelectionListener(buttonListener);
		this._buttonIndex[id] = i;
		this._data[id] = new AjxVector();
	}

	// create and add source list view
	this.sourceListView = this._createSourceListView();
	this._addListView(this.sourceListView, this._sourceListViewDivId);
	this.sourceListView.addSelectionListener(new AjxListener(this, this._sourceListener));

	// create and add target list view
	this.targetListView = this._createTargetListView();
	this._addListView(this.targetListView, this._targetListViewDivId);
	this.targetListView.addSelectionListener(new AjxListener(this, this._targetListener));

	// create and add the remove button
	this._removeButtonId = Dwt.getNextId("DwtChooserRemoveButton_");
	this._removeButton = this._setupButton(DwtChooser.REMOVE_BTN_ID, this._removeButtonId, this._removeButtonDivId, AjxMsg.remove);
	this._removeButton.addSelectionListener(new AjxListener(this, this._removeButtonListener));

	if (this._allButtons) {
		// create and add "Add All" and "Remove All" buttons
		this._addAllButtonId = Dwt.getNextId();
		this._addAllButton = this._setupButton(DwtChooser.ADD_ALL_BTN_ID, this._addAllButtonId, this._addAllButtonDivId, AjxMsg.addAll);
		this._addAllButton.addSelectionListener(new AjxListener(this, this._addAllButtonListener));
		this._removeAllButtonId = Dwt.getNextId();
		this._removeAllButton = this._setupButton(DwtChooser.REMOVE_ALL_BTN_ID, this._removeAllButtonId, this._removeAllButtonDivId, AjxMsg.removeAll);
		this._removeAllButton.addSelectionListener(new AjxListener(this, this._removeAllButtonListener));
		if (this._selectStyle == DwtChooser.SINGLE_SELECT) {
			this._addAllButton.setVisible(false);
			this._removeAllButton.setVisible(false);
		}
	}

	if (this._hasTextField) {
		var params = {parent: this, type: DwtInputField.STRING};
		this._textField = new DwtInputField(params);
		this._textField.reparentHtmlElement(this._textFieldTdId);
		this._textField._chooser = this;
		this._textField.setHandler(DwtEvent.ONKEYUP, DwtChooser._onKeyUp);
		Dwt.setSize(this._textField.getInputElement(), "100%", Dwt.DEFAULT);
	}

	if (this._selectStyle == DwtChooser.SINGLE_SELECT) {
		this.sourceListView.multiSelectEnabled = false;
		this.targetListView.multiSelectEnabled = false;
	}
};

/**
 * Returns a source list view object.
 * 
 * @private
 */
DwtChooser.prototype._createSourceListView =
function() {
	return new DwtChooserListView(this, DwtChooserListView.SOURCE, this._slvClassName);
};

/**
 * Returns a target list view object.
 * 
 * @private
 */
DwtChooser.prototype._createTargetListView =
function() {
	return new DwtChooserListView(this, DwtChooserListView.TARGET, this._tlvClassName);
};

/**
 * Adds a list view into the DOM and sets its size to fit in its container.
 *
 * @param listView		[DwtChooserListView]	the list view
 * @param listViewDivId	[string]				ID of container DIV
 * 
 * @private
 */
DwtChooser.prototype._addListView =
function(listView, listViewDivId) {
	var listDiv = document.getElementById(listViewDivId);
 	listDiv.appendChild(listView.getHtmlElement());
	listView.setUI(null, true); // renders headers and empty list
	listView._initialized = true;
};

/**
 * Sizes the list views based on the given available width and height.
 *
 * @param {number}	width	the width (in pixels)
 * @param {number}	height	the height (in pixels)
 */
DwtChooser.prototype.resize =
function(width, height) {
	if (!width || !height) return;
	if (width == Dwt.DEFAULT && height == Dwt.DEFAULT) return;

	var buttonsDiv = document.getElementById(this._buttonsDivId);
	var btnSz = Dwt.getSize(buttonsDiv);
	var w, sh, th;
	if (this._layoutStyle == DwtChooser.HORIZ_STYLE) {
		w = this._listSize ? this._listSize : (width == Dwt.DEFAULT) ? width : Math.floor(((width - btnSz.x) / 2) - 12);
		sh = th = height;
	} else {
		w = width;
		if (this._selectStyle == DwtChooser.SINGLE_SELECT) {
			sh = this._listSize ? this._listSize : (height == Dwt.DEFAULT) ? height : height - btnSz.y - this._singleHeight - 30;
			th = (height == Dwt.DEFAULT) ? height : height - btnSz.y - sh - 30;
		} else {
			sh = th = this._listSize ? this._listSize : (height == Dwt.DEFAULT) ? height : Math.floor(((height - btnSz.y) / 2) - 12);
		}
	}
	this.sourceListView.setSize((w == Dwt.DEFAULT) ? w : w+2, sh);
	this.targetListView.setSize((w == Dwt.DEFAULT) ? w : w+2, th);
};

/**
 * Creates a transfer or remove button.
 *
 * @param {string}	id					the button ID
 * @param {string}	buttonId			the ID of button element
 * @param {string}	buttonDivId		the ID of DIV that contains button
 * @param {string}	label				the button text
 * 
 * @private
 */
DwtChooser.prototype._setupButton =
function(id, buttonId, buttonDivId, label) {
	var button = new DwtButton({parent:this, id:buttonId});
	button.setText(label);
	button.id = buttonId;
	button._buttonId = id;

	var buttonDiv = document.getElementById(buttonDivId);
	buttonDiv.appendChild(button.getHtmlElement());

	return button;
};

// Listeners

/**
 * Single-click selects an item, double-click adds selected items to target list.
 *
 * @param {DwtEvent}	ev		the click event
 * 
 * @private
 */
DwtChooser.prototype._sourceListener =
function(ev) {
	if (ev.detail == DwtListView.ITEM_DBL_CLICKED) {
		// double-click performs transfer
		this.transfer(this.sourceListView.getSelection(), this._activeButtonId);
		this.sourceListView.deselectAll();
	} else if (this._activeButtonId == DwtChooser.REMOVE_BTN_ID) {
		// single-click activates appropriate transfer button if needed
		var id = this._lastActiveTransferButtonId ? this._lastActiveTransferButtonId : this._buttonInfo[0].id;
		this._setActiveButton(id);
	}
	this.targetListView.deselectAll();
	this._enableButtons();
};

/**
 * Single-click selects an item, double-click removes it from the target list.
 *
 * @param {DwtEvent}		ev		the click event
 * 
 * @private
 */
DwtChooser.prototype._targetListener =
function(ev) {
	if (ev.detail == DwtListView.ITEM_DBL_CLICKED) {
		this.remove(this.targetListView.getSelection());
	} else {
		this._setActiveButton(DwtChooser.REMOVE_BTN_ID);
		this.sourceListView.deselectAll();
		this._enableButtons();
	}
};

/**
 * Clicking a transfer button moves selected items to the target list.
 *
 * @param {DwtEvent}		ev		the click event
 * 
 * @private
 */
DwtChooser.prototype._transferButtonListener =
function(ev) {
	var button = DwtControl.getTargetControl(ev);
	var id = button._buttonId;
	var sel = this.sourceListView.getSelection();
	if (sel && sel.length) {
		this.transfer(sel, id);
		var list = this.sourceListView.getList();
		if (list && list.size()) {
			this._selectFirst(DwtChooserListView.SOURCE);
		} else {
			this._enableButtons();
		}
	} else {
		var email = this._getEmailFromText();
		if (email) {
			this.transfer([email], id);
		} else {
			this._setActiveButton(id);
		}
	}
};

/**
 * Clicking the remove button removes selected items from the target list.
 *
 * @param {DwtEvent}		ev		the click event
 * 
 * @private
 */
DwtChooser.prototype._removeButtonListener =
function(ev) {
	this.remove(this.targetListView.getSelection());
	var list = this.targetListView.getList();
	if (list && list.size()) {
		this._selectFirst(DwtChooserListView.TARGET);
	} else {
		this._enableButtons();
	}
};

/**
 * Populates the target list with all items.
 *
 * @param {DwtEvent}		ev		the click event
 * 
 * @private
 */
DwtChooser.prototype._addAllButtonListener =
function(ev) {
	this.transfer(this.sourceListView.getList().clone());
	this._selectFirst(DwtChooserListView.TARGET);
};

/**
 * Clears the target list.
 *
 * @param {DwtEvent}		ev		the click event
 * 
 * @private
 */
DwtChooser.prototype._removeAllButtonListener =
function(ev) {
	this.remove(this.targetListView.getList().clone());
	this._selectFirst(DwtChooserListView.SOURCE);
};



// Miscellaneous methods

/**
 * Enable/disable buttons as appropriate.
 *
 * @private
 */
DwtChooser.prototype._enableButtons =
function(sForce, tForce) {
	var sourceList = this.sourceListView.getList();
	var targetList = this.targetListView.getList();
	var sourceEnabled = (sForce || (this.sourceListView.getSelectionCount() > 0));
	for (var i = 0; i < this._buttonInfo.length; i++) {
		var id = this._buttonInfo[i].id;
		this._button[id].setEnabled(sourceEnabled);
	}
	var targetEnabled = (tForce || (this.targetListView.getSelectionCount() > 0));
	this._removeButton.setEnabled(targetEnabled);

	if (this._allButtons && (this._selectStyle == DwtChooser.MULTI_SELECT)) {
		var sourceSize = sourceList ? sourceList.size() : 0;
		var targetSize = targetList ? targetList.size() : 0;
		this._addAllButton.setEnabled(sourceSize > 0);
		this._removeAllButton.setEnabled(targetSize > 0);
	}
};

/**
 * Selects the first item in the given list view.
 *
 * @param {constant}	view	the source or target
 * 
 * @private
 */
DwtChooser.prototype._selectFirst =
function(view, index) {
	var listView = (view == DwtChooserListView.SOURCE) ? this.sourceListView : this.targetListView;
	var list = listView.getList();
	if (list && list.size() > 0) {
		listView.setSelection(list.get(0));
	}
};

/**
 * Makes a button "active" (the default for double-clicks). Done by
 * manipulating the style class. The active/non-active class is set as the
 * "_origClassName" so that activation/triggering still work. This only
 * applies if there are multiple transfer buttons.
 *
 * @param {string}	id		the ID of button to make active
 * 
 * @private
 */
DwtChooser.prototype._setActiveButton =
function(id) {
	if (!this._hasMultiButtons) {
		return;
	}
	if (id != this._activeButtonId) {
		var buttonId = (this._activeButtonId == DwtChooser.REMOVE_BTN_ID) ? this._removeButtonId : this._buttonId[this._activeButtonId];
		if (buttonId) {
			var oldButton = DwtControl.findControl(document.getElementById(buttonId));
			if (oldButton) {
				oldButton.setDisplayState(DwtControl.NORMAL);
			}
		}
		buttonId = (id == DwtChooser.REMOVE_BTN_ID) ? this._removeButtonId : this._buttonId[id];
		var button = DwtControl.findControl(document.getElementById(buttonId));
		if (button) {
			button.setDisplayState(DwtControl.DEFAULT);
		}
		this._activeButtonId = id;
		if (id != DwtChooser.REMOVE_BTN_ID) {
			this._lastActiveTransferButtonId = id;
		}
	}
};

/**
 * Returns true if the list contains the item. Default implementation is identity.
 *
 * @param {Object}	item	the item
 * @param {AjxVector}	list	the list to check against
 * 
 * @private
 */
DwtChooser.prototype._isDuplicate =
function(item, list) {
	return list.contains(item);
};

/**
 * Adds an item to the end of the source list.
 *
 * @param {Object}	item		the item to add
 * @param {boolean}	skipNotify	if <code>true</code>, don't notify listeners
 * 
 * @private
 */
DwtChooser.prototype._addToSource =
function(item, index, skipNotify) {
	if (!item) return;
	if (!item._chooserIndex) {
		var list = this.sourceListView.getList();
		item._chooserIndex = list ? list.size() + 1 : 1;
	}
	this.sourceListView.addItem(item, index, skipNotify);
};

/**
 * Adds an item to the target list. If there are multiple transfer buttons, it keeps
 * the items grouped depending on which button was used to move them.
 *
 * @param {Object}	item		the item to add
 * @param {string}	id			the ID of the transfer button that was used
 * @param {boolean}	skipNotify	if <code>true</code>, don't notify listeners
 * 
 * @private
 */
DwtChooser.prototype._addToTarget =
function(item, id, skipNotify) {
	if (!item) return;
	id = id ? id : this._activeButtonId;
	if (this._noDuplicates && this._data[id] && this._isDuplicate(item, this._data[id])) {
		return;
	}

	// item is being added to target list with multiple transfer buttons,
	// so we need to clone it on second and subsequent transfers
	var list = this.targetListView.getList();
	if (list && list.contains(item) && item.clone) {
		var newItem = item.clone();
		newItem.id = Dwt.getNextId();
		item = newItem;
	}

	var idx = null;
	if (this._hasMultiButtons) {
		// get a list of all the items in order
		list = [];
		for (var i = 0; i < this._buttonInfo.length; i++) {
			list = list.concat(this._data[this._buttonInfo[i].id].getArray());
		}
		// find the first item with a higher button index
		var buttonIdx = this._buttonIndex[id];
		for (idx = 0; idx < list.length; idx++) {
			var testButtonIdx = this._buttonIndex[list[idx]._buttonId];
			if (testButtonIdx > buttonIdx) {
				break;
			}
		}
	}

	item._buttonId = id;
	item.id = item.id || Dwt.getNextId();
	this._data[id].add(item);
	this.targetListView.addItem(item, idx, skipNotify);
};

/**
 * Removes an item from the source list.
 *
 * @param {Object}	item		the item to remove
 * @param {boolean}	skipNotify	if <code>true</code>, don't notify listeners
 * 
 * @private
 */
DwtChooser.prototype._removeFromSource =
function(item, skipNotify) {
	if (!item) return;
	var list = this.sourceListView.getList();
	if (!list) return;
	if (!list.contains(item)) return;

	this.sourceListView.removeItem(item, skipNotify);
};

/**
 * Removes an item from the target list.
 *
 * @param {Object}	item		the item to remove
 * @param {boolean}	skipNotify	if <code>true</code>, don't notify listeners
 * 
 * @private
 */
DwtChooser.prototype._removeFromTarget =
function(item, skipNotify) {
	if (!item) return;
	var list = this.targetListView.getList();
	if (!list) return;
	if (!list.contains(item)) return;

	this._data[item._buttonId].remove(item);
	this.targetListView.removeItem(item, skipNotify);
};

DwtChooser.prototype._getInsertionIndex =
function(view, item) {
	var list = view.getList();
	if (!list) return null;
	var a = list.getArray();
	for (var i = 0; i < a.length; i++) {
		if (item._chooserIndex && a[i]._chooserIndex && (a[i]._chooserIndex >= item._chooserIndex)) {
			return i;
		}
	}
	return null;
};

DwtChooser.prototype._getEmailFromText =
function() {
	if (this._hasTextField) {
		var text = this._textField.getValue();
		var email = AjxEmailAddress.parse(text);
		if (email) {
			email.id = Dwt.getNextId();
			return email;
		}
	}
};

DwtChooser._onKeyUp =
function(ev) {
	var el = DwtUiEvent.getTarget(ev);
	var obj = DwtControl.findControl(el);	// DwtInputField
	var chooser = obj._chooser;
	var key = DwtKeyEvent.getCharCode(ev);
	if (key == 3 || key == 13) {
		var email = chooser._getEmailFromText();
		if (email) {
			chooser.transfer([email], chooser._activeButtonId);
			el.value = "";
		}
	}
	chooser._enableButtons(el.value.length);
};

/**
 * Creates a chooser list view.
 * @constructor
 * @class
 * This base class represents a list view which contains items that can be transferred from it
 * (source) or to it (target). Subclasses should implement  _getHeaderList(),
 * _sortColumn(), and _createItemHtml().
 *
 * @param {hash}	params		a hash of parameters
 * @param {DwtComposite}      params.parent		the containing widget
 * @param {constant}      params.type			the source or target
 * @param {string}      params.className		the CSS class
 * @param {constant}      params.view			the context for use in creating IDs
 * 
 * @extends		DwtListView
 */
DwtChooserListView = function(params) {
	
	if (arguments.length == 0) return;
	params = Dwt.getParams(arguments, DwtChooserListView.PARAMS);
	params.className = params.className || "DwtChooserListView";
	params.headerList = this._getHeaderList(parent);
	DwtListView.call(this, params);

	this.type = params.type;
	this._chooserParent = params.parent.parent;
};

DwtChooserListView.PARAMS = ["parent", "type", "className", "view"];

/**
 * Defines the "source" list view type.
 */
DwtChooserListView.SOURCE = 1;
/**
 * Defines the "target" list view type.
 */
DwtChooserListView.TARGET = 2;

DwtChooserListView.prototype = new DwtListView;
DwtChooserListView.prototype.constructor = DwtChooserListView;

DwtChooserListView.prototype._getHeaderList = function() {};

DwtChooserListView.prototype.toString = 
function() {
	return "DwtChooserListView";
};

/*
* Override to handle empty results set. Always omit the "No Results" message if
* this is a target list view, or if we've been told to ignore it in the source view.
*/
DwtChooserListView.prototype.setUI =
function(defaultColumnSort, noResultsOk) {
	noResultsOk = noResultsOk ? noResultsOk : ((this.type == DwtChooserListView.TARGET) ||
												this.parent._sourceEmptyOk);
	DwtListView.prototype.setUI.call(this, defaultColumnSort, noResultsOk);
};

/**
 * DwtListView override to ignore right-clicks in list view.
 *
 * @param {Element}	clickedEl		the element that was clicked
 * @param {DwtEvent}	ev				the click event
 * 
 * @private
 */
DwtChooserListView.prototype._itemClicked = 
function(clickedEl, ev) {
	// Ignore right-clicks, we don't support action menus
	if (!ev.shiftKey && !ev.ctrlKey && ev.button == DwtMouseEvent.RIGHT) {
		return;
	} else {
		DwtListView.prototype._itemClicked.call(this, clickedEl, ev);
	}
};

/**
 * Called when a column header has been clicked.
 *
 * @param {string}	columnItem		the ID for column that was clicked
 * @param {boolean}	ascending		if <code>true</code>, sort in ascending order
 * 
 * @private
 */
DwtChooserListView.prototype._sortColumn = 
function(columnItem, ascending) {
	this._chooserParent.search(columnItem, ascending);
};

DwtChooserListView.prototype._getHeaderSashLocation =
function() {

	var el = this.getHtmlElement();
	if (Dwt.getPosition(el) == Dwt.ABSOLUTE_STYLE) {
		return DwtListView.prototype._getHeaderSashLocation.call(this);
	}

	var thisLoc = Dwt.toWindow(el, 0, 0);
	var contLoc = Dwt.toWindow(this._chooserParent.getHtmlElement(), 0, 0);
	if (!this._tmpPoint) {
		this._tmpPoint = new DwtPoint();
	}
	this._tmpPoint.x = thisLoc.x - contLoc.x;
	this._tmpPoint.y = thisLoc.y - contLoc.y;

	return this._tmpPoint;
};
}

if (AjxPackage.define("zimbraMail.abook.model.ZmAddrBook")) {
/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite Web Client
 * Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013 Zimbra Software, LLC.
 * 
 * The contents of this file are subject to the Zimbra Public License
 * Version 1.4 ("License"); you may not use this file except in
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.zimbra.com/license.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * ***** END LICENSE BLOCK *****
 */

/**
 * @overview
 * This file contains the address book class.
 */

/**
 * Creates an address book.
 * @constructor
 * @class
 * This class represents an address book.
 * 
 * @author Parag Shah
 *
 * @param	{Hash}	params		a hash of parameters
 * @param {int}	params.id			a numeric ID
 * @param {String}	params.name		[string]	the name
 * @param {ZmOrganizer}	params.parent		the parent organizer
 * @param {ZmTree}	params.tree		a tree model that contains this organizer
 * @param {int}	params.color		the color of this address book
 * @param {String}	params.owner	the owner of the address book (if shared)
 * @param {String}	params.zid 		the the share ID of a shared address book
 * @param {String}	params.rid		the the remote folder id of a shared address book
 * @param {String}	params.restUrl	the REST URL of this organizer
 * 
 * @extends		ZmFolder
 */
ZmAddrBook = function(params) {
	params.type = ZmOrganizer.ADDRBOOK;
	ZmFolder.call(this, params);
};

ZmAddrBook.prototype = new ZmFolder;
ZmAddrBook.prototype.constructor = ZmAddrBook;


// Consts

ZmAddrBook.ID_ADDRESSBOOK = ZmOrganizer.ID_ADDRBOOK; 							// XXX: may not be necessary

// Static methods

/**
 * Sorts the address books.
 * 
 * @param	{ZmAddrBook}	addrBookA		the address book
 * @param	{ZmAddrBook}	addrBookB		the address book
 * @return	{int}	0 if the address books are the same; 1 if "a" is before "b"; -1 if "b" is before "a"
 */
ZmAddrBook.sortCompare = 
function(addrBookA, addrBookB) {
	var check = ZmOrganizer.checkSortArgs(addrBookA, addrBookB);
	if (check != null) return check;

	// links appear after personal address books
	if (addrBookA.link != addrBookB.link) {
		return addrBookA.link ? 1 : -1;
	}

	// trash folder should always go last w/in personal addrbooks
	if (addrBookA.nId == ZmFolder.ID_TRASH) { return 1; }
	if (addrBookB.nId == ZmFolder.ID_TRASH) { return -1; }

	// sort by calendar name
	var addrBookAName = addrBookA.name.toLowerCase();
	var addrBookBName = addrBookB.name.toLowerCase();
	if (addrBookAName < addrBookBName) { return -1; }
	if (addrBookAName > addrBookBName) { return 1; }
	return 0;
};

// Public methods

/**
 * Returns a string representation of the object.
 * 
 * @return		{String}		a string representation of the object
 */
ZmAddrBook.prototype.toString = 
function() {
	return "ZmAddrBook";
};

ZmAddrBook.prototype.getIcon = 
function() {
	if (this.nId == ZmFolder.ID_ROOT)			{ return null; }
	if (this.nId == ZmFolder.ID_TRASH)			{ return "Trash"; }
	if (this.link || this.isRemote())			{ return "SharedContactsFolder"; }
	if (this.nId == ZmFolder.ID_AUTO_ADDED)		{ return "EmailedContacts"; }
	return "ContactsFolder";
};

/**
 * Checks if the address book supports public access.
 * 
 * @return	{Boolean}		always returns <code>true</code>
 */
ZmAddrBook.prototype.supportsPublicAccess =
function() {
	// AddrBook's can be accessed outside of ZCS (i.e. REST)
	return true;
};

/**
 * @private
 */
ZmAddrBook.prototype.mayContain =
function(what) {
	if (!what) return true;

	if (this.id == ZmOrganizer.ID_DLS) {
		return false;
	}

	if (what instanceof ZmAddrBook) {
		if (!appCtxt.multiAccounts) {
			return true;
		}
		return this.mayContainFolderFromAccount(what.getAccount()); // cannot move folders across accounts, unless the target is local
	}

	if (this.nId == ZmOrganizer.ID_ROOT) {
		// cannot drag anything onto root folder
		return false;
	}
	if (this.link) {
		// cannot drop anything onto a read-only addrbook
		if (this.isReadOnly()) {
			return false;
		}
	}

	// An item or an array of items is being moved
	var items = (what instanceof Array) ? what : [what];
	var item = items[0];

	if (item.type != ZmItem.CONTACT && item.type != ZmItem.GROUP) {
		// only contacts are valid for addr books.
		return false;
	}

	// can't move items to folder they're already in; we're okay if
	// we have one item from another folder
	if (item.folderId) {
		var invalid = true;
		for (var i = 0; i < items.length; i++) {
			var tree = appCtxt.getById(items[i].folderId);
			if (tree != this) {
				invalid = false;
				break;
			}
		}

		return !invalid;
	}

	return true;
};
}
if (AjxPackage.define("zimbraMail.abook.model.ZmContact")) {
/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite Web Client
 * Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013 Zimbra Software, LLC.
 * 
 * The contents of this file are subject to the Zimbra Public License
 * Version 1.4 ("License"); you may not use this file except in
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.zimbra.com/license.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * ***** END LICENSE BLOCK *****
 */

/**
 * @overview
 * This file contains the contact class.
 */

if (!window.ZmContact) {
/**
 * Creates an empty contact.
 * @class
 * This class represents a contact (typically a person) with all its associated versions
 * of email address, home and work addresses, phone numbers, etc. Contacts can be filed/sorted
 * in different ways, with the default being Last, First. A contact is an item, so
 * it has tagging and flagging support, and belongs to a list.
 * <p>
 * Most of a contact's data is kept in attributes. These include name, phone, etc. Meta-data and
 * data common to items are not kept in attributes. These include flags, tags, folder, and
 * modified/created dates. Since the attribute data for contacts is loaded only once, a contact
 * gets its attribute values from that canonical list.
 * </p>
 *
 * @param {int}	id		the unique ID
 * @param {ZmContactList}	list		the list that contains this contact
 * @param {constant}	type		the item type
 * 
 * @extends		ZmItem
 */
ZmContact = function(id, list, type, folderId) {
	if (arguments.length == 0) { return; }

	type = type || ZmItem.CONTACT;
	ZmItem.call(this, type, id, list);

	this.folderId = folderId;

	this.attr = {};
	this.isGal = (this.list && this.list.isGal);

	this.participants = new AjxVector(); // XXX: need to populate this guy (see ZmConv)
};

ZmContact.prototype = new ZmItem;
ZmContact.prototype.constructor = ZmContact;

// fields
ZmContact.F_anniversary				= "anniversary";
ZmContact.F_assistantPhone			= "assistantPhone";
ZmContact.F_attachment				= "attachment";
ZmContact.F_birthday				= "birthday";
ZmContact.F_callbackPhone			= "callbackPhone";
ZmContact.F_carPhone				= "carPhone";
ZmContact.F_company					= "company";
ZmContact.F_companyPhone			= "companyPhone";
ZmContact.F_custom					= "custom";
ZmContact.F_description				= "description";
ZmContact.F_department				= "department";
ZmContact.F_dlist					= "dlist";				// Group fields
ZmContact.F_email					= "email";
ZmContact.F_email2					= "email2";
ZmContact.F_email3					= "email3";
ZmContact.F_email4					= "email4";
ZmContact.F_email5					= "email5";
ZmContact.F_email6					= "email6";
ZmContact.F_email7					= "email7";
ZmContact.F_email8					= "email8";
ZmContact.F_email9					= "email9";
ZmContact.F_email10					= "email10";
ZmContact.F_email11					= "email11";
ZmContact.F_email12					= "email12";
ZmContact.F_email13					= "email13";
ZmContact.F_email14					= "email14";
ZmContact.F_email15					= "email15";
ZmContact.F_email16					= "email16";
ZmContact.F_fileAs					= "fileAs";
ZmContact.F_firstName				= "firstName";
ZmContact.F_folderId				= "folderId";
ZmContact.F_homeCity				= "homeCity";
ZmContact.F_homeCountry				= "homeCountry";
ZmContact.F_homeFax					= "homeFax";
ZmContact.F_homePhone				= "homePhone";
ZmContact.F_homePhone2				= "homePhone2";
ZmContact.F_homePostalCode			= "homePostalCode";
ZmContact.F_homeState				= "homeState";
ZmContact.F_homeStreet				= "homeStreet";
ZmContact.F_homeURL					= "homeURL";
ZmContact.F_image					= "image";				// contact photo
ZmContact.F_imAddress 				= "imAddress";			// IM addresses
ZmContact.F_imAddress1 				= "imAddress1";			// IM addresses
ZmContact.F_imAddress2 				= "imAddress2";
ZmContact.F_imAddress3				= "imAddress3";
ZmContact.F_jobTitle				= "jobTitle";
ZmContact.F_lastName				= "lastName";
ZmContact.F_maidenName				= "maidenName";
ZmContact.F_middleName				= "middleName";
ZmContact.F_mobilePhone				= "mobilePhone";
ZmContact.F_namePrefix				= "namePrefix";
ZmContact.F_nameSuffix				= "nameSuffix";
ZmContact.F_nickname				= "nickname";
ZmContact.F_notes					= "notes";
ZmContact.F_otherCity				= "otherCity";
ZmContact.F_otherCountry			= "otherCountry";
ZmContact.F_otherFax				= "otherFax";
ZmContact.F_otherPhone				= "otherPhone";
ZmContact.F_otherPostalCode			= "otherPostalCode";
ZmContact.F_otherState				= "otherState";
ZmContact.F_otherStreet				= "otherStreet";
ZmContact.F_otherURL				= "otherURL";
ZmContact.F_pager					= "pager";
ZmContact.F_phoneticFirstName       = "phoneticFirstName";
ZmContact.F_phoneticLastName        = "phoneticLastName";
ZmContact.F_phoneticCompany         = "phoneticCompany";
ZmContact.F_type					= "type";
ZmContact.F_workAltPhone			= "workAltPhone";
ZmContact.F_workCity				= "workCity";
ZmContact.F_workCountry				= "workCountry";
ZmContact.F_workEmail1				= "workEmail1";
ZmContact.F_workEmail2				= "workEmail2";
ZmContact.F_workEmail3				= "workEmail3";
ZmContact.F_workFax					= "workFax";
ZmContact.F_workMobile				= "workMobile";
ZmContact.F_workPhone				= "workPhone";
ZmContact.F_workPhone2				= "workPhone2";
ZmContact.F_workPostalCode			= "workPostalCode";
ZmContact.F_workState				= "workState";
ZmContact.F_workStreet				= "workStreet";
ZmContact.F_workURL					= "workURL";
ZmContact.X_fileAs					= "fileAs";				// extra fields
ZmContact.X_firstLast				= "firstLast";
ZmContact.X_fullName				= "fullName";
ZmContact.X_vcardXProps             = "vcardXProps";
ZmContact.X_outlookUserField        = "outlookUserField";
ZmContact.MC_cardOwner				= "cardOwner";			// My card fields
ZmContact.MC_workCardMessage		= "workCardMessage";
ZmContact.MC_homeCardMessage		= "homeCardMessage";
ZmContact.MC_homePhotoURL			= "homePhotoURL";
ZmContact.MC_workPhotoURL			= "workPhotoURL";
ZmContact.GAL_MODIFY_TIMESTAMP		= "modifyTimeStamp";	// GAL fields
ZmContact.GAL_CREATE_TIMESTAMP		= "createTimeStamp";
ZmContact.GAL_ZIMBRA_ID				= "zimbraId";
ZmContact.GAL_OBJECT_CLASS			= "objectClass";
ZmContact.GAL_MAIL_FORWARD_ADDRESS	= "zimbraMailForwardingAddress";
ZmContact.GAL_CAL_RES_TYPE			= "zimbraCalResType";
ZmContact.GAL_CAL_RES_LOC_NAME		= "zimbraCalResLocationDisplayName";

// file as
var i = 1;
ZmContact.FA_LAST_C_FIRST			= i++;
ZmContact.FA_FIRST_LAST 			= i++;
ZmContact.FA_COMPANY 				= i++;
ZmContact.FA_LAST_C_FIRST_COMPANY	= i++;
ZmContact.FA_FIRST_LAST_COMPANY		= i++;
ZmContact.FA_COMPANY_LAST_C_FIRST	= i++;
ZmContact.FA_COMPANY_FIRST_LAST		= i++;
ZmContact.FA_CUSTOM					= i++;

// Field information

ZmContact.ADDRESS_FIELDS = [
    // NOTE: sync with field order in ZmEditContactView's templates
	ZmContact.F_homeCity,
	ZmContact.F_homeCountry,
	ZmContact.F_homePostalCode,
	ZmContact.F_homeState,
	ZmContact.F_homeStreet,
	ZmContact.F_workCity,
	ZmContact.F_workCountry,
	ZmContact.F_workPostalCode,
	ZmContact.F_workState,
	ZmContact.F_workStreet,
    ZmContact.F_otherCity,
    ZmContact.F_otherCountry,
    ZmContact.F_otherPostalCode,
    ZmContact.F_otherState,
    ZmContact.F_otherStreet
];
ZmContact.EMAIL_FIELDS = [
	ZmContact.F_email,
	ZmContact.F_workEmail1,
	ZmContact.F_workEmail2,
	ZmContact.F_workEmail3
];
ZmContact.IM_FIELDS = [
	ZmContact.F_imAddress
];
ZmContact.OTHER_FIELDS = [
    // NOTE: sync with field order in ZmEditContactView's templates
	ZmContact.F_birthday,
    ZmContact.F_anniversary,
	ZmContact.F_custom
];
ZmContact.PHONE_FIELDS = [
    // NOTE: sync with field order in ZmEditContactView's templates
    ZmContact.F_mobilePhone,
    ZmContact.F_workPhone,
    ZmContact.F_workFax,
    ZmContact.F_companyPhone,
    ZmContact.F_homePhone,
    ZmContact.F_homeFax,
    ZmContact.F_pager,
    ZmContact.F_callbackPhone,
	ZmContact.F_assistantPhone,
	ZmContact.F_carPhone,
	ZmContact.F_otherPhone,
    ZmContact.F_otherFax,
	ZmContact.F_workAltPhone,
	ZmContact.F_workMobile
];
ZmContact.PRIMARY_FIELDS = [
    // NOTE: sync with field order in ZmEditContactView's templates
    ZmContact.F_image,
    ZmContact.F_namePrefix,
    ZmContact.F_firstName,
    ZmContact.F_phoneticFirstName,
    ZmContact.F_middleName,
	ZmContact.F_maidenName,
    ZmContact.F_lastName,
    ZmContact.F_phoneticLastName,
    ZmContact.F_nameSuffix,
    ZmContact.F_nickname,
    ZmContact.F_jobTitle,
    ZmContact.F_department,
	ZmContact.F_company,
    ZmContact.F_phoneticCompany,
	ZmContact.F_fileAs,
	ZmContact.F_folderId,
	ZmContact.F_notes
];
ZmContact.URL_FIELDS = [
    // NOTE: sync with field order in ZmEditContactView's templates
	ZmContact.F_homeURL,
	ZmContact.F_workURL,
	ZmContact.F_otherURL
];
ZmContact.GAL_FIELDS = [
	ZmContact.GAL_MODIFY_TIMESTAMP,
	ZmContact.GAL_CREATE_TIMESTAMP,
	ZmContact.GAL_ZIMBRA_ID,
	ZmContact.GAL_OBJECT_CLASS,
	ZmContact.GAL_MAIL_FORWARD_ADDRESS,
	ZmContact.GAL_CAL_RES_TYPE,
	ZmContact.GAL_CAL_RES_LOC_NAME,
	ZmContact.F_type
];
ZmContact.MYCARD_FIELDS = [
	ZmContact.MC_cardOwner,
	ZmContact.MC_homeCardMessage,
	ZmContact.MC_homePhotoURL,
	ZmContact.MC_workCardMessage,
	ZmContact.MC_workPhotoURL
];
ZmContact.X_FIELDS = [
	ZmContact.X_firstLast,
	ZmContact.X_fullName,
    ZmContact.X_vcardXProps
];


ZmContact.IGNORE_NORMALIZATION = [];

ZmContact.updateFieldConstants = function() {

ZmContact.DISPLAY_FIELDS = [].concat(
	ZmContact.ADDRESS_FIELDS,
	ZmContact.EMAIL_FIELDS,
	ZmContact.IM_FIELDS,
	ZmContact.OTHER_FIELDS,
	ZmContact.PHONE_FIELDS,
	ZmContact.PRIMARY_FIELDS,
	ZmContact.URL_FIELDS
);

ZmContact.IGNORE_FIELDS = [].concat(
	ZmContact.GAL_FIELDS,
	ZmContact.MYCARD_FIELDS,
	ZmContact.X_FIELDS
);

ZmContact.ALL_FIELDS = [].concat(
	ZmContact.DISPLAY_FIELDS, ZmContact.IGNORE_FIELDS
);

ZmContact.IS_DATE = {};
ZmContact.IS_DATE[ZmContact.F_birthday] = true;
ZmContact.IS_DATE[ZmContact.F_anniversary] = true;

ZmContact.IS_IGNORE = {};
for (var i = 0; i < ZmContact.IGNORE_FIELDS.length; i++) {
	ZmContact.IS_IGNORE[ZmContact.IGNORE_FIELDS[i]] = true;
}

// number of distribution list members to fetch at a time
ZmContact.DL_PAGE_SIZE = 100;

}; // updateFieldConstants()
ZmContact.updateFieldConstants();

/**
 * This structure can be queried to determine if the first
 * entry in a multi-value entry is suffixed with "1". Most
 * attributes add a numerical suffix to all but the first
 * entry.
 * <p>
 * <strong>Note:</strong>
 * In most cases, {@link ZmContact#getAttributeName} is a better choice.
 */
ZmContact.IS_ADDONE = {};
ZmContact.IS_ADDONE[ZmContact.F_custom] = true;
ZmContact.IS_ADDONE[ZmContact.F_imAddress] = true;
ZmContact.IS_ADDONE[ZmContact.X_outlookUserField] = true;

/**
 * Gets an indexed attribute name taking into account if the field
 * with index 1 should append the "1" or not. Code should call this
 * function in lieu of accessing {@link ZmContact.IS_ADDONE} directly.
 */
ZmContact.getAttributeName = function(name, index) {
	index = index || 1;
	return index > 1 || ZmContact.IS_ADDONE[name] ? name+index : name;
};

/**
 * Returns a string representation of the object.
 * 
 * @return		{String}		a string representation of the object
 */
ZmContact.prototype.toString =
function() {
	return "ZmContact";
};

// Class methods

/**
 * Creates a contact from an XML node.
 *
 * @param {Object}	node		a "cn" XML node
 * @param {Hash}	args		args to pass to the constructor
 * @return	{ZmContact}	the contact
 */
ZmContact.createFromDom =
function(node, args) {
	// check global cache for this item first
	var contact = appCtxt.cacheGet(node.id);

	// make sure the revision hasnt changed, otherwise contact is out of date
	if (contact == null || (contact && contact.rev != node.rev)) {
		contact = new ZmContact(node.id, args.list);
		contact._loadFromDom(node);
	} else {
		AjxUtil.hashUpdate(contact.attr, node._attrs);	// merge new attrs just in case we don't have them
		contact.list = args.list || new ZmContactList(null);
	}

	return contact;
};

/**
 * Compares two contacts based on how they are filed. Intended for use by
 * sort methods.
 *
 * @param {ZmContact}		a		a contact
 * @param {ZmContact}		b		a contact
 * @return	{int}	0 if the contacts are the same; 1 if "a" is before "b"; -1 if "b" is before "a"
 */
ZmContact.compareByFileAs =
function(a, b) {
	var aFileAs = (a instanceof ZmContact) ? a.getFileAs(true) : ZmContact.computeFileAs(a._attrs).toLowerCase();
	var bFileAs = (b instanceof ZmContact) ? b.getFileAs(true) : ZmContact.computeFileAs(b._attrs).toLowerCase();

	if (!bFileAs || (aFileAs > bFileAs)) return 1;
	if (aFileAs < bFileAs) return -1;
	return 0;
};

/**
 * Figures out the filing string for the contact according to the chosen method.
 *
 * @param {ZmContact|Hash}	contact		a contact or a hash of contact attributes
 */
ZmContact.computeFileAs =
function(contact) {
	var attr = (contact instanceof ZmContact) ? contact.getAttrs() : contact;
	if (!attr) return;

	var val = parseInt(attr.fileAs);
	var fa;
	var idx = 0;

	switch (val) {
		case ZmContact.FA_LAST_C_FIRST: 										// Last, First
		default: {
			// if GAL contact, use full name instead (bug fix #4850,4009)
			if (contact && contact.isGal) {
				if (attr.fullName) { // bug fix #27428 - if fullName is Array, return first
					return (attr.fullName instanceof Array) ? attr.fullName[0] : attr.fullName;
				}
				return ((attr.email instanceof Array) ? attr.email[0] : attr.email);
			}
			fa = ZmContact.fileAsLastFirst(attr.firstName, attr.lastName, attr.fullName, attr.nickname);
		}
		break;

		case ZmContact.FA_FIRST_LAST: { 										// First Last
			fa = ZmContact.fileAsFirstLast(attr.firstName, attr.lastName, attr.fullName, attr.nickname);
		}
		break;

		case ZmContact.FA_COMPANY: {											// Company
			if (attr.company) fa = attr.company;
		}
		break;

		case ZmContact.FA_LAST_C_FIRST_COMPANY: {								// Last, First (Company)
			var name = ZmContact.fileAsLastFirst(attr.firstName, attr.lastName, attr.fullName, attr.nickname);
			fa = ZmContact.fileAsNameCompany(name, attr.company);
		}
		break;

		case ZmContact.FA_FIRST_LAST_COMPANY: {									// First Last (Company)
			var name = ZmContact.fileAsFirstLast(attr.firstName, attr.lastName, attr.fullName, attr.nickname);
			fa = ZmContact.fileAsNameCompany(name, attr.company);
		}
		break;

		case ZmContact.FA_COMPANY_LAST_C_FIRST: {								// Company (Last, First)
			var name = ZmContact.fileAsLastFirst(attr.firstName, attr.lastName);
			fa = ZmContact.fileAsCompanyName(name, attr.company);
		}
		break;

		case ZmContact.FA_COMPANY_FIRST_LAST: {									// Company (First Last)
			var name = ZmContact.fileAsFirstLast(attr.firstName, attr.lastName);
			fa = ZmContact.fileAsCompanyName(name, attr.company);
		}
		break;

		case ZmContact.FA_CUSTOM: {												// custom looks like this: "8:foobar"
			return attr.fileAs.substring(2);
		}
		break;
	}
	return fa || attr.fullName || "";
};

/**
 * Name printing helper "First Last".
 * 
 * @param	{String}	first		the first name
 * @param	{String}	last		the last name
 * @param	{String}	fullname		the fullname
 * @param	{String}	nickname		the nickname
 * @return	{String}	the name format
 */
ZmContact.fileAsFirstLast =
function(first, last, fullname, nickname) {
	if (first && last)
		return AjxMessageFormat.format(ZmMsg.fileAsFirstLast, [first, last]);
	return first || last || fullname || nickname || "";
};

/**
 * Name printing helper "Last, First".
 * 
 * @param	{String}	first		the first name
 * @param	{String}	last		the last name
 * @param	{String}	fullname		the fullname
 * @param	{String}	nickname		the nickname
 * @return	{String}	the name format
 */
ZmContact.fileAsLastFirst =
function(first, last, fullname, nickname) {
	if (first && last)
		return AjxMessageFormat.format(ZmMsg.fileAsLastFirst, [first, last]);
	return first || last || fullname || nickname || "";
};

/**
 * Name printing helper "Name (Company)".
 * 
 * @param	{String}	name		the contact name
 * @param	{String}	company		the company
 * @return	{String}	the name format
 */
ZmContact.fileAsNameCompany =
function(name, company) {
	if (name && company)
		return AjxMessageFormat.format(ZmMsg.fileAsNameCompany, [name, company]);
	if (company)
		return AjxMessageFormat.format(ZmMsg.fileAsCompanyAsSecondaryOnly, [company]);
	return name;
};

/**
 * Name printing helper "Company (Name)".
 * 
 * @param	{String}	name		the contact name
 * @param	{String}	company		the company
 * @return	{String}	the name format
 */
ZmContact.fileAsCompanyName =
function(name, company) {
	if (company && name)
		return AjxMessageFormat.format(ZmMsg.fileAsCompanyName, [name, company]);
	if (name)
		return AjxMessageFormat.format(ZmMsg.fileAsNameAsSecondaryOnly, [name]);
	return company;
};

/**
 * Computes the custom file as string by prepending "8:" to the given custom fileAs string.
 * 
 * @param {Hash}	customFileAs	a set of contact attributes
 * @return	{String}	the name format
 */
ZmContact.computeCustomFileAs =
function(customFileAs) {
	return [ZmContact.FA_CUSTOM, ":", customFileAs].join("");
};

/*
 * 
 * These next few static methods handle a contact that is either an anonymous
 * object or an actual ZmContact. The former is used to optimize loading. The
 * anonymous object is upgraded to a ZmContact when needed.
 *  
 */

/**
 * Gets an attribute.
 * 
 * @param	{ZmContact}	contact		the contact
 * @param	{String}	attr		the attribute
 * @return	{Object}	the attribute value or <code>null</code> for none
 */
ZmContact.getAttr =
function(contact, attr) {
	return (contact instanceof ZmContact)
		? contact.getAttr(attr)
		: (contact && contact._attrs) ? contact._attrs[attr] : null;
};

/**
 * Normalizes the numbering of the given attribute names and
 * returns a new object with the re-numbered attributes. For
 * example, if the attributes contains a "foo2" but no "foo",
 * then the "foo2" attribute will be renamed to "foo" in the
 * returned object.
 *
 * @param {Hash}	attrs  a hash of attributes to normalize.
 * @param {String}	[prefix] if specified, only the the attributes that match the given prefix will be returned
 * @param {Array}	[ignore] if specified, the attributes that are present in the array will not be normalized
 * @return	{Hash}	a hash of normalized attributes
 */
ZmContact.getNormalizedAttrs = function(attrs, prefix, ignore) {
	var nattrs = {};
	if (attrs) {
		// normalize attribute numbering
		var names = AjxUtil.keys(attrs);
		names.sort(ZmContact.__BY_ATTRIBUTE);
		var a = {};
		for (var i = 0; i < names.length; i++) {
			var name = names[i];
			// get current count
			var nprefix = name.replace(/\d+$/,"");
			if (prefix && prefix != nprefix) continue;
			if (AjxUtil.isArray(ignore) && AjxUtil.indexOf(ignore, nprefix)!=-1) {
				nattrs[name] = attrs[name];
			} else {
				if (!a[nprefix]) a[nprefix] = 0;
				// normalize, if needed
				var nname = ZmContact.getAttributeName(nprefix, ++a[nprefix]);
				nattrs[nname] = attrs[name];
			}
		}
	}
	return nattrs;
};

ZmContact.__RE_ATTRIBUTE = /^(.*?)(\d+)$/;
ZmContact.__BY_ATTRIBUTE = function(a, b) {
	var aa = a.match(ZmContact.__RE_ATTRIBUTE) || [a,a,1];
	var bb = b.match(ZmContact.__RE_ATTRIBUTE) || [b,b,1];
	return aa[1] == bb[1] ? Number(aa[2]) - Number(bb[2]) : aa[1].localeCompare(bb[1]);
};

/**
 * Sets the attribute.
 * 
 * @param	{ZmContact}	contact		the contact
 * @param	{String}	attr		the attribute
 * @param	{Object}	value		the attribute value
 */
ZmContact.setAttr =
function(contact, attr, value) {
	if (contact instanceof ZmContact)
		contact.setAttr(attr, value);
	else
		contact._attrs[attr] = value;
};

/**
 * Checks if the contact is in the trash.
 * 
 * @param	{ZmContact}	contact		the contact
 * @return	{Boolean}	<code>true</code> if in trash
 */
ZmContact.isInTrash =
function(contact) {
	var folderId = (contact instanceof ZmContact) ? contact.folderId : contact.l;
	var folder = appCtxt.getById(folderId);
	return (folder && folder.isInTrash());
};

/**
 * @private
 */
ZmContact.prototype.load =
function(callback, errorCallback, batchCmd) {
	var jsonObj = {GetContactsRequest:{_jsns:"urn:zimbraMail"}};
	var request = jsonObj.GetContactsRequest;
	request.cn = [{id:this.id}];

	var respCallback = new AjxCallback(this, this._handleLoadResponse, [callback]);

	if (batchCmd) {
		var jsonObj = {GetContactsRequest:{_jsns:"urn:zimbraMail"}};
		jsonObj.GetContactsRequest.cn = {id:this.id};
		batchCmd.addRequestParams(jsonObj, respCallback, errorCallback);
	} else {
		appCtxt.getAppController().sendRequest({jsonObj:jsonObj,
												asyncMode:true,
												callback:respCallback,
												errorCallback:errorCallback});
	}
};

/**
 * @private
 */
ZmContact.prototype._handleLoadResponse =
function(callback, result) {
	var resp = result.getResponse().GetContactsResponse;

	// for now, we just assume only one contact was requested at a time
	this.attr = resp.cn[0]._attrs;
	this.isLoaded = true;

	if (callback)
		callback.run(resp.cn[0], this);
};

/**
 * @private
 */
ZmContact.prototype.clear =
function() {
	// bug fix #41666 - override base class method and do nothing
};

/**
 * Checks if the contact attributes are empty.
 * 
 * @return	{Boolean}	<code>true</code> if empty
 */
ZmContact.prototype.isEmpty =
function() {
	for (var i in this.attr) {
		return false;
	}
	return true;
};

/**
 * Checks if the contact is shared.
 * 
 * @return	{Boolean}	<code>true</code> if shared
 */
ZmContact.prototype.isShared =
function() {
	return this.addrbook && this.addrbook.link;
};

/**
 * Checks if the contact is read-only.
 * 
 * @return	{Boolean}	<code>true</code> if read-only
 */
ZmContact.prototype.isReadOnly =
function() {
	if (this.isGal) { return true; }

	return this.isShared()
		? this.addrbook && this.addrbook.isReadOnly()
		: false;
};

/**
 * Checks if the contact is a group.
 * 
 * @return	{Boolean}	<code>true</code> if a group
 */
ZmContact.prototype.isGroup =
function() {
	return Boolean(this.getAttr(ZmContact.F_dlist) || this.type == ZmItem.GROUP);
};

// parses "dlist" attr into AjxEmailAddress objects stored in 3 vectors (all, good, and bad)
/**
 * Gets the group members.
 * 
 * @return	{AjxVector}		the group members or <code>null</code> if not group
 */
ZmContact.prototype.getGroupMembers =
function() {
	return this.isGroup()
		? AjxEmailAddress.parseEmailString(this.getAttr(ZmContact.F_dlist))
		: null;
};

/**
 * Gets the icon.
 * 
 * @return	{String}	the icon
 */
ZmContact.prototype.getIcon =
function() {
	if (this.isGal)			{ return "GALContact"; }
	if (this.isShared())	{ return "SharedContact"; }
	if (this.isGroup())		{ return "Group"; }
	return "Contact";
};

/**
 * Gets the folder id.
 * 
 * @return	{String}		the folder id	
 */
ZmContact.prototype.getFolderId =
function() {
	return this.isShared()
		? this.folderId.split(":")[0]
		: this.folderId;
};

/**
 * Gets the attribute.
 * 
 * @param	{String}	name		the attribute name
 * @return	{String}	the value
 */
ZmContact.prototype.getAttr =
function(name) {
	var val = this.attr[name];
	return val ? ((val instanceof Array) ? val[0] : val) : "";
};

/**
 * Sets the attribute.
 * 
 * @param	{String}	name		the attribute name
 * @param	{String}	value		the attribute value
 */
ZmContact.prototype.setAttr =
function(name, value) {
	this.attr[name] = value;
};

/**
 * Sets the participant status.
 *
 * @param	{String}	value the participant status value
 */
ZmContact.prototype.setParticipantStatus =
function(ptst) {
	this.participantStatus = ptst;
};

/**
 * gets the participant status.
 *
 * @return	{String}    the value
 */
ZmContact.prototype.getParticipantStatus =
function() {
	return this.participantStatus;
};

/**
 * Sets the participant role.
 *
 * @param	{String}	value the participant role value
 */
ZmContact.prototype.setParticipantRole =
function(role) {
	this.participantRole = role;
};

/**
 * gets the participant role.
 *
 * @return	{String}    the value
 */
ZmContact.prototype.getParticipantRole =
function() {
	return this.participantRole;
};

/**
 * Removes the attribute.
 * 
 * @param	{String}	name		the attribute name
 */
ZmContact.prototype.removeAttr =
function(name) {
	delete this.attr[name];
};

/**
 * Gets the contact attributes.
 *
 * @param {String}	[prefix] if specified, only the the attributes that match the given prefix will be returned
 * @return	{Hash}	a hash of attribute/value pairs
 */
ZmContact.prototype.getAttrs = function(prefix) {
	var attrs = this.attr;
	if (prefix) {
		attrs = {};
		for (var aname in this.attr) {
			if (aname.replace(/\d+$/,"") == prefix) {
				attrs[aname] = this.attr[aname];
			}
		}
	}
	return attrs;
};

/**
 * Gets a normalized set of attributes where the attribute
 * names have been re-numbered as needed. For example, if the
 * attributes contains a "foo2" but no "foo", then the "foo2"
 * attribute will be renamed to "foo" in the returned object.
 * <p>
 * <strong>Note:</strong>
 * This method is expensive so should be called once and
 * cached temporarily as needed instead of being called
 * for each normalized attribute that is needed.
 * 
 * @param {String}	[prefix]		if specified, only the
 *                        the attributes that match the given
 *                        prefix will be returned.
 * @return	{Hash}	a hash of attribute/value pairs
 */
ZmContact.prototype.getNormalizedAttrs = function(prefix) {
	return ZmContact.getNormalizedAttrs(this.attr, prefix, ZmContact.IGNORE_NORMALIZATION);
};

/**
* Creates a contact from the given set of attributes. Used to create contacts on
* the fly (rather than by loading them). This method is called by a list's <code>create()</code>
* method.
* <p>
* If this is a GAL contact, we assume it is being added to the contact list.</p>
*
* @param {Hash}	attr			the attribute/value pairs for this contact
* @param {ZmBatchCommand}	batchCmd	the batch command that contains this request
*/
ZmContact.prototype.create =
function(attr, batchCmd) {
	var jsonObj = {CreateContactRequest:{_jsns:"urn:zimbraMail"}};
	var request = jsonObj.CreateContactRequest;
	var cn = request.cn = {};

	var folderId = attr[ZmContact.F_folderId] || ZmFolder.ID_CONTACTS;
	var folder = appCtxt.getById(folderId);
	if (folder && folder.isRemote()) {
		folderId = folder.getRemoteId();
	}
	cn.l = folderId;
	cn.a = [];

	for (var name in attr) {
		if (name == ZmContact.F_folderId ||
			name == "objectClass" ||
			name == "zimbraId" ||
			name == "createTimeStamp" ||
			name == "modifyTimeStamp") { continue; }

		this._addRequestAttr(cn, name, attr[name]);
	}

	var respCallback = new AjxCallback(this, this._handleResponseCreate, [attr, batchCmd != null]);

	if (batchCmd) {
		batchCmd.addRequestParams(jsonObj, respCallback);
	} else {
		appCtxt.getAppController().sendRequest({jsonObj:jsonObj, asyncMode:true, callback:respCallback});
	}
};

/**
 * @private
 */
ZmContact.prototype._handleResponseCreate =
function(attr, isBatchMode, result) {
	// dont bother processing creates when in batch mode (just let create
	// notifications handle them)
	if (isBatchMode) { return; }

	var resp = result.getResponse().CreateContactResponse;
	cn = resp ? resp.cn[0] : null;
	var id = cn ? cn.id : null;
	if (id) {
		this._fileAs = null;
		this._fullName = null;
		this.id = id;
		this.modified = cn.md;
		this.folderId = cn.l || ZmOrganizer.ID_ADDRBOOK;
		for (var a in attr) {
			if (!(attr[a] == undefined || attr[a] == ''))
				this.setAttr(a, attr[a]);
		}
		var msg = this.isGroup() ? ZmMsg.groupCreated : ZmMsg.contactCreated;
		appCtxt.getAppController().setStatusMsg(msg);
	} else {
		var msg = this.isGroup() ? ZmMsg.errorCreateGroup : ZmMsg.errorCreateContact;
		var detail = ZmMsg.errorTryAgain + "\n" + ZmMsg.errorContact;
		appCtxt.getAppController().setStatusMsg(msg, ZmStatusView.LEVEL_CRITICAL, detail);
	}
};

/**
 * Creates a contct from a VCF part of a message.
 * 
 * @param	{String}	msgId		the message
 * @param	{String}	vcardPartId	the vcard part id
 */
ZmContact.prototype.createFromVCard =
function(msgId, vcardPartId) {
	var jsonObj = {CreateContactRequest:{_jsns:"urn:zimbraMail"}};
	var cn = jsonObj.CreateContactRequest.cn = {l:ZmFolder.ID_CONTACTS};
	cn.vcard = {mid:msgId, part:vcardPartId};

	var params = {
		jsonObj: jsonObj,
		asyncMode: true,
		callback: (new AjxCallback(this, this._handleResponseCreateVCard)),
		errorCallback: (new AjxCallback(this, this._handleErrorCreateVCard))
	};

	appCtxt.getAppController().sendRequest(params);
};

/**
 * @private
 */
ZmContact.prototype._handleResponseCreateVCard =
function(result) {
	appCtxt.getAppController().setStatusMsg(ZmMsg.contactCreated);
};

/**
 * @private
 */
ZmContact.prototype._handleErrorCreateVCard =
function(ex) {
	appCtxt.getAppController().setStatusMsg(ZmMsg.errorCreateContact, ZmStatusView.LEVEL_CRITICAL);
};

/**
 * Updates contact attributes.
 *
 * @param {Hash}	attr		a set of attributes and new values
 * @param {AjxCallback}	callback	the callback
 */
ZmContact.prototype.modify =
function(attr, callback) {
	if (this.list.isGal) { return; }

	// change force to 0 and put up dialog if we get a MODIFY_CONFLICT fault?
	var jsonObj = {ModifyContactRequest:{_jsns:"urn:zimbraMail", replace:"0", force:"1"}};
	var cn = jsonObj.ModifyContactRequest.cn = {id:this.id};
	cn.a = [];
	var continueRequest = false;

	for (var name in attr) {
		if (name == ZmContact.F_folderId) { continue; }
		this._addRequestAttr(cn, name, (attr[name] && attr[name].value) || attr[name]);
		continueRequest = true;
	}

    // bug: 45026
    if (ZmContact.F_firstName in attr || ZmContact.F_lastName in attr || ZmContact.F_company in attr || ZmContact.X_fileAs in attr) {
        var contact = {};
        var fields = [ZmContact.F_firstName, ZmContact.F_lastName, ZmContact.F_company, ZmContact.X_fileAs];
        for (var i = 0; i < fields.length; i++) {
            var field = fields[i];
            var value = attr[field];
            contact[field] = value != null ? value : this.getAttr(field);
        }
        var fullName = ZmContact.computeFileAs(contact); 
        this._addRequestAttr(cn, ZmContact.X_fullName, fullName);
    }

	if (continueRequest) {
		var respCallback = new AjxCallback(this, this._handleResponseModify, [attr, callback]);
		appCtxt.getAppController().sendRequest({jsonObj:jsonObj, asyncMode:true, callback:respCallback});
	} else {
		if (attr[ZmContact.F_folderId]) {
			this._setFolder(attr[ZmContact.F_folderId]);
		}
	}
};

/**
 * @private
 */
ZmContact.prototype._handleResponseModify =
function(attr, callback, result) {
	var resp = result.getResponse().ModifyContactResponse;
	var cn = resp ? resp.cn[0] : null;
	var id = cn ? cn.id : null;

	if (id && id == this.id) {
		appCtxt.setStatusMsg(this.isGroup() ? ZmMsg.groupSaved : ZmMsg.contactSaved);
		// was this contact moved to another folder?
		if (attr[ZmContact.F_folderId] && this.folderId != attr[ZmContact.F_folderId]) {
			this._setFolder(attr[ZmContact.F_folderId]);
		}
	} else {
        var detail = ZmMsg.errorTryAgain + "\n" + ZmMsg.errorContact;
        appCtxt.getAppController().setStatusMsg(ZmMsg.errorModifyContact, ZmStatusView.LEVEL_CRITICAL, detail);
	}
	// NOTE: we no longer process callbacks here since notification handling
	//       takes care of everything
};

/**
 * @private
 */
ZmContact.prototype._handleResponseMove =
function(newFolderId, resp) {
	var newFolder = newFolderId && appCtxt.getById(newFolderId);
	var count = 1;
	if (newFolder)
		appCtxt.setStatusMsg(AjxMessageFormat.format(ZmMsg.actionMove, [count, AjxMessageFormat.format(ZmMsg[ZmItem.COUNT_KEY[ZmItem.CONTACT]], count), newFolder.name]));
	
	this._notify(ZmEvent.E_MODIFY, resp);
};

/**
 * @private
 */
ZmContact.prototype._setFolder =
function(newFolderId) {
	var folder = appCtxt.getById(this.folderId);
	var fId = folder ? folder.nId : null;
	if (fId == newFolderId) { return; }

	// moving out of a share or into one is handled differently (create then hard delete)
	var newFolder = appCtxt.getById(newFolderId);
	if (this.isShared() || (newFolder && newFolder.link)) {
		if (this.list) {
			this.list.moveItems({items:[this], folder:newFolder});
		}
	} else {
		var jsonObj = {ContactActionRequest:{_jsns:"urn:zimbraMail"}};
		jsonObj.ContactActionRequest.action = {id:this.id, op:"move", l:newFolderId};
		var respCallback = new AjxCallback(this, this._handleResponseMove, [newFolderId]);
		var accountName = appCtxt.multiAccounts && appCtxt.accountList.mainAccount.name;
		appCtxt.getAppController().sendRequest({jsonObj:jsonObj, asyncMode:true, callback:respCallback, accountName:accountName});
	}
};

/**
 * @private
 */
ZmContact.prototype.notifyModify =
function(obj, batchMode) {

	var result = ZmItem.prototype.notifyModify.apply(this, arguments);

	var context = window.parentAppCtxt || window.appCtxt;
	context.clearAutocompleteCache(ZmAutocomplete.AC_TYPE_CONTACT);

	if (result) {
		return result;
	}

	// cache old fileAs/fullName before resetting them
	var oldFileAs = this.getFileAs();
	var oldFullName = this.getFullName();
	this._resetCachedFields();

	var oldAttrCache = {};
	if (obj._attrs) {
		// remove attrs that were not returned back from the server
		var oldAttrs = this.getAttrs();
		for (var a in oldAttrs) {
			oldAttrCache[a] = oldAttrs[a];
			if (obj._attrs[a] == null)
				this.removeAttr(a);
		}

		// set attrs returned by server
		for (var a in obj._attrs) {
			this.setAttr(a, obj._attrs[a]);
		}
	}

	var details = {
		attr: obj._attrs,
		oldAttr: oldAttrCache,
		fullNameChanged: (this.getFullName() != oldFullName),
		fileAsChanged: (this.getFileAs() != oldFileAs),
		contact: this
	};

	// update this contact's list per old/new attrs
	this.list.modifyLocal(obj, details);
	this._notify(ZmEvent.E_MODIFY, obj);

	var buddy = this.getBuddy();
	if (buddy) {
		buddy._notifySetName(buddy.name);	// trigger a refresh
	}
};

/**
 * @private
 */
ZmContact.prototype.notifyDelete =
function() {
	ZmItem.prototype.notifyDelete.call(this);
	var context = window.parentAppCtxt || window.appCtxt;
	context.clearAutocompleteCache(ZmAutocomplete.AC_TYPE_CONTACT);
};

/**
 * Initializes this contact using an email address.
 *
 * @param {AjxEmailAddress|String}	email		an email address or an email string
 * @param {Boolean}	strictName	if <code>true</code>, do not try to set name from user portion of address
 */
ZmContact.prototype.initFromEmail =
function(email, strictName) {
	if (email instanceof AjxEmailAddress) {
		this.setAttr(ZmContact.F_email, email.getAddress());
		this._initFullName(email, strictName);
	} else {
		this.setAttr(ZmContact.F_email, email);
	}
};

/**
 * Initializes this contact using a phone number.
 *
 * @param {String}	phone		the phone string
 * @param {String}	field		the field or company phone if <code>null</code>
 */
ZmContact.prototype.initFromPhone =
function(phone, field) {
	this.setAttr(field || ZmContact.F_companyPhone, phone);
};

/**
 * Gets the email address.
 * 
 * @param {boolean}		asObj	if true, return an AjxEmailAddress
 * 
 * @return	the email address
 */
ZmContact.prototype.getEmail =
function(asObj) {

	var email = (this.getAttr(ZmContact.F_email) ||
				 this.getAttr(ZmContact.F_workEmail1) ||
				 this.getAttr(ZmContact.F_email2) ||
				 this.getAttr(ZmContact.F_workEmail2) ||
				 this.getAttr(ZmContact.F_email3) ||
				 this.getAttr(ZmContact.F_workEmail3));
	
	if (asObj) {
		email = AjxEmailAddress.parse(email);
        if(email){
		    email.isGroup = this.isGroup;
		    email.canExpand = this.canExpand;
        }
	}
	
	return email;
};
    
/**
 * Gets the lookup email address, when an contact object is located using email address we store
 * the referred email address in this variable for easy lookup
 *
 * @param {boolean}		asObj	if true, return an AjxEmailAddress
 *
 * @return	the lookup address
 */
ZmContact.prototype.getLookupEmail =
function(asObj) {
    var email = this._lookupEmail;

    if (asObj && email) {
        email = AjxEmailAddress.parse(email);
        email.isGroup = this.isGroup;
        email.canExpand = this.canExpand;
    }

	return  email;
};

/**
 * Gets the IM address.
 *
 * @return	the IM address
 */
ZmContact.prototype.getIMAddress =
function() {
	return this.getAttr(ZmContact.F_imAddress1) ||
		   this.getAttr(ZmContact.F_imAddress2) ||
		   this.getAttr(ZmContact.F_imAddress3);
};

/**
 * Gets the IM buddy.
 * 
 * @return	the IM buddy
 */
ZmContact.prototype.getBuddy =
function() {
	var buddy;
	if (appCtxt.get(ZmSetting.IM_ENABLED) && ZmImApp.loggedIn()) {
		var roster = AjxDispatcher.run("GetRoster");
		buddy = roster.getRosterItem(this.getAttr(ZmContact.F_email), false)  ||
				roster.getRosterItem(this.getAttr(ZmContact.F_email2), false) ||
				roster.getRosterItem(this.getAttr(ZmContact.F_email3), false) ||
				roster.getRosterItem(this.getIMAddress(), true);
	}
	return buddy;
};

/**
 * Gets the IM presence.
 * 
 * @return	{Object}	the presence
 */
ZmContact.prototype.getImPresence =
function() {
	var buddy = this.getBuddy();
	return (buddy) ? buddy.getPresence() : null;
};

/**
 * Gets the emails.
 * 
 * @return	{Array}	 an array of all valid emails for this contact
 */
ZmContact.prototype.getEmails =
function() {
	var emails = [];
	var attrs = this.getAttrs();
	for (var index = 0; index < ZmContact.EMAIL_FIELDS.length; index++) {
		var field = ZmContact.EMAIL_FIELDS[index];
		for (var i = 1; true; i++) {
			var aname = ZmContact.getAttributeName(field, i);
			if (!attrs[aname]) break;
			emails.push(attrs[aname]);
		}
	}
	return emails;
};

/**
 * Gets the full name.
 * 
 * @return	{String}	the full name
 */
ZmContact.prototype.getFullName =
function(html) {
    var fullNameHtml = null;
	if (!this._fullName || html) {
		var fullName = this.getAttr(ZmContact.X_fullName); // present if GAL contact
		if (fullName) {
			this._fullName = (fullName instanceof Array) ? fullName[0] : fullName;
		}
        else {
            this._fullName = this.getFullNameForDisplay(false);
        }

        if (html) {
            fullNameHtml = this.getFullNameForDisplay(html);
        }
	}

	// as a last resort, set it to fileAs
	if (!this._fullName) {
		this._fullName = this.getFileAs();
	}

	return fullNameHtml || this._fullName;
};

/*
* Gets the fullname for display -- includes (if applicable): prefix, first, middle, maiden, last, suffix
*
* @param {boolean}  if phonetic fields should be used
* @return {String}  the fullname for display
*/
ZmContact.prototype.getFullNameForDisplay =
function(html){
    var prefix = this.getAttr(ZmContact.F_namePrefix);
    var first = this.getAttr(ZmContact.F_firstName);
    var middle = this.getAttr(ZmContact.F_middleName);
    var maiden = this.getAttr(ZmContact.F_maidenName);
    var last = this.getAttr(ZmContact.F_lastName);
    var suffix = this.getAttr(ZmContact.F_nameSuffix);
    var pattern = ZmMsg.fullname;
    if (suffix) {
        pattern = maiden ? ZmMsg.fullnameMaidenSuffix : ZmMsg.fullnameSuffix;
    }
    else if (maiden) {
        pattern = ZmMsg.fullnameMaiden;
    }
    var formatter = new AjxMessageFormat(pattern);
    var args = [prefix,first,middle,maiden,last,suffix];
    if (!html){
        return AjxStringUtil.trim(formatter.format(args), true);
    }

    return this._getFullNameHtml(formatter, args);
};

/**
 * @param formatter
 * @param parts {Array} Name parts: [prefix,first,middle,maiden,last,suffix]
 */
ZmContact.prototype._getFullNameHtml = function(formatter, parts) {
    var a = [];
    var segments = formatter.getSegments();
    for (var i = 0; i < segments.length; i++) {
        var segment = segments[i];
        if (segment instanceof AjxFormat.TextSegment) {
            a.push(segment.format());
            continue;
        }
        // NOTE: Assume that it's a AjxMessageFormat.MessageSegment
        // NOTE: if not a AjxFormat.TextSegment.
        var index = segment.getIndex();
        var base = parts[index];
        var text = ZmContact.__RUBY_FIELDS[index] && this.getAttr(ZmContact.__RUBY_FIELDS[index]);
        a.push(AjxStringUtil.htmlRubyEncode(base, text));
    }
    return a.join("");
};
ZmContact.__RUBY_FIELDS = [
    null, ZmContact.F_phoneticFirstName, null, null,
    ZmContact.F_phoneticLastName, null
];

/**
 * Gets the tool tip for this contact.
 * 
 * @param	{String}	email		the email address
 * @param	{Boolean}	isGal		(not used)
 * @param	{String}	hint		the hint text
 * @return	{String}	the tool tip in HTML
 */
ZmContact.prototype.getToolTip =
function(email, isGal, hint) {
	// XXX: we dont cache tooltip info anymore since its too dynamic :/
	// i.e. IM status can change anytime so always rebuild tooltip and bug 13834
	var subs = {
		contact: this,
		entryTitle: this.getFileAs(),
		hint: hint,
		buddy: this.getBuddy()
	};

	return (AjxTemplate.expand("abook.Contacts#Tooltip", subs));
};

/**
 * Gets the filing string for this contact, computing it if necessary.
 * 
 * @param	{Boolean}	lower		<code>true</code> to use lower case
 * @return	{String}	the file as string
 */
ZmContact.prototype.getFileAs =
function(lower) {
	// update/null if modified
	if (!this._fileAs) {
		this._fileAs = ZmContact.computeFileAs(this);
		this._fileAsLC = this._fileAs ? this._fileAs.toLowerCase() : null;
	}
	// if for some reason fileAsLC is not set even though fileAs is, reset it
	if (lower && !this._fileAsLC) {
		this._fileAsLC = this._fileAs.toLowerCase();
	}
	return lower ? this._fileAsLC : this._fileAs;
};

/**
 * Gets the header.
 * 
 * @return	{String}	the header
 */
ZmContact.prototype.getHeader =
function() {
	return this.id ? this.getFileAs() : ZmMsg.newContact;
};

/**
 * Get the image URL.
 * 
 * @return	{String}	the image URL
 */
ZmContact.prototype.getImageUrl =
function() {
  	var image = this.getAttr(ZmContact.F_image);
  	if(!image || !image.part) { return null; }
  	var msgFetchUrl = appCtxt.get(ZmSetting.CSFE_MSG_FETCHER_URI);
  	return  [msgFetchUrl, "&id=", this.id, "&part=", image.part, "&t=", (new Date()).getTime()].join("");
};

/**
 * Gets the company field. Company field has a getter b/c fileAs may be the Company name so
 * company field should return "last, first" name instead *or* prepend the title
 * if fileAs is not Company (assuming it exists).
 * 
 * @return	{String}	the company
 */
ZmContact.prototype.getCompanyField =
function() {

	var attrs = this.getAttrs();
	if (attrs == null) return null;

	var fa = parseInt(attrs.fileAs);
	var val = [];
	var idx = 0;

	if (fa == ZmContact.FA_LAST_C_FIRST || fa == ZmContact.FA_FIRST_LAST) {
		// return the title, company name
		if (attrs.jobTitle) {
			val[idx++] = attrs.jobTitle;
			if (attrs.company)
				val[idx++] = ", ";
		}
		if (attrs.company)
			val[idx++] = attrs.company;

	} else if (fa == ZmContact.FA_COMPANY) {
		// return the first/last name
		if (attrs.lastName) {
			val[idx++] = attrs.lastName;
			if (attrs.firstName)
				val[idx++] = ", ";
		}

		if (attrs.firstName)
			val[idx++] = attrs.firstName;

		if (attrs.jobTitle)
			val[idx++] = " (" + attrs.jobTitle + ")";

	} else {
		// just return the title
		if (attrs.jobTitle) {
			val[idx++] = attrs.jobTitle;
			// and/or company name if applicable
			if (attrs.company && (attrs.fileAs == null || fa == ZmContact.FA_LAST_C_FIRST || fa == ZmContact.FA_FIRST_LAST))
				val[idx++] = ", ";
		}
		if (attrs.company && (attrs.fileAs == null || fa == ZmContact.FA_LAST_C_FIRST || fa == ZmContact.FA_FIRST_LAST))
			 val[idx++] = attrs.company;
	}
	if (val.length == 0) return null;
	return val.join("");
};

/**
 * Gets the work address.
 * 
 * @param	{Object}	instance		(not used)
 * @return	{String}	the work address
 */
ZmContact.prototype.getWorkAddrField =
function(instance) {
	var attrs = this.getAttrs();
	return this._getAddressField(attrs.workStreet, attrs.workCity, attrs.workState, attrs.workPostalCode, attrs.workCountry);
};

/**
 * Gets the home address.
 * 
 * @param	{Object}	instance		(not used)
 * @return	{String}	the home address
 */
ZmContact.prototype.getHomeAddrField =
function(instance) {
	var attrs = this.getAttrs();
	return this._getAddressField(attrs.homeStreet, attrs.homeCity, attrs.homeState, attrs.homePostalCode, attrs.homeCountry);
};

/**
 * Gets the other address.
 * 
 * @param	{Object}	instance		(not used)
 * @return	{String}	the other address
 */
ZmContact.prototype.getOtherAddrField =
function(instance) {
	var attrs = this.getAttrs();
	return this._getAddressField(attrs.otherStreet, attrs.otherCity, attrs.otherState, attrs.otherPostalCode, attrs.otherCountry);
};

/**
 * Gets the address book.
 * 
 * @return	{ZmAddrBook}	the address book
 */
ZmContact.prototype.getAddressBook =
function() {
	if (!this.addrbook) {
		this.addrbook = appCtxt.getById(this.folderId);
	}
	return this.addrbook;
};

/**
 * @private
 */
ZmContact.prototype._getAddressField =
function(street, city, state, zipcode, country) {
	if (street == null && city == null && state == null && zipcode == null && country == null) return null;

	var html = [];
	var idx = 0;

	if (street) {
		html[idx++] = street;
		if (city || state || zipcode)
			html[idx++] = "\n";
	}

	if (city) {
		html[idx++] = city;
		if (state)
			html[idx++] = ", ";
		else if (zipcode)
			html[idx++] = " ";
	}

	if (state) {
		html[idx++] = state;
		if (zipcode)
			html[idx++] = " ";
	}

	if (zipcode)
		html[idx++] = zipcode;

	if (country)
		html[idx++] = "\n" + country;

	return html.join("");
};

/**
 * Sets the full name based on an email address.
 * 
 * @private
 */
ZmContact.prototype._initFullName =
function(email, strictName) {
	var name = email.getName();
	name = AjxStringUtil.trim(name.replace(AjxEmailAddress.commentPat, '')); // strip comment (text in parens)

	if (name && name.length) {
		this._setFullName(name, [" "]);
	} else if (!strictName) {
		name = email.getAddress();
		if (name && name.length) {
			var i = name.indexOf("@");
			if (i == -1) return;
			name = name.substr(0, i);
			this._setFullName(name, [".", "_"]);
		}
	}
};

/**
 * Tries to extract a set of name components from the given text, with the
 * given list of possible delimiters. The first delimiter contained in the
 * text will be used. If none are found, the first delimiter in the list is used.
 * 
 * @private
 */
ZmContact.prototype._setFullName =
function(text, delims) {
	var delim = delims[0];
	for (var i = 0; i < delims.length; i++) {
		if (text.indexOf(delims[i]) != -1) {
			delim = delims[i];
			break;
		}
	}
    var parts = text.split(delim);
    var func = this["__setFullName_"+AjxEnv.DEFAULT_LOCALE] || this.__setFullName;
    func.call(this, parts, text, delims);
};

ZmContact.prototype.__setFullName = function(parts, text, delims) {
    this.setAttr(ZmContact.F_firstName, parts[0]);
    if (parts.length == 2) {
        this.setAttr(ZmContact.F_lastName, parts[1]);
    } else if (parts.length == 3) {
        this.setAttr(ZmContact.F_middleName, parts[1]);
        this.setAttr(ZmContact.F_lastName, parts[2]);
    }
};
ZmContact.prototype.__setFullName_ja = function(parts, text, delims) {
    if (parts.length > 2) {
        this.__setFullName(parts, text, delims);
        return;
    }
    // TODO: Perhaps do some analysis to auto-detect Japanese vs.
    // TODO: non-Japanese names. For example, if the name text is
    // TODO: comprised of kanji, treat it as "last first"; else if
    // TODO: first part is all uppercase, treat it as "last first";
    // TODO: else treat it as "first last".
    this.setAttr(ZmContact.F_lastName, parts[0]);
    if (parts.length > 1) {
        this.setAttr(ZmContact.F_firstName, parts[1]);
    }
};
ZmContact.prototype.__setFullName_ja_JP = ZmContact.prototype.__setFullName_ja;

/**
 * @private
 */
ZmContact.prototype._addRequestAttr =
function(cn, name, value) {
	var a = {n:name};
	if (name == ZmContact.F_image && AjxUtil.isString(value) && value.length) {
		// handle contact photo
		if (value.indexOf("aid_") != -1) {
			a.aid = value.substring(4);
		} else {
			a.part = value.substring(5);
		}
	} else {
		a._content = value || "";
	}

    if (value instanceof Array) {
        if (!cn._attrs)
            cn._attrs = {};
        cn._attrs[name] = value || "";
    }
    else  {
        if (!cn.a)
            cn.a = [];
        cn.a.push(a);
    }
};

/**
 * Reset computed fields.
 * 
 * @private
 */
ZmContact.prototype._resetCachedFields =
function() {
	this._fileAs = this._fileAsLC = this._fullName = null;
};

/**
 * Parse contact node.
 * 
 * @private
 */
ZmContact.prototype._loadFromDom =
function(node) {
	this.isLoaded = true;
	this.rev = node.rev;
	this.sf = node.sf || node._attrs.sf;
	this.folderId = node.l;
	this.created = node.cd;
	this.modified = node.md;

	this.attr = node._attrs || {};

	// for shared contacts, we get these fields outside of the attr part
	if (node.email)		{ this.attr[ZmContact.F_email] = node.email; }
	if (node.email2)	{ this.attr[ZmContact.F_email2] = node.email2; }
	if (node.email3)	{ this.attr[ZmContact.F_email3] = node.email3; }

	this.type = (this.attr[ZmContact.F_dlist] != null) ? ZmItem.GROUP : ZmItem.CONTACT;

	// check if the folderId is found in our address book (otherwise, we assume
	// this contact to be a shared contact)
	var ac = window.parentAppCtxt || window.appCtxt;
	this.addrbook = ac.getById(this.folderId);

	// dont process tags/flags for shared contacts until we get server support
	if (!this.isShared()) {
		this._parseFlags(node.f);
		this._parseTags(node.t);
	} else {
		// shared contacts are never fully loaded since we never cache them
		this.isLoaded = false;
	}

	// bug: 22174
	// We ignore the server's computed file-as property and instead
	// format it based on the user's locale.
	this._fileAs = ZmContact.computeFileAs(this);

	// Is this a distribution list?
	this.isDL = this.isGal && (this.attr[ZmContact.F_type] == "group");
	if (this.isDL) {
		this.canExpand = node.exp;
		var emails = this.getEmails();
		var ac = window.parentAppCtxt || window.appCtxt;
		for (var i = 0; i < emails.length; i++) {
			ac.setIsExpandableDL(emails[i], this.canExpand);
		}
	}
};

/**
 * Gets display text for an attendee. Prefers name over email.
 *
 * @param {constant}	type		the attendee type
 * @param {Boolean}	shortForm		if <code>true</code>, return only name or email
 * @return	{String}	the attendee
 */
ZmContact.prototype.getAttendeeText =
function(type, shortForm) {
	var email = this.getEmail(true);
	return (email?email.toString(shortForm || (type && type != ZmCalBaseItem.PERSON)):"");
};

/**
 * Gets display text for an attendee. Prefers name over email.
 *
 * @param {constant}	type		the attendee type
 * @param {Boolean}	shortForm		if <code>true</code>, return only name or email
 * @return	{String}	the attendee
 */
ZmContact.prototype.getAttendeeKey =
function() {
	var email = this.getLookupEmail() || this.getEmail();
	var name = this.getFullName();
	return email ? email : name;
};

/**
 * Gets the unknown fields.
 * 
 * @param	{function}	[sortByNameFunc]	sort by function
 * @return	{Array}	an array of field name/value pairs
 */
ZmContact.prototype.getUnknownFields = function(sortByNameFunc) {
	var map = ZmContact.__FIELD_MAP;
	if (!map) {
		map = ZmContact.__FIELD_MAP = {};
		for (var i = 0; i < ZmContact.DISPLAY_FIELDS; i++) {
			map[ZmContact.DISPLAY_FIELDS[i]] = true;
		}
	}
	var fields = [];
	var attrs = this.getAttrs();
	for (var aname in attrs) {
		var field = aname.replace(/\d+$/,"");
		if (map[aname]) continue;
		fields.push(field);
	}
	return this.getFields(fields, sortByNameFunc);
};

/**
 * Gets the fields.
 * 
 * @param	{Array}	field		the fields
 * @param	{function}	[sortByNameFunc]	sort by function
 * @return	{Array}	an array of field name/value pairs
 */
ZmContact.prototype.getFields =
function(fields, sortByNameFunc) {
	// TODO: [Q] Should sort function handle just the field names or the attribute names?
	var selection;
	var attrs = this.getAttrs();
	for (var index = 0; index < fields.length; index++) {
		for (var i = 1; true; i++) {
			var aname = ZmContact.getAttributeName(fields[index], i);
			if (!attrs[aname]) break;
			if (!selection) selection = {};
			selection[aname] = attrs[aname];
		}
	}
	if (sortByNameFunc && selection) {
		var keys = AjxUtil.keys(selection);
		keys.sort(sortByNameFunc);
		var nfields = {};
		for (var i = 0; i < keys; i++) {
			var key = keys[i];
			nfields[key] = fields[key];
		}
		selection = nfields;
	}
	return selection;
};

/**
 * Returns a list of distribution list members for this contact. Only the
 * requested range is returned.
 *
 * @param offset	{int}			offset into list to start at
 * @param limit		{int}			number of members to fetch and return
 * @param callback	{AjxCallback}	callback to run with results
 */
ZmContact.prototype.getDLMembers =
function(offset, limit, callback) {

	var result = {list:[], more:false, isDL:{}};
	if (!this.isDL) { return result; }

	var email = this.getEmail();
	var app = appCtxt.getApp(ZmApp.CONTACTS);
	var dl = app.getDL(email);
	if (!dl) {
		dl = result;
		dl.more = true;
		app.cacheDL(email, dl);
	}

	limit = limit || ZmContact.DL_PAGE_SIZE;
	var start = offset || 0;
	var end = (offset + limit) - 1;

	// see if we already have the requested members, or know that we don't
	if (dl.list.length >= end + 1 || !dl.more) {
		var list = dl.list.slice(offset, end + 1);
		result = {list:list, more:dl.more || (dl.list.length > end + 1), isDL:dl.isDL};
		DBG.println("dl", "found cached DL members");
		this._handleResponseGetDLMembers(start, limit, callback, result);
		return;
	}

	DBG.println("dl", "server call " + offset + " / " + limit);
	if (!dl.total || (offset < dl.total)) {
		var jsonObj = {GetDistributionListMembersRequest:{_jsns:"urn:zimbraAccount", offset:offset, limit:limit}};
		var request = jsonObj.GetDistributionListMembersRequest;
		request.dl = {_content: this.getEmail()};
		var respCallback = new AjxCallback(this, this._handleResponseGetDLMembers, [offset, limit, callback]);
		appCtxt.getAppController().sendRequest({jsonObj:jsonObj, asyncMode:true, callback:respCallback});
	} else {
		this._handleResponseGetDLMembers(start, limit, callback, result);
		return;
	}
};

ZmContact.prototype._handleResponseGetDLMembers =
function(offset, limit, callback, result) {

	if (!result.list) {
		var list = [];
		var resp = result.getResponse().GetDistributionListMembersResponse;
		var dl = appCtxt.getApp(ZmApp.CONTACTS).getDL(this.getEmail());
		var more = dl.more = resp.more;
		var isDL = {};
		var members = resp.dlm;
		if (members && members.length) {
			for (var i = 0, len = members.length; i < len; i++) {
				var member = members[i]._content;
				list.push(member);
				dl.list[offset + i] = member;
				if (members[i].isDL) {
					isDL[member] = dl.isDL[member] = true;
				}
			}
		}
		dl.total = resp.total;
		DBG.println("dl", list.join("<br>"));
		var result = {list:list, more:more, isDL:isDL};
	}
	DBG.println("dl", "returning list of " + result.list.length + ", more is " + result.more);
	callback.run(result);
};

/**
 * Returns a list of all the distribution list members for this contact.
 *
 * @param callback	{AjxCallback}	callback to run with results
 */
ZmContact.prototype.getAllDLMembers =
function(callback) {

	var result = {list:[], more:false, isDL:{}};
	if (!this.isDL) { return result; }

	var dl = appCtxt.getApp(ZmApp.CONTACTS).getDL(this.getEmail());
	if (dl && !dl.more) {
		result = {list:dl.list.slice(), more:false, isDL:dl.isDL};
		callback.run(result);
		return;
	}

	var nextCallback = new AjxCallback(this, this._getNextDLChunk, [callback]);
	this.getDLMembers(dl ? dl.list.length : 0, null, nextCallback);
};

ZmContact.prototype._getNextDLChunk =
function(callback, result) {

	var dl = appCtxt.getApp(ZmApp.CONTACTS).getDL(this.getEmail());
	if (result.more) {
		var nextCallback = new AjxCallback(this, this._getNextDLChunk, [callback]);
		this.getDLMembers(dl.list.length, null, nextCallback);
	} else {
		result.list = dl.list.slice();
		callback.run(result);
	}
};

// these need to be kept in sync with ZmContact.F_*
ZmContact._AB_FIELD = {
	firstName:				ZmMsg.AB_FIELD_firstName,		// file as info
	lastName:				ZmMsg.AB_FIELD_lastName,
	middleName:				ZmMsg.AB_FIELD_middleName,
	fullName:				ZmMsg.AB_FIELD_fullName,
	jobTitle:				ZmMsg.AB_FIELD_jobTitle,
	company:				ZmMsg.AB_FIELD_company,
	email:					ZmMsg.AB_FIELD_email,			// email addresses
	email2:					ZmMsg.AB_FIELD_email2,
	email3:					ZmMsg.AB_FIELD_email3,
	imAddress1:				ZmMsg.AB_FIELD_imAddress1,		// IM addresses
	imAddress2:				ZmMsg.AB_FIELD_imAddress2,
	imAddress3:				ZmMsg.AB_FIELD_imAddress3,
	image: 					ZmMsg.AB_FIELD_image,			// contact photo
	attachment:				ZmMsg.AB_FIELD_attachment,
	workStreet:				ZmMsg.AB_FIELD_street,			// work address info
	workCity:				ZmMsg.AB_FIELD_city,
	workState:				ZmMsg.AB_FIELD_state,
	workPostalCode:			ZmMsg.AB_FIELD_postalCode,
	workCountry:			ZmMsg.AB_FIELD_country,
	workURL:				ZmMsg.AB_FIELD_URL,
	workPhone:				ZmMsg.AB_FIELD_workPhone,
	workPhone2:				ZmMsg.AB_FIELD_workPhone2,
	workFax:				ZmMsg.AB_FIELD_workFax,
	assistantPhone:			ZmMsg.AB_FIELD_assistantPhone,
	companyPhone:			ZmMsg.AB_FIELD_companyPhone,
	callbackPhone:			ZmMsg.AB_FIELD_callbackPhone,
	homeStreet:				ZmMsg.AB_FIELD_street,			// home address info
	homeCity:				ZmMsg.AB_FIELD_city,
	homeState:				ZmMsg.AB_FIELD_state,
	homePostalCode:			ZmMsg.AB_FIELD_postalCode,
	homeCountry:			ZmMsg.AB_FIELD_country,
	homeURL:				ZmMsg.AB_FIELD_URL,
	homePhone:				ZmMsg.AB_FIELD_homePhone,
	homePhone2:				ZmMsg.AB_FIELD_homePhone2,
	homeFax:				ZmMsg.AB_FIELD_homeFax,
	mobilePhone:			ZmMsg.AB_FIELD_mobilePhone,
	pager:					ZmMsg.AB_FIELD_pager,
	carPhone:				ZmMsg.AB_FIELD_carPhone,
	otherStreet:			ZmMsg.AB_FIELD_street,			// other info
	otherCity:				ZmMsg.AB_FIELD_city,
	otherState:				ZmMsg.AB_FIELD_state,
	otherPostalCode:		ZmMsg.AB_FIELD_postalCode,
	otherCountry:			ZmMsg.AB_FIELD_country,
	otherURL:				ZmMsg.AB_FIELD_URL,
	otherPhone:				ZmMsg.AB_FIELD_otherPhone,
	otherFax:				ZmMsg.AB_FIELD_otherFax,
	notes:					ZmMsg.notes,					// misc fields
	birthday:				ZmMsg.AB_FIELD_birthday
};

ZmContact._AB_FILE_AS = {
	1:						ZmMsg.AB_FILE_AS_lastFirst,
	2:						ZmMsg.AB_FILE_AS_firstLast,
	3:						ZmMsg.AB_FILE_AS_company,
	4:						ZmMsg.AB_FILE_AS_lastFirstCompany,
	5:						ZmMsg.AB_FILE_AS_firstLastCompany,
	6:						ZmMsg.AB_FILE_AS_companyLastFirst,
	7:						ZmMsg.AB_FILE_AS_companyFirstLast
};

} // if (!window.ZmContact)
}
if (AjxPackage.define("zimbraMail.abook.model.ZmContactList")) {
/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite Web Client
 * Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2013 Zimbra Software, LLC.
 * 
 * The contents of this file are subject to the Zimbra Public License
 * Version 1.4 ("License"); you may not use this file except in
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.zimbra.com/license.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * ***** END LICENSE BLOCK *****
 */

/**
 * @overview
 * This file contains the contact list class.
 * 
 */

/**
 * Create a new, empty contact list.
 * @class
 * This class represents a list of contacts. In general, the list is the result of a
 * search. It may be the result of a <code>&lt;GetContactsRequest&gt;</code>, which returns all of the user's
 * local contacts. That list is considered to be canonical.
 * <p>
 * Loading of all local contacts has been optimized by delaying the creation of {@link ZmContact} objects until
 * they are needed. That has a big impact on IE, and not much on Firefox. Loading a subset
 * of attributes did not have much impact on load time, probably because a large majority
 * of contacts contain only those minimal fields.</p>
 *
 * @author Conrad Damon
 *
 * @param {ZmSearch}	search	the search that generated this list
 * @param {Boolean}	isGal		if <code>true</code>, this is a list of GAL contacts
 * @param {constant}	type		the item type
 * 
 * @extends		ZmList
 */
ZmContactList = function(search, isGal, type) {

	if (arguments.length == 0) { return; }
	type = type || ZmItem.CONTACT;
	ZmList.call(this, type, search);

	this.isGal = (isGal === true);
	this.isCanonical = false;
	this.isLoaded = false;

	this._app = appCtxt.getApp(ZmApp.CONTACTS);
	this._emailToContact = this._app._byEmail;
	this._imAddressToContact = this._app._byIM;
	this._phoneToContact = this._app._byPhone;
};

ZmContactList.prototype = new ZmList;
ZmContactList.prototype.constructor = ZmContactList;


// Constants

// Support for loading user's local contacts from a large string

ZmContactList.URL = "/Contacts?fmt=cf&t=2&all";	// REST URL for loading user's local contacts
ZmContactList.CONTACT_SPLIT_CHAR	= '\u001E';	// char for splitting string into contacts
ZmContactList.FIELD_SPLIT_CHAR		= '\u001D';	// char for splitting contact into fields
// fields that belong to a contact rather than its attrs
ZmContactList.IS_CONTACT_FIELD = {"id":true, "l":true, "d":true, "fileAsStr":true, "rev":true};


/**
 * Returns a string representation of the object.
 * 
 * @return		{String}		a string representation of the object
 */
ZmContactList.prototype.toString =
function() {
	return "ZmContactList";
};

/**
 * @private
 */
ZmContactList.prototype.addLoadedCallback =
function(callback) {
	if (this.isLoaded) {
		callback.run();
		return;
	}
	if (!this._loadedCallbacks) {
		this._loadedCallbacks = [];
	}
	this._loadedCallbacks.push(callback);
};

/**
 * @private
 */
ZmContactList.prototype._finishLoading =
function() {
	DBG.timePt("done loading " + this.size() + " contacts");
	this.isLoaded = true;
	if (this._loadedCallbacks) {
		var callback;
		while (callback = this._loadedCallbacks.shift()) {
			callback.run();
		}
	}
};

/**
 * Retrieves the contacts from the back end, and parses the response. The list is then sorted.
 * This method is used only by the canonical list of contacts, in order to load their content.
 * <p>
 * Loading a minimal set of attributes did not result in a significant performance gain.
 * </p>
 * 
 * @private
 */
ZmContactList.prototype.load =
function(callback, errorCallback, accountName) {
	// only the canonical list gets loaded
	this.isCanonical = true;
	var respCallback = new AjxCallback(this, this._handleResponseLoad, [callback]);
	DBG.timePt("requesting contact list", true);

	var params = {asyncMode:true, noBusyOverlay:true, callback:respCallback, errorCallback:errorCallback};
	params.restUri = AjxUtil.formatUrl({path:["/home/", (accountName || appCtxt.getUsername()), ZmContactList.URL].join(""), qsReset:true});
	DBG.println(AjxDebug.DBG1, "loading contacts from " + params.restUri);
	appCtxt.getAppController().sendRequest(params);
};

/**
 * @private
 */
ZmContactList.prototype._handleResponseLoad =
function(callback, result) {
	DBG.timePt("got contact list");

	var text = result.getResponse();
	if (text) {
		var contacts = text.split(ZmContactList.CONTACT_SPLIT_CHAR);
		for (var i = 0, len = contacts.length; i < len; i++) {
			var fields = contacts[i].split(ZmContactList.FIELD_SPLIT_CHAR);
			var contact = {}, attrs = {};;
			for (var j = 0, len1 = fields.length; j < len1; j += 2) {
				if (ZmContactList.IS_CONTACT_FIELD[fields[j]]) {
					contact[fields[j]] = fields[j + 1];
				} else {
					attrs[fields[j]] = fields[j + 1];
				}
			}
			contact._attrs = attrs;
			this._addContact(contact);
		}
	}
	this._finishLoading();

	if (callback) {
		callback.run();
	}
};

ZmContactList.prototype.add = 
function(item, index) {
	if (!item.id || !this._idHash[item.id]) {
		this._vector.add(item, index);
		if (item.id) {
			this._idHash[item.id] = item;
		}
		this._updateHashes(item, true);
	}
};

ZmContactList.prototype.cache = 
function(offset, newList) {
	var getId = function(){
		return this.id;
	}
	var exists = function(obj) {
		return this._vector.containsLike(obj, getId);
	}
	var unique = newList.sub(exists, this);

	this.getVector().merge(offset, unique);
	// reparent each item within new list, and add it to ID hash
	var list = unique.getArray();
	for (var i = 0; i < list.length; i++) {
		var item = list[i];
		item.list = this;
		if (item.id) {
			this._idHash[item.id] = item;
		}
	}
};

/**
 * @private
 */
ZmContactList.prototype._addContact =
function(contact) {

	// note that we don't create a ZmContact here (optimization)
	contact.list = this;
	this._updateHashes(contact, true);
	var fn = [], fl = [];
	if (contact._attrs[ZmContact.F_firstName])	{ fn.push(contact._attrs[ZmContact.F_firstName]); }
	if (contact._attrs[ZmContact.F_middleName])	{ fn.push(contact._attrs[ZmContact.F_middleName]); }
	if (contact._attrs[ZmContact.F_lastName])	{ fn.push(contact._attrs[ZmContact.F_lastName]); }
	if (fn.length) {
		contact._attrs[ZmContact.X_fullName] = fn.join(" ");
	}
	if (contact._attrs[ZmContact.F_firstName])	{ fl.push(contact._attrs[ZmContact.F_firstName]); }
	if (contact._attrs[ZmContact.F_lastName])	{ fl.push(contact._attrs[ZmContact.F_lastName]); }
	contact._attrs[ZmContact.X_firstLast] = fl.join(" ");

	this.add(contact);
};

/**
 * Converts an anonymous contact object (contained by the JS returned by load request)
 * into a ZmContact, and updates the containing list if it is the canonical one.
 *
 * @param {Object}	contact		a contact
 * @param {int}	idx		the index of contact in canonical list
 * 
 * @private
 */
ZmContactList.prototype._realizeContact =
function(contact, idx) {

	if (contact instanceof ZmContact) { return contact; }
	if (contact && contact.type == ZmItem.CONTACT) { return contact; }	// instanceof often fails in new window

	var args = {list:this};
	var obj = eval(ZmList.ITEM_CLASS[this.type]);
	var realContact = obj && obj.createFromDom(contact, args);

	if (this.isCanonical) {
		var a = this.getArray();
		idx = idx || this._getIndexById(contact.id);
		a[idx] = realContact;
		this._updateHashes(realContact, true);
		this._idHash[contact.id] = realContact;
	}

	return realContact;
};

/**
 * Finds the array index for the contact with the given ID.
 *
 * @param {int}	id		the contact ID
 * @return	{int}	the index
 * @private
 */
ZmContactList.prototype._getIndexById =
function(id) {
	var a = this.getArray();
	for (var i = 0; i < a.length; i++) {
		if (a[i].id == id) {
			return i;
		}
	}
	return null;
};

/**
 * Override in order to make sure the contacts have been realized. We don't
 * call realizeContact() since this is not the canonical list.
 *
 * @param {int}	offset		the starting index
 * @param {int}	limit		the size of sublist
 * @return	{AjxVector}	a vector of {@link ZmContact} objects
 */
ZmContactList.prototype.getSubList =
function(offset, limit, folderId) {
	if (folderId && this.isCanonical) {
		// only collect those contacts that belong to the given folderId if provided
		var newlist = [];
		var sublist = this.getArray();
		var offsetCount = 0;
		this.setHasMore(false);

		for (var i = 0; i < sublist.length; i++) {
			sublist[i] = this._realizeContact(sublist[i], i);
			var folder = appCtxt.getById(sublist[i].folderId);
			if (folder && folder.nId == ZmOrganizer.normalizeId(folderId)) {
				if (offsetCount >= offset) {
					if (newlist.length == limit) {
						this.setHasMore(true);
						break;
					}
					newlist.push(sublist[i]);
				}
				offsetCount++;
			}
		}

		return AjxVector.fromArray(newlist);
	} else {
		var vec = ZmList.prototype.getSubList.call(this, offset, limit);
		if (vec) {
			var a = vec.getArray();
			for (var i = 0; i < a.length; i++) {
				a[i] = this._realizeContact(a[i], offset + i);
			}
		}

		return vec;
	}
};

/**
 * Override in order to make sure the contact has been realized. Canonical list only.
 *
 * @param {int}	id		the contact ID
 * @return	{ZmContact}	the contact or <code>null</code> if not found
 */
ZmContactList.prototype.getById =
function(id) {
	if (!id || !this.isCanonical) return null;

	var contact = this._idHash[id];
	return contact ? this._realizeContact(contact) : null;
};

/**
 * Gets the contact with the given address, if any (canonical list only).
 *
 * @param {String}	address	an email address
 * @return	{ZmContact}	the contact or <code>null</code> if not found
 */
ZmContactList.prototype.getContactByEmail =
function(address) {
	if (!address || !this.isCanonical) return null;

	var contact = this._emailToContact[address.toLowerCase()];
	if (contact) {
		contact = this._realizeContact(contact);
		contact._lookupEmail = address;	// so caller knows which address matched
		return contact;
	} else {
		return null;
	}
};

/**
 * Gets the contact.
 * 
 * @param	{String}	addr		the IM address
 * @return	{ZmContact}	the contact or <code>null</code> if not found
 */
ZmContactList.prototype.getContactByIMAddress =
function(addr) {
	var contact = this._imAddressToContact[addr.toLowerCase()];
	return contact ? this._realizeContact(contact) : null;
};

/**
 * Gets information about the contact with the given phone number, if any (canonical list only).
 *
 * @param {String}	phone	the phone number
 * @return	{Hash}	an object with <code>contact</code> = the contact & <code>field</code> = the field with the matching phone number
 */
ZmContactList.prototype.getContactByPhone =
function(phone) {
	if (!phone || !this.isCanonical) return null;

	var digits = this._getPhoneDigits(phone);
	var data = this._phoneToContact[digits];
	if (data) {
		data.contact = this._realizeContact(data.contact);
		return data;
	} else {
		return null;
	}
};

/**
 * Moves a list of items to the given folder.
 * <p>
 * This method calls the base class for normal "moves" UNLESS we're dealing w/
 * shared items (or folder) in which case we must send a CREATE request for the
 * given folder to the server followed by a hard delete of the shared contact.
 * </p>
 *
 * @param {Hash}	params		a hash of parameters
 * @param	{Array}       params.items			a list of items to move
 * @param	{ZmFolder}	params.folder		the destination folder
 * @param	{Hash}	       params.attrs		the additional attrs for SOAP command
 * @param	{Boolean}	params.outOfTrash	if <code>true</code>, we are moving contacts out of trash
 */
ZmContactList.prototype.moveItems =
function(params) {

	params = Dwt.getParams(arguments, ["items", "folder", "attrs", "outOfTrash"]);
	params.items = AjxUtil.toArray(params.items);

	var moveBatchCmd = new ZmBatchCommand(true, null, true);
	var loadBatchCmd = new ZmBatchCommand(true, null, true);
	var softMove = [];
	var hardMove = [];

	// if the folder we're moving contacts to is a shared folder, then dont bother
	// checking whether each item is shared or not
	if (params.items[0] && params.items[0] instanceof ZmItem) {
		for (var i = 0; i < params.items.length; i++) {
			var contact = params.items[i];

			if (contact.isReadOnly()) { continue; }

			if (contact.isShared() || params.folder.link) {
				hardMove.push(contact);
				if (!contact.isLoaded) {
					loadBatchCmd.add(new AjxCallback(contact, contact.load, [null, null]));
				}
				moveBatchCmd.add(this._getCopyCmd(contact, params.folder));
			} else {
				softMove.push(contact);
			}
		}
	} else {
		softMove = params.items;
	}

	if (hardMove.length > 0) {
		var params1 = {
			items: hardMove,
			action: "delete",
			actionText: ZmMsg.actionMove,
			actionArg: params.folder.getName(false, false, true)
		};

		if (loadBatchCmd.size()) {
			var respCallback = new AjxCallback(this, this._handleResponseLoadMove, [moveBatchCmd, params1]);
			loadBatchCmd.run(respCallback);
		} else {
			var deleteCmd = new AjxCallback(this, this._itemAction, [params1]);
			moveBatchCmd.add(deleteCmd);

			var respCallback = new AjxCallback(this, this._handleResponseMoveBatchCmd);
			moveBatchCmd.run(respCallback);
		}
	}

	// for "soft" moves, handle moving out of Trash differently
	if (softMove.length > 0) {
		var params1 = AjxUtil.hashCopy(params);
		params1.attrs = params.attrs || {};
		params1.attrs.l = params.folder.id;
		params1.action = "move";
        params1.accountName = appCtxt.multiAccounts && appCtxt.accountList.mainAccount.name;
        if (params1.folder.id == ZmFolder.ID_TRASH) {
            params1.actionText = ZmMsg.actionTrash;
            // bug: 47389 avoid moving to local account's Trash folder.
            params1.accountName = appCtxt.multiAccounts && params.items[0].getAccount().name;
        } else {
            params1.actionText = ZmMsg.actionMove;
            params1.actionArg = params.folder.getName(false, false, true);
        }
		params1.callback = params.outOfTrash && new AjxCallback(this, this._handleResponseMoveItems, params);

		this._itemAction(params1);
	}
};

/**
 * @private
 */
ZmContactList.prototype._handleResponseMoveBatchCmd =
function(result) {
	var resp = result.getResponse().BatchResponse.ContactActionResponse;
	// XXX: b/c the server does not return notifications for actions done on
	//      shares, we manually notify - TEMP UNTIL WE GET BETTER SERVER SUPPORT
	var ids = resp[0].action.id.split(",");
	for (var i = 0; i < ids.length; i++) {
		var contact = appCtxt.cacheGet(ids[i]);
		if (contact && contact.isShared()) {
			contact.notifyDelete();
			appCtxt.cacheRemove(ids[i]);
		}
	}
};

/**
 * @private
 */
ZmContactList.prototype._handleResponseLoadMove =
function(moveBatchCmd, params) {
	var deleteCmd = new AjxCallback(this, this._itemAction, [params]);
	moveBatchCmd.add(deleteCmd);

	var respCallback = new AjxCallback(this, this._handleResponseMoveBatchCmd);
	moveBatchCmd.run(respCallback);
};

/**
 * @private
 */
ZmContactList.prototype._handleResponseBatchLoad =
function(batchCmd, folder, result, contact) {
	batchCmd.add(this._getCopyCmd(contact, folder));
};

/**
 * @private
 */
ZmContactList.prototype._getCopyCmd =
function(contact, folder) {
	var temp = new ZmContact(null, this);
	for (var j in contact.attr) {
		temp.attr[j] = contact.attr[j];
	}
	temp.attr[ZmContact.F_folderId] = folder.id;

	return new AjxCallback(temp, temp.create, [temp.attr]);
};

/**
 * Deletes contacts after checking that this is not a GAL list.
 *
 * @param {Hash}	params		a hash of parameters
 * @param	{Array}	       params.items			the list of items to delete
 * @param	{Boolean}	params.hardDelete	if <code>true</code>, force physical removal of items
 * @param	{Object}	params.attrs			the additional attrs for SOAP command
 */
ZmContactList.prototype.deleteItems =
function(params) {
	if (this.isGal) {
		DBG.println(AjxDebug.DBG1, "Cannot delete GAL contacts");
		return;
	}
	ZmList.prototype.deleteItems.call(this, params);
};

/**
 * Sets the is GAL flag.
 * 
 * @param	{Boolean}	isGal		<code>true</code> if contact list is GAL
 */
ZmContactList.prototype.setIsGal =
function(isGal) {
	this.isGal = isGal;
};

ZmContactList.prototype.notifyCreate =
function(node) {
	var obj = eval(ZmList.ITEM_CLASS[this.type]);
	if (obj) {
		var item = obj.createFromDom(node, {list:this});
		var index = this._sortIndex(item);
		// only add if it sorts into this list
		var listSize = this.size();
		var visible = false;
		if (index < listSize || listSize == 0 || (index==listSize && !this._hasMore)) {
			this.add(item, index);
			this.createLocal(item);
			visible = true;
		}
		this._notify(ZmEvent.E_CREATE, {items: [item], sortIndex: index, visible: visible});
	}
};

/**
 * Moves the items.
 * 
 * @param	{Array}	items		an array of {@link ZmContact} objects
 * @param	{String}	folderId	the folder id
 */
ZmContactList.prototype.moveLocal =
function(items, folderId) {
	// don't remove any contacts from the canonical list
	if (!this.isCanonical)
		ZmList.prototype.moveLocal.call(this, items, folderId);
	if (folderId == ZmFolder.ID_TRASH) {
		for (var i = 0; i < items.length; i++) {
			this._updateHashes(items[i], false);
		}
	}
};

/**
 * Deletes the items.
 * 
 * @param	{Array}	items		an array of {@link ZmContact} objects
 */
ZmContactList.prototype.deleteLocal =
function(items) {
	ZmList.prototype.deleteLocal.call(this, items);
	for (var i = 0; i < items.length; i++) {
		this._updateHashes(items[i], false);
	}
};

/**
 * Handle modified contact.
 * 
 * @private
 */
ZmContactList.prototype.modifyLocal =
function(item, details) {
	if (details) {
		// notify item's list
		details.items = [item];
		this._notify(ZmEvent.E_MODIFY, details);
	}

	var contact = details.contact;
	if (this.isCanonical) {
		// Remove traces of old contact - NOTE: we pass in null for the ID on
		// PURPOSE to avoid overwriting the existing cached contact
		var oldContact = new ZmContact(null, this);
		oldContact.id = details.contact.id;
		oldContact.attr = details.oldAttr;
		this._updateHashes(oldContact, false);

		// add new contact to hashes
		this._updateHashes(contact, true);
	}

	// place in correct position in list
	if (details.fileAsChanged) {
		this.remove(contact);
		var index = this._sortIndex(contact);
		var listSize = this.size();
		if (index < listSize || listSize == 0 || (index == listSize && !this._hasMore)) {
			this.add(contact, index);
		}
	}

	// reset addrbook property
	if (contact.addrbook && (contact.addrbook.id != contact.folderId)) {
		contact.addrbook = appCtxt.getById(contact.folderId);
	}
};

/**
 * Creates the item local.
 * 
 * @param	{ZmContact}	item		the item
 */
ZmContactList.prototype.createLocal =
function(item) {
	this._updateHashes(item, true);
};

/**
 * @private
 */
ZmContactList.prototype._updateHashes =
function(contact, doAdd) {

	this._app.updateCache(contact, doAdd);

	// Update email hash.
	for (var index = 0; index < ZmContact.EMAIL_FIELDS.length; index++) {
		var field = ZmContact.EMAIL_FIELDS[index];
		for (var i = 1; true; i++) {
			var aname = ZmContact.getAttributeName(field, i);
			var avalue = ZmContact.getAttr(contact, aname);
			if (!avalue) break;
			if (doAdd) {
				this._emailToContact[avalue.toLowerCase()] = contact;
			} else {
				delete this._emailToContact[avalue.toLowerCase()];
			}
		}
	}

	// Update phone hash.
	if (appCtxt.get(ZmSetting.VOICE_ENABLED)) {
		for (var index = 0; index < ZmContact.PHONE_FIELDS.length; index++) {
			var field = ZmContact.PHONE_FIELDS[index];
			for (var i = 1; true; i++) {
				var aname = ZmContact.getAttributeName(field, i);
				var avalue = ZmContact.getAttr(contact, aname);
				if (!avalue) break;
				var digits = this._getPhoneDigits(avalue);
				if (digits) {
					if (doAdd) {
						this._phoneToContact[avalue] = {contact: contact, field: aname};
					} else {
						delete this._phoneToContact[avalue];
					}
				}
			}
		}
	}

	// Update IM hash.
	if (appCtxt.get(ZmSetting.IM_ENABLED)) {
		for (var index = 0; index < ZmContact.IM_FIELDS.length; index++) {
			var field = ZmContact.IM_FIELDS[index];
			for (var i = 1; true; i++) {
				var aname = ZmContact.getAttributeName(field, i);
				var avalue = ZmContact.getAttr(contact, aname);
				if (!avalue) break;
				if (doAdd) {
					this._imAddressToContact[avalue.toLowerCase()] = contact;
				} else {
					delete this._imAddressToContact[avalue.toLowerCase()];
				}
			}
		}
	}
};

/**
 * Strips all non-digit characters from a phone number.
 * 
 * @private
 */
ZmContactList.prototype._getPhoneDigits =
function(phone) {
	return phone.replace(/[^\d]/g, '');
};

/**
 * Returns the position at which the given contact should be inserted in this list.
 * 
 * @private
 */
ZmContactList.prototype._sortIndex =
function(contact) {
	var a = this._vector.getArray();
	for (var i = 0; i < a.length; i++) {
		if (ZmContact.compareByFileAs(a[i], contact) > 0) {
			return i;
		}
	}
	return a.length;
};

/**
 * @private
 */
ZmContactList.prototype._handleResponseModifyItem =
function(item, result) {
	// NOTE: we overload and do nothing b/c base class does more than we want
	//       (since everything is handled by notifications)
};
}
if (AjxPackage.define("zimbraMail.abook.view.ZmContactsHelper")) {
/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite Web Client
 * Copyright (C) 2007, 2008, 2009, 2010, 2011, 2013 Zimbra Software, LLC.
 * 
 * The contents of this file are subject to the Zimbra Public License
 * Version 1.4 ("License"); you may not use this file except in
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.zimbra.com/license.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * ***** END LICENSE BLOCK *****
 */

/**
 * @overview
 * This file contains a contact helper class.
 * 
 */

/**
 * Default constructor for helper class.
 * @class
 * Miscellaneous contacts-related utility functions. So far, mostly things that
 * {@link ZmContactPicker} and {@link ZmGroupView} both need to perform a contacts search and
 * display results in a list view.
 *
 * @author Conrad Damon
 */
ZmContactsHelper = function() {};

/**
 * Performs a contact search (in either personal contacts or in the GAL) and populates
 * the source list view with the results.
 *
 * @param	{Hash}	params		a hash of parameters
 * @param {Object}	params.obj			the object that is doing the search
 * @param {String}	params.query			the query string to search on
 * @param {String}	params.queryHint		the query hint (i.e. searching shared folders)
 * @param {Boolean}	params.ascending		if <code>true</code>, sort in ascending order
 * @param {int}	params.lastId		the ID of last item displayed (for pagination)
 * @param {String}	params.lastSortVal	the value of sort field for above item
 * @param {AjxCallback}	params.respCallback	the callback to call once response comes back from server
 * @param {AjxCallback}	params.errorCallback	the callback to call if error returned from server
 * @param {String}	params.accountName	the account to make search request on behalf of
 * @param {Array}   params.conds	the conds to restrict the search by (array of {attr:"", op:"", value:""} hashes)
 */
ZmContactsHelper.search =
function(params) {
	var o = params.obj;
	if (o._searchButton) {
		o._searchButton.setEnabled(false);
	}

	params.sortBy = params.ascending ? ZmSearch.NAME_ASC : ZmSearch.NAME_DESC;
	params.types = AjxVector.fromArray([ZmItem.CONTACT]);
	params.offset = params.offset || 0;
	params.limit = ZmContactsApp.SEARCHFOR_MAX;
	params.contactSource = o._contactSource;
	params.field = "contact";

	var search = new ZmSearch(params);
	search.execute({callback:params.respCallback, errorCallback:params.errorCallback});
};

/**
 * Take the contacts and create a list of their email addresses (a contact may have more than one)
 * 
 * @private
 */
ZmContactsHelper._processSearchResponse = 
function(resp) {
	var vec = resp.getResults(ZmItem.CONTACT);

	// Take the contacts and create a list of their email addresses (a contact may have more than one)
	var list = [];
	var a = vec.getArray();
	for (var i = 0; i < a.length; i++) {
		var contact = a[i];
		if (contact.isGroup() && !contact.isDL) {
			var members = contact.getGroupMembers().good.toString(AjxEmailAddress.SEPARATOR);
			ZmContactsHelper._addContactToList(list, contact, members, true);
		} else {
			var emails = contact.isGal ? [contact.getEmail()] : contact.getEmails();
			for (var j = 0; j < emails.length; j++) {
				ZmContactsHelper._addContactToList(list, contact, emails[j]);
			}
		}
	}
	
	return list;
};

/**
 * @private
 */
ZmContactsHelper._addContactToList = 
function(list, contact, addr, isGroup) {
	
	var fileAs = contact.getFileAs();
	var name = (fileAs != addr) ? fileAs : "";
	var email = new AjxEmailAddress(addr, null, name, null, isGroup);
	email.id = Dwt.getNextId();
	email.__contact = contact;
	email.icon = contact.getIcon();
	if (contact.isDL) {
		email.isGroup = true;
		email.canExpand = contact.canExpand;
		var ac = window.parentAppCtxt || window.appCtxt;
		ac.setIsExpandableDL(addr, email.canExpand);
	}
	list.push(email);
};

/**
 * The items are AjxEmailAddress objects
 * 
 * @private
 */
ZmContactsHelper._getEmailField =
function(html, idx, item, field, colIdx) {
	if (field == ZmItem.F_TYPE) {
		html[idx++] = AjxImg.getImageHtml(item.icon);
	} else if (field == ZmItem.F_NAME) {
		html[idx++] = "<nobr>";
		html[idx++] = AjxStringUtil.htmlEncode(item.name);
		html[idx++] = "</nobr>";
	} else if (field == ZmItem.F_EMAIL) {
		html[idx++] = AjxStringUtil.htmlEncode(item.address);
	} else if (field == ZmItem.F_DEPARTMENT) {
		if (item.__contact) {
			html[idx++] = AjxStringUtil.htmlEncode(ZmContact.getAttr(item.__contact, ZmContact.F_department));
		}
	}
	return idx;
};
}
if (AjxPackage.define("zimbraMail.abook.view.ZmContactPicker")) {
/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite Web Client
 * Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013 Zimbra Software, LLC.
 * 
 * The contents of this file are subject to the Zimbra Public License
 * Version 1.4 ("License"); you may not use this file except in
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.zimbra.com/license.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * ***** END LICENSE BLOCK *****
 */

/**
 * @overview
 * This file contains the contact picker classes.
 * 
 */

/**
 * Creates a dialog that lets the user select addresses from a contact list.
 * @constructor
 * @class
 * This class creates and manages a dialog that lets the user select addresses
 * from a contact list. Two lists are maintained, one with contacts to select
 * from, and one that contains the selected addresses. Between them are buttons
 * to shuffle addresses back and forth between the two lists.
 *
 * @author Conrad Damon
 * 
 * @param {Array}	buttonInfo		the transfer button IDs and labels
 * 
 * @extends		DwtDialog
 */
ZmContactPicker = function(buttonInfo) {

	DwtDialog.call(this, {parent:appCtxt.getShell(), title:ZmMsg.selectAddresses, id: "ZmContactPicker"});

	this._buttonInfo = buttonInfo;
	this._initialized = false;
	this._offset = 0;
	this._defaultQuery = ".";
	this._list = new AjxVector();
	this._detailed = appCtxt.get(ZmSetting.DETAILED_CONTACT_SEARCH_ENABLED);
	this._searchCleared = {};
	this._ignoreSetDragBoundries = true;

	this._searchErrorCallback = new AjxCallback(this, this._handleErrorSearch);
};

ZmContactPicker.prototype = new DwtDialog;
ZmContactPicker.prototype.constructor = ZmContactPicker;

// Consts

ZmContactPicker.CHOOSER_HEIGHT = 300;

ZmContactPicker.SEARCH_BASIC = "search";
ZmContactPicker.SEARCH_NAME = "name";
ZmContactPicker.SEARCH_EMAIL = "email";
ZmContactPicker.SEARCH_DEPT = "dept";
ZmContactPicker.SEARCH_PHONETIC = "phonetic";

ZmContactPicker.HINT = {};
ZmContactPicker.HINT[ZmContactPicker.SEARCH_BASIC] = ZmMsg.contactPickerHint;
ZmContactPicker.HINT[ZmContactPicker.SEARCH_NAME] = ZmMsg.contactPickerHint;
ZmContactPicker.HINT[ZmContactPicker.SEARCH_EMAIL] = ZmMsg.contactPickerEmailHint;
ZmContactPicker.HINT[ZmContactPicker.SEARCH_DEPT] = ZmMsg.contactPickerDepartmentHint;
ZmContactPicker.HINT[ZmContactPicker.SEARCH_PHONETIC] = ZmMsg.contactPickerPhoneticHint;

ZmContactPicker.SHOW_ON_GAL = [ZmContactPicker.SEARCH_BASIC, ZmContactPicker.SEARCH_NAME, ZmContactPicker.SEARCH_EMAIL, ZmContactPicker.SEARCH_DEPT];
ZmContactPicker.SHOW_ON_NONGAL = [ZmContactPicker.SEARCH_BASIC, ZmContactPicker.SEARCH_NAME, ZmContactPicker.SEARCH_PHONETIC, ZmContactPicker.SEARCH_EMAIL];
	

// Public methods

/**
 * Returns a string representation of the object.
 * 
 * @return		{String}		a string representation of the object
 */
ZmContactPicker.prototype.toString =
function() {
	return "ZmContactPicker";
};

/**
* Displays the contact picker dialog. The source list is populated with
* contacts, and the target list is populated with any addresses that are
* passed in. The address button that was used to popup the dialog is set
* as the active button.
*
* @param {String}	buttonId	the button ID of the button that called us
* @param {Hash}	addrs		a hash of 3 vectors (one for each type of address)
* @param {String}	str		initial search string
*/
ZmContactPicker.prototype.popup =
function(buttonId, addrs, str, account) {

	if (!this._initialized) {
		this._initialize(account);
		this._initialized = true;
	}
	else if (appCtxt.multiAccounts && this._account != account) {
		this._account = account;
		this._resetSelectDiv();
	}
	this._offset = 0;

	var searchFor = this._selectDiv ? this._selectDiv.getValue() : ZmContactsApp.SEARCHFOR_CONTACTS;

	// reset column sorting preference
	this._chooser.sourceListView.setSortByAsc(ZmItem.F_NAME, true);

	// reset button states
	this._chooser.reset();
	if (buttonId) {
		this._chooser._setActiveButton(buttonId);
	}

	// populate target list if addrs were passed in
	if (addrs) {
		for (var id in addrs) {
			this._chooser.addItems(addrs[id], DwtChooserListView.TARGET, true, id);
		}
	}

	for (var fieldId in this._searchField) {
		var field = this._searchField[fieldId];
		field.disabled = false;
		if (str) {
			field.className = "";
			field.value = AjxUtil.isObject(str) ? (str[fieldId] || "") : str;
			this._searchCleared[fieldId] = true;
		} else {
			field.className = "searchFieldHint";
			field.value = ZmContactPicker.HINT[fieldId];
			this._searchCleared[fieldId] = false;
		}
	}
	var focusField = this._searchField[ZmContactPicker.SEARCH_BASIC] || this._searchField[ZmContactPicker.SEARCH_NAME];
	focusField.focus();

	// reset paging buttons
	this._prevButton.setEnabled(false);
	this._nextButton.setEnabled(false);

	this.search(null, null, true);

	DwtDialog.prototype.popup.call(this);
	if ((this.getLocation().x < 0 ||  this.getLocation().y < 0) ){
		// parent window size is smaller than Dialog size
		this.setLocation(0,30);
		var size = Dwt.getWindowSize();
		var currentSize = this.getSize();
		var dragElement = document.getElementById(this._dragHandleId);
		DwtDraggable.setDragBoundaries(dragElement, 100 - currentSize.x, size.x - 100, 0, size.y - 100);
	}
};

/**
 * Closes the dialog.
 * 
 */
ZmContactPicker.prototype.popdown =
function() {
	// disable search field (hack to fix bleeding cursor)

	for (var fieldId in this._searchField) {
		this._searchField[fieldId].disabled = true;
	}

	this._contactSource = null;
	this._list.removeAll();

	DwtDialog.prototype.popdown.call(this);
};

/**
 * Performs a search.
 * 
 * @private
 */
ZmContactPicker.prototype.search =
function(colItem, ascending, firstTime, lastId, lastSortVal) {
	if (!AjxUtil.isSpecified(ascending)) {
		ascending = true;
	}

	var query;
	var queryHint = [];
	var emailQueryTerm = "";
	var phoneticQueryTerms = [];  // Should be ORed with nameQuery if both are present
	var conds = [];
	if (this._detailed) {
		var nameQuery = this.getSearchFieldValue(ZmContactPicker.SEARCH_NAME);
		var emailQuery = this.getSearchFieldValue(ZmContactPicker.SEARCH_EMAIL);
		var deptQuery = this.getSearchFieldValue(ZmContactPicker.SEARCH_DEPT);
		var phoneticQuery = this.getSearchFieldValue(ZmContactPicker.SEARCH_PHONETIC);
		var isGal = this._selectDiv && (this._selectDiv.getValue() == ZmContactsApp.SEARCHFOR_GAL);
		query = nameQuery || "";
		if (emailQuery) {
			if (isGal) {
				conds.push([{attr:ZmContact.F_email, op:"has", value: emailQuery},
				{attr:ZmContact.F_email2, op:"has", value: emailQuery},
				{attr:ZmContact.F_email3, op:"has", value: emailQuery},
				{attr:ZmContact.F_email4, op:"has", value: emailQuery},
				{attr:ZmContact.F_email5, op:"has", value: emailQuery},
				{attr:ZmContact.F_email6, op:"has", value: emailQuery},
				{attr:ZmContact.F_email7, op:"has", value: emailQuery},
				{attr:ZmContact.F_email8, op:"has", value: emailQuery},
				{attr:ZmContact.F_email9, op:"has", value: emailQuery},
				{attr:ZmContact.F_email10, op:"has", value: emailQuery},
				{attr:ZmContact.F_email11, op:"has", value: emailQuery},
				{attr:ZmContact.F_email12, op:"has", value: emailQuery},
				{attr:ZmContact.F_email13, op:"has", value: emailQuery},
				{attr:ZmContact.F_email14, op:"has", value: emailQuery},
				{attr:ZmContact.F_email15, op:"has", value: emailQuery},
				{attr:ZmContact.F_email16, op:"has", value: emailQuery}
				]);
			} else {
				emailQueryTerm = "to:"+emailQuery+"*";
			}
		}
		if (deptQuery && isGal) {
			conds.push({attr:ZmContact.F_department, op:"has", value: deptQuery});
		}
		if (phoneticQuery && !isGal) {
			var phoneticQueryPieces = phoneticQuery.split(/\s+/);
			for (var i=0; i<phoneticQueryPieces.length; i++) {
				phoneticQueryTerms.push("#"+ZmContact.F_phoneticFirstName + ":" + phoneticQueryPieces[i]);
				phoneticQueryTerms.push("#"+ZmContact.F_phoneticLastName + ":" + phoneticQueryPieces[i]);
			}
		}
	} else {
		query = this.getSearchFieldValue(ZmContactPicker.SEARCH_BASIC);
	}
	

	if (this._selectDiv) {
		var searchFor = this._selectDiv.getValue();
		this._contactSource = (searchFor == ZmContactsApp.SEARCHFOR_CONTACTS || searchFor == ZmContactsApp.SEARCHFOR_PAS)
			? ZmItem.CONTACT
			: ZmId.SEARCH_GAL;

		if (searchFor == ZmContactsApp.SEARCHFOR_PAS) {
			queryHint.push(ZmSearchController.generateQueryForShares([ZmId.ITEM_CONTACT]) || "is:local");
		} else if (searchFor == ZmContactsApp.SEARCHFOR_CONTACTS) {
			queryHint.push("is:local");
		} else if (searchFor == ZmContactsApp.SEARCHFOR_GAL) {
            ascending = true;
        }
	} else {
		this._contactSource = appCtxt.get(ZmSetting.CONTACTS_ENABLED, null, this._account)
			? ZmItem.CONTACT
			: ZmId.SEARCH_GAL;

		if (this._contactSource == ZmItem.CONTACT) {
			queryHint.push("is:local");
		}
	}

    if (!query.length && this._contactSource == ZmId.SEARCH_GAL) {
		query = this._defaultQuery;
	}

    if (this._contactSource == ZmItem.CONTACT && query != "") {
        query = query.replace(/\"/g, '\\"');
        query = "\"" + query + "\"";
    }
	if (phoneticQueryTerms.length) {
		// OR any phonetic query terms with the main (name) query
		if (query.length) {
			phoneticQueryTerms.unshift(query);
		}
		query = "(" + phoneticQueryTerms.join(" OR ") + ")";
	}
	if (emailQueryTerm.length) {
		query = query + " " + emailQueryTerm;  // MUST match email term, hence AND rather than OR
	}

	this._searchIcon.className = "DwtWait16Icon";

	// XXX: line below doesn't have intended effect (turn off column sorting for GAL search)
	this._chooser.sourceListView.sortingEnabled = (this._contactSource == ZmItem.CONTACT);

	var params = {
		obj:			this,
		ascending:		ascending,
		query:			query,
		queryHint:		queryHint.join(" "),
		conds:			conds,
		offset:			this._list.size(),
		lastId:			lastId,
		lastSortVal:	lastSortVal,
		respCallback:	(new AjxCallback(this, this._handleResponseSearch, [firstTime])),
		errorCallback:	this._searchErrorCallback,
		accountName:	(this._account && this._account.name),
		expandDL:		true
	};
	ZmContactsHelper.search(params);
};

/**
 * @private
 */
ZmContactPicker.prototype._contentHtml =
function(account) {
	var showSelect;
	var context = appCtxt.isChildWindow ? parentAppCtxt : appCtxt;

	if (context.multiAccounts) {
		var list = context.accountList.visibleAccounts;
		for (var i = 0; i < list.length; i++) {
			var account = list[i];
			if (context.get(ZmSetting.CONTACTS_ENABLED, null, account) &&
				(context.get(ZmSetting.GAL_ENABLED, null, account) ||
				 context.get(ZmSetting.SHARING_ENABLED, null, account)))
			{
				showSelect = true;
				break;
			}
		}
	} else {
		showSelect = (context.get(ZmSetting.CONTACTS_ENABLED) &&
					  (context.get(ZmSetting.GAL_ENABLED) ||
					   context.get(ZmSetting.SHARING_ENABLED)));
	}

	var subs = {
		id: this._htmlElId,
		showSelect: showSelect,
		detailed: this._detailed
	};

	return (AjxTemplate.expand("abook.Contacts#ZmContactPicker", subs));
};

/**
 * @private
 */
ZmContactPicker.prototype._resetSelectDiv =
function() {
	this._selectDiv.clearOptions();

	if (appCtxt.get(ZmSetting.CONTACTS_ENABLED, null, this._account)) {
		this._selectDiv.addOption(ZmMsg.contacts, false, ZmContactsApp.SEARCHFOR_CONTACTS);

		if (appCtxt.get(ZmSetting.SHARING_ENABLED, null, this._account))
			this._selectDiv.addOption(ZmMsg.searchPersonalSharedContacts, false, ZmContactsApp.SEARCHFOR_PAS);
	}

	if (appCtxt.get(ZmSetting.GAL_ENABLED, null, this._account)) {
		this._selectDiv.addOption(ZmMsg.GAL, true, ZmContactsApp.SEARCHFOR_GAL);
	}

	if (!appCtxt.get(ZmSetting.INITIALLY_SEARCH_GAL, null, this._account) ||
		!appCtxt.get(ZmSetting.GAL_ENABLED, null, this._account))
	{
		this._selectDiv.setSelectedValue(ZmContactsApp.SEARCHFOR_CONTACTS);
	}
};

ZmContactPicker.prototype.clearSearch =
function(el) {
	if (el) {
		for (var fieldId in this._searchField) {
			if (el == this._searchField[fieldId]) {
				if (!this._searchCleared[fieldId]) {
					el.className = el.value = "";
					this._searchCleared[fieldId] = true;
				}
				break;
			}
		}
	}
};

ZmContactPicker.prototype.getSearchFieldValue =
function(fieldId) {
	if (!fieldId && !this._detailed)
		fieldId = ZmContactPicker.SEARCH_BASIC;
	var field = this._searchField[fieldId];
	return (this._searchCleared[fieldId] && field) ? AjxStringUtil.trim(field.value) : "";
};

/**
 * called only when ZmContactPicker is first created. Sets up initial layout.
 * 
 * @private
 */
ZmContactPicker.prototype._initialize =
function(account) {

	// create static content and append to dialog parent
	this.setContent(this._contentHtml(account));

	this._searchIcon = document.getElementById(this._htmlElId + "_searchIcon");

	// add search button
	this._searchButton = new DwtButton({parent:this, parentElement:(this._htmlElId+"_searchButton")});
	this._searchButton.setText(ZmMsg.search);
	this._searchButton.addSelectionListener(new AjxListener(this, this._searchButtonListener));

	// add select menu
	var selectCellId = this._htmlElId + "_listSelect";
	var selectCell = document.getElementById(selectCellId);
	if (selectCell) {
		this._selectDiv = new DwtSelect({parent:this, parentElement:selectCellId, id: Dwt.getNextId("ZmContactPickerSelect_")});
		this._resetSelectDiv();
		this._selectDiv.addChangeListener(new AjxListener(this, this._searchTypeListener));
	} else {
		this.setSize("600");
	}

	// add chooser
	this._chooser = new ZmContactChooser({parent:this, buttonInfo:this._buttonInfo});
	this._chooser.reparentHtmlElement(this._htmlElId + "_chooser");
	this._chooser.resize(this.getSize().x-25, ZmContactPicker.CHOOSER_HEIGHT);

	// add paging buttons
	var pageListener = new AjxListener(this, this._pageListener);
	this._prevButton = new DwtButton({parent:this, parentElement:(this._htmlElId+"_pageLeft")});
	this._prevButton.setText(ZmMsg.previous);
	this._prevButton.setImage("LeftArrow");
	this._prevButton.addSelectionListener(pageListener);

	this._nextButton = new DwtButton({parent:this, style:DwtLabel.IMAGE_RIGHT, parentElement:(this._htmlElId+"_pageRight")});
	this._nextButton.setText(ZmMsg.next);
	this._nextButton.setImage("RightArrow");
	this._nextButton.addSelectionListener(pageListener);

	var pageContainer = document.getElementById(this._htmlElId + "_paging");
	if (pageContainer) {
		Dwt.setSize(pageContainer, this._chooser.sourceListView.getSize().x);
	}

	// init listeners
	this.setButtonListener(DwtDialog.OK_BUTTON, new AjxListener(this, this._okButtonListener));
	this.setButtonListener(DwtDialog.CANCEL_BUTTON, new AjxListener(this, this._cancelButtonListener));

	this._searchField = {};
	this._searchRow = {};
	var fieldMap = {}, field, rowMap = {};
	if (this._detailed) {
		fieldMap[ZmContactPicker.SEARCH_NAME] = this._htmlElId + "_searchNameField";
		fieldMap[ZmContactPicker.SEARCH_EMAIL] = this._htmlElId + "_searchEmailField";
		fieldMap[ZmContactPicker.SEARCH_DEPT] = this._htmlElId + "_searchDepartmentField";
		fieldMap[ZmContactPicker.SEARCH_PHONETIC] = this._htmlElId + "_searchPhoneticField";
		rowMap[ZmContactPicker.SEARCH_NAME] = this._htmlElId + "_searchNameRow";
		rowMap[ZmContactPicker.SEARCH_PHONETIC] = this._htmlElId + "_searchPhoneticRow";
		rowMap[ZmContactPicker.SEARCH_EMAIL] = this._htmlElId + "_searchEmailRow";
		rowMap[ZmContactPicker.SEARCH_DEPT] = this._htmlElId + "_searchDepartmentRow";
	} else {
		fieldMap[ZmContactPicker.SEARCH_BASIC] = this._htmlElId + "_searchField";
		rowMap[ZmContactPicker.SEARCH_BASIC] = this._htmlElId + "_searchRow";
	}
	for (var fieldId in fieldMap) {
		field = Dwt.byId(fieldMap[fieldId]);
		if (field) {
			this._searchField[fieldId] = field;
			Dwt.setHandler(field, DwtEvent.ONKEYUP, ZmContactPicker._keyPressHdlr);
			Dwt.setHandler(field, DwtEvent.ONCLICK, ZmContactPicker._onclickHdlr);
		}
	}
	for (var rowId in rowMap) {
		row = Dwt.byId(rowMap[rowId]);
		if (row) {
			this._searchRow[rowId] = row;
		}
	}
	this._updateSearchRows(this._selectDiv && this._selectDiv.getValue() || ZmContactsApp.SEARCHFOR_CONTACTS);
	this._keyPressCallback = new AjxCallback(this, this._searchButtonListener);
};

// Listeners

/**
 * @private
 */
ZmContactPicker.prototype._searchButtonListener =
function(ev) {
	this._offset = 0;
	this._list.removeAll();
	this.search();
};

/**
 * @private
 */
ZmContactPicker.prototype._handleResponseSearch =
function(firstTime, result) {
	var resp = result.getResponse();
	var more = resp.getAttribute("more");
	var offset = resp.getAttribute("offset");
	var isPagingSupported = AjxUtil.isSpecified(offset);
	var info = resp.getAttribute("info");
	var expanded = info && info[0].wildcard[0].expanded == "0";

	if (!firstTime && !isPagingSupported &&
		(expanded || (this._contactSource == ZmId.SEARCH_GAL && more)))
	{
		var d = appCtxt.getMsgDialog();
		d.setMessage(ZmMsg.errorSearchNotExpanded);
		d.popup();
		if (expanded) { return; }
	}

	// this method will expand the list depending on the number of email
	// addresses per contact.
	var list = AjxVector.fromArray(ZmContactsHelper._processSearchResponse(resp));

	if (isPagingSupported) {
		this._list.merge(offset, list);
		this._list.hasMore = more;
	} else {
		this._list = list;
	}

	if (list.size() == 0 && firstTime) {
		this._chooser.sourceListView._setNoResultsHtml();
	}

	// if we don't get a full number of addresses in the results, repeat the search.
	// Could search several times.
	if (((this._list.size() - this._offset) < ZmContactsApp.SEARCHFOR_MAX) && more) {
		// We want to base our search off the ID of the last contact in the response,
		// NOT the last contact with an email address
		var vec = resp.getResults(ZmItem.CONTACT);

		var email = (vec.size() > 0) ? vec.getVector().getLast() : null;
		if (email) {
			if (email.__contact) {
				lastId = email.__contact.id;
				lastSortVal = email.__contact.sf;
			} else {
				lastId = email.id;
				lastSortVal = email.sf;
			}
		}
        if (!lastSortVal && isPagingSupported) {
			// BAIL. Server didn't send us enough info to make the next request
			this._searchIcon.className = "ImgSearch";
			return;
		}
        else if (!lastSortVal && !isPagingSupported) {
            //paging not supported, show what we have
            this._showResults(isPagingSupported, more, this.getSubList());
        }
        else {
            this.search(null, null, null, lastId, lastSortVal);
        }
	}
	else {
		list = this.getSubList();
		// If the AB ends with a long list of contacts w/o addresses,
		// we may never get a list back.  If that's the case, roll back the offset
		// and refetch, should disable the "next page" button.
		if (!list) {
			this._offset -= ZmContactsApp.SEARCHFOR_MAX;
			if (this._offset < 0) {
				this._offset = 0;
			}
			list = this.getSubList();
		}
		if (!more) {
			more = (this._offset+ZmContactsApp.SEARCHFOR_MAX) < this._list.size();
		}
		this._showResults(isPagingSupported, more, list);
	}

};

ZmContactPicker.prototype._showResults =
function(isPagingSupported, more, list) {
	// if offset is returned, then this account support gal paging
	if (this._contactSource == ZmId.SEARCH_GAL && !isPagingSupported) {
		this._prevButton.setEnabled(false);
		this._nextButton.setEnabled(false);
	} else {
		this._prevButton.setEnabled(this._offset > 0);
		this._nextButton.setEnabled(more);
	}

	this._resetColHeaders(); // bug #2269 - enable/disable sort column per type of search
	this._chooser.setItems(list);

	this._searchIcon.className = "ImgSearch";
	this._searchButton.setEnabled(true);
};

ZmContactPicker.prototype._updateSearchRows =
function(searchFor) {
	var fieldIds = (searchFor == ZmContactsApp.SEARCHFOR_GAL) ? ZmContactPicker.SHOW_ON_GAL : ZmContactPicker.SHOW_ON_NONGAL;
	for (var fieldId in this._searchRow) {
		Dwt.setVisible(this._searchRow[fieldId], AjxUtil.indexOf(fieldIds, fieldId)!=-1);
	}
	for (var fieldId in this._searchField) {
		var field = this._searchField[fieldId];
		if (this._tabGroup.contains(field))
			this._tabGroup.removeMember(field);
	}
	for (var i=0; i<fieldIds.length; i++) {
		this._tabGroup.addMember(this._searchField[fieldIds[i]]);
	}
};

/**
 * @private
 */
ZmContactPicker.prototype._handleErrorSearch =
function() {
	this._searchButton.setEnabled(true);
	return false;
};

/**
 * @private
 */
ZmContactPicker.prototype._pageListener =
function(ev) {
	if (ev.item == this._prevButton) {
		this._offset -= ZmContactsApp.SEARCHFOR_MAX;
		if (this._offset < 0) {
			this._offset = 0;
		}
		this._showResults(true, true, this.getSubList()); // show cached results
	}
	else {
		var lastId;
		var lastSortVal;
		this._offset += ZmContactsApp.SEARCHFOR_MAX;
		var list = this.getSubList();
		if (!list || ((list.size() < ZmContactsApp.SEARCHFOR_MAX) && this._list.hasMore)) {
			if (!list) {
				list = this._chooser.sourceListView.getList();
			}
			var email = (list.size() > 0) ? list.getLast() : null;
			if (email) {
				lastId = email.__contact.id;
				lastSortVal = email.__contact.sf;
			}
			this.search(null, null, null, lastId, lastSortVal);
		} else {
			var more = this._list.hasMore;
			if (!more) {
				more = (this._offset+ZmContactsApp.SEARCHFOR_MAX) < this._list.size();
			}
			this._showResults(true, more, list); // show cached results
		}
	}
};

/**
 * Gets a sub-list of contacts.
 * 
 * @return	{AjxVector}		a vector of {ZmContact} objects
 */
ZmContactPicker.prototype.getSubList =
function() {
	var size = this._list.size();

	var end = this._offset+ZmContactsApp.SEARCHFOR_MAX;

	if (end > size) {
		end = size;
	}

	return (this._offset < end)
		? (AjxVector.fromArray(this._list.getArray().slice(this._offset, end))) : null;
};

/**
 * @private
 */
ZmContactPicker.prototype._searchTypeListener =
function(ev) {
	var oldValue = ev._args.oldValue;
	var newValue = ev._args.newValue;

	if (oldValue != newValue) {
		this._updateSearchRows(newValue);
		this._searchButtonListener();
	}
};

/**
 * @private
 */
ZmContactPicker.prototype._resetColHeaders =
function () {
    var slv = this._chooser.sourceListView;
    var tlv = this._chooser.targetListView;
    slv.headerColCreated = false;
    tlv.headerColCreated = false;
    var isGal = this._selectDiv && (this._selectDiv.getValue() == ZmContactsApp.SEARCHFOR_GAL);

    // find the participant column
    var part = 0;
    for (var i = 0; i < slv._headerList.length; i++) {
        var field = slv._headerList[i]._field;
        if (field == ZmItem.F_NAME) {
            part = i;
        }
        if (field == ZmItem.F_DEPARTMENT) {
            slv._headerList[i]._visible = isGal && this._detailed;
        }
    }

    var sortable = isGal ? null : ZmItem.F_NAME;
    slv._headerList[part]._sortable = sortable;
    slv.createHeaderHtml(sortable);

    for (i = 0; i < tlv._headerList.length; i++) {
        if (tlv._headerList[i]._field == ZmItem.F_DEPARTMENT) {
            tlv._headerList[i]._visible = isGal && this._detailed;
        }
    }
    tlv.createHeaderHtml();
};

/**
 * Done choosing addresses, add them to the compose form.
 * 
 * @private
 */
ZmContactPicker.prototype._okButtonListener =
function(ev) {
	var data = this._chooser.getItems();
	DwtDialog.prototype._buttonListener.call(this, ev, [data]);
};

/**
 * Call custom popdown method.
 * 
 * @private
 */
ZmContactPicker.prototype._cancelButtonListener =
function(ev) {
	DwtDialog.prototype._buttonListener.call(this, ev);
	this.popdown();
};

/**
 * @private
 */
ZmContactPicker._keyPressHdlr =
function(ev) {
	var stb = DwtControl.getTargetControl(ev);
	var charCode = DwtKeyEvent.getCharCode(ev);
	stb.clearSearch(DwtUiEvent.getTarget(ev));
	if (stb._keyPressCallback && (charCode == 13 || charCode == 3)) {
		stb._keyPressCallback.run();
		return false;
	}
	return true;
};

/**
 * @private
 */
ZmContactPicker._onclickHdlr =
function(ev) {
	var stb = DwtControl.getTargetControl(ev);
	stb.clearSearch(DwtUiEvent.getTarget(ev));
};

/***********************************************************************************/

/**
 * Creates a contact chooser.
 * @class
 * This class creates a specialized chooser for the contact picker.
 *
 * @param {DwtComposite}	parent			the contact picker
 * @param {Array}		buttonInfo		transfer button IDs and labels
 * 
 * @extends		DwtChooser
 * 
 * @private
 */
ZmContactChooser = function(params) {
	DwtChooser.call(this, params);
};

ZmContactChooser.prototype = new DwtChooser;
ZmContactChooser.prototype.constructor = ZmContactChooser;

/**
 * @private
 */
ZmContactChooser.prototype._createSourceListView =
function() {
	return new ZmContactChooserSourceListView(this);
};

/**
 * @private
 */
ZmContactChooser.prototype._createTargetListView =
function() {
	return new ZmContactChooserTargetListView(this, (this._buttonInfo.length > 1));
};

/**
 * The item is a AjxEmailAddress. Its address is used for comparison.
 *
 * @param {AjxEmailAddress}	item	an email address
 * @param {AjxVector}	list	list to check in
 * 
 * @private
 */
ZmContactChooser.prototype._isDuplicate =
function(item, list) {
	return list.containsLike(item, item.getAddress);
};

/***********************************************************************************/

/**
 * Creates a source list view.
 * @class
 * This class creates a specialized source list view for the contact chooser.
 * 
 * @param {DwtComposite}	parent			the contact picker
 * 
 * @extends		DwtChooserListView
 * 
 * @private
 */
ZmContactChooserSourceListView = function(parent) {
	DwtChooserListView.call(this, {parent:parent, type:DwtChooserListView.SOURCE, view:ZmId.VIEW_CONTACT_SRC});
	this.setScrollStyle(Dwt.CLIP);
};

ZmContactChooserSourceListView.prototype = new DwtChooserListView;
ZmContactChooserSourceListView.prototype.constructor = ZmContactChooserSourceListView;

/**
 * Returns a string representation of the object.
 * 
 * @return		{String}		a string representation of the object
 * @private
 */
ZmContactChooserSourceListView.prototype.toString =
function() {
	return "ZmContactChooserSourceListView";
};

ZmContactChooserSourceListView.prototype.getToolTipContent =
function(ev) {
	
	if (this._hoveredItem) {
		var ttParams = {
			address:		this._hoveredItem.address,
			contact:		this._hoveredItem.__contact,
			ev:				ev,
			noRightClick:	true
		};
		var ttCallback = new AjxCallback(this,
			function(callback) {
				appCtxt.getToolTipMgr().getToolTip(ZmToolTipMgr.PERSON, ttParams, callback);
			});
		return {callback:ttCallback};
	}
	else {
		return "";
	}
};

/**
 * @private
 */
ZmContactChooserSourceListView.prototype._getHeaderList =
function() {
	var headerList = [];
	headerList.push(new DwtListHeaderItem({field:ZmItem.F_TYPE, icon:"Folder", width:ZmMsg.COLUMN_WIDTH_FOLDER_CN}));
	headerList.push(new DwtListHeaderItem({field:ZmItem.F_NAME, text:ZmMsg._name, width:ZmMsg.COLUMN_WIDTH_NAME_CN, resizeable: true}));
	headerList.push(new DwtListHeaderItem({field:ZmItem.F_DEPARTMENT, text:ZmMsg.department, width:ZmMsg.COLUMN_WIDTH_DEPARTMENT_CN, resizeable: true}));
	headerList.push(new DwtListHeaderItem({field:ZmItem.F_EMAIL, text:ZmMsg.email, resizeable: true}));


	return headerList;
};

 
// Override of DwtListView.prototype._resetColWidth to set width; without overrriding causes vertical scrollbars to disapper
// on header resize
ZmContactChooserSourceListView.prototype._resetColWidth =
function() {

	if (!this.headerColCreated) { return; }

	var lastColIdx = this._getLastColumnIndex();
    if (lastColIdx) {
        var lastCol = this._headerList[lastColIdx];
        var lastCell = document.getElementById(lastCol._id);
		if (lastCell) {
			var div = lastCell.firstChild;
			lastCell.style.width = div.style.width = (lastCol._width || ""); 
		}
    }
};

/**
 * override for scrollbars in IE
 * @param headerIdx
 */
ZmContactChooserSourceListView.prototype._calcRelativeWidth =
function(headerIdx) {
	var column = this._headerList[headerIdx];
	if (!column._width || (column._width && column._width == "auto")) {
		var cell = document.getElementById(column._id);
		// UGH: clientWidth is 5px more than HTML-width (20px for IE to deal with scrollbars)
		return (cell) ? (cell.clientWidth - (AjxEnv.isIE ? Dwt.SCROLLBAR_WIDTH : 5)) : null;
	}
	return column._width;
};

/**
 * @private
 */
ZmContactChooserSourceListView.prototype._mouseOverAction =
function(ev, div) {
	DwtChooserListView.prototype._mouseOverAction.call(this, ev, div);
	var id = ev.target.id || div.id;
	var item = this.getItemFromElement(div);
	this._hoveredItem = (id && item) ? item : null;
	return true;
};

/**
 * @private
 */
ZmContactChooserSourceListView.prototype._getCellContents =
function(html, idx, item, field, colIdx, params) {
	if (field == ZmItem.F_EMAIL && AjxEnv.isIE) {
		var maxWidth = AjxStringUtil.getWidth(item.address);
		html[idx++] = "<div style='float; left; overflow: visible; width: " + maxWidth + ";'>";
		idx = ZmContactsHelper._getEmailField(html, idx, item, field, colIdx, params);
		html[idx++] = "</div>";		
	}
	else {
		idx = ZmContactsHelper._getEmailField(html, idx, item, field, colIdx, params);
	}
	return idx;
};

/**
 * Returns a string of any extra attributes to be used for the TD.
 *
 * @param item		[object]	item to render
 * @param field		[constant]	column identifier
 * @param params	[hash]*		hash of optional params
 * 
 * @private
 */
ZmContactChooserSourceListView.prototype._getCellAttrText =
function(item, field, params) {
	if (field == ZmItem.F_EMAIL) {
		return "style='position: relative; overflow: visible;'";
	}
};

/***********************************************************************************/

/**
 * Creates the target list view.
 * @class
 * This class creates a specialized target list view for the contact chooser.
 * 
 * @param {DwtComposite}	parent			the contact picker
 * @param {constant}		showType		the show type
 * @extends		DwtChooserListView
 * 
 * @private
 */
ZmContactChooserTargetListView = function(parent, showType) {
	this._showType = showType; // call before base class since base calls getHeaderList

	DwtChooserListView.call(this, {parent:parent, type:DwtChooserListView.TARGET,
								   view:ZmId.VIEW_CONTACT_TGT});

	this.setScrollStyle(Dwt.CLIP);
};

ZmContactChooserTargetListView.prototype = new DwtChooserListView;
ZmContactChooserTargetListView.prototype.constructor = ZmContactChooserTargetListView;

/**
 * Returns a string representation of the object.
 * 
 * @return		{String}		a string representation of the object
 */
ZmContactChooserTargetListView.prototype.toString =
function() {
	return "ZmContactChooserTargetListView";
};

/**
 * @private
 */
ZmContactChooserTargetListView.prototype._getHeaderList =
function() {
	var headerList = [];
	var view = this._view;
	if (this._showType) {
		headerList.push(new DwtListHeaderItem({field:ZmItem.F_TYPE, icon:"ContactsPicker", width:ZmMsg.COLUMN_WIDTH_TYPE_CN}));
	}
	headerList.push(new DwtListHeaderItem({field:ZmItem.F_NAME, text:ZmMsg._name, width:ZmMsg.COLUMN_WIDTH_NAME_CN, resizeable: true}));
    headerList.push(new DwtListHeaderItem({field:ZmItem.F_DEPARTMENT, text:ZmMsg.department, width:ZmMsg.COLUMN_WIDTH_DEPARTMENT_CN, resizeable: true}));
    headerList.push(new DwtListHeaderItem({field:ZmItem.F_EMAIL, text:ZmMsg.email, resizeable: true}));

	return headerList;
};

ZmContactChooserTargetListView.prototype._mouseOverAction =
ZmContactChooserSourceListView.prototype._mouseOverAction;

/**
 * The items are AjxEmailAddress objects.
 * 
 * @private
 */
ZmContactChooserTargetListView.prototype._getCellContents =
function(html, idx, item, field, colIdx, params) {
	if (field == ZmItem.F_TYPE) {
		item.setType(item._buttonId);
		html[idx++] = ZmMsg[item.getTypeAsString()];
		html[idx++] = ":";
	}
	else if (field == ZmItem.F_EMAIL && AjxEnv.isIE) {
		var maxWidth = AjxStringUtil.getWidth(item.address) + 10;
		html[idx++] = "<div style='float; left;  width: " + maxWidth + ";'>";
		idx = ZmContactsHelper._getEmailField(html, idx, item, field, colIdx, params);
		html[idx++] = "</div>";
	}
	else {
		idx = ZmContactsHelper._getEmailField(html, idx, item, field, colIdx);
	}
	return idx;
};


// Override of DwtListView.prototype._resetColWidth to set width; without overrriding causes vertical scrollbars to disapper
// on header resize
ZmContactChooserTargetListView.prototype._resetColWidth =
function() {

	if (!this.headerColCreated) { return; }

	var lastColIdx = this._getLastColumnIndex();

	
    if (lastColIdx) {
        var lastCol = this._headerList[lastColIdx];
        var lastCell = document.getElementById(lastCol._id);
		if (lastCell) {
			var div = lastCell.firstChild;
			lastCell.style.width = div.style.width = (lastCol._width || "");
		}
    }
};

/**
 * override for scrollbars in IE
 * @param headerIdx
 */
ZmContactChooserTargetListView.prototype._calcRelativeWidth =
function(headerIdx) {
	var column = this._headerList[headerIdx];
	if (!column._width || (column._width && column._width == "auto")) {
		var cell = document.getElementById(column._id);
		// UGH: clientWidth is 5px more than HTML-width (20px for IE to deal with scrollbars)
		return (cell) ? (cell.clientWidth - (AjxEnv.isIE ? Dwt.SCROLLBAR_WIDTH : 5)) : null;
	}
	return column._width;
};

/**
 * Returns a string of any extra attributes to be used for the TD.
 *
 * @param item		[object]	item to render
 * @param field		[constant]	column identifier
 * @param params	[hash]*		hash of optional params
 * 
 * @private
 */
ZmContactChooserTargetListView.prototype._getCellAttrText =
function(item, field, params) {
	if (field == ZmItem.F_EMAIL) {
		return "style='position: relative; overflow: visible;'";
	}
};
}
}
if (AjxPackage.define("abook.Contacts")) {
AjxTemplate.register("abook.Contacts#ZmEditContactView", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_DEBUG\"></div><table valign=top width='100%' cellspacing=0>";
	buffer[_i++] =  AjxTemplate.expand("#ZmEditContactView_header", data) ;
	buffer[_i++] =  AjxTemplate.expand("#ZmEditContactView_body", data) ;
	buffer[_i++] = "</table>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "abook.Contacts#ZmEditContactView"
}, false);
AjxTemplate.register("abook.Contacts", AjxTemplate.getTemplate("abook.Contacts#ZmEditContactView"), AjxTemplate.getParams("abook.Contacts#ZmEditContactView"));

AjxTemplate.register("abook.Contacts#ZmEditContactView_header", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<tr valign=top class=contactHeaderTable><td class=contactHeaderCell><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_IMAGE\" tabindex=\"100\"></div><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_REMOVE_IMAGE_row\" style='padding-left:7px;font-size:.8em'><nobr><a id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_VIEW_IMAGE\" href=\"#view\" tabindex=\"101\">";
	buffer[_i++] = ZmMsg.view;
	buffer[_i++] = "</a>\n";
	buffer[_i++] = "                    |\n";
	buffer[_i++] = "                    <a id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_REMOVE_IMAGE\" href=\"#remove\" tabindex=\"102\">";
	buffer[_i++] = ZmMsg.remove;
	buffer[_i++] = "</a></nobr></div></td><td class=\"contactHeaderCell\" valign=\"bottom\"><div style='padding:.125em'><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_FULLNAME\"></div></div>";
	 if (appCtxt.get(ZmSetting.TAGGING_ENABLED)) { 
	buffer[_i++] = "<div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_TAG\"></div>";
	 } 
	buffer[_i++] =  AjxTemplate.expand("#ZmEditContactView_headerName", data) ;
	buffer[_i++] =  AjxTemplate.expand("#ZmEditContactView_headerJobInfo", data) ;
	buffer[_i++] = "</td><td class=contactHeaderCell><table><tr><td>";
	buffer[_i++] = ZmMsg.fileAsLabel;
	buffer[_i++] = "</td><td><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_FILE_AS\" tabindex=\"400\"></div></td></tr><tr><td>";
	buffer[_i++] = ZmMsg.locationLabel;
	buffer[_i++] = "</td><td><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_FOLDER\" tabindex=\"401\"></div></td></tr>";
	 if (appCtxt.multiAccounts) { 
	buffer[_i++] = "<tr><td>";
	buffer[_i++] = ZmMsg.accountLabel;
	buffer[_i++] = "</td><td><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_ACCOUNT\" tabindex=\"402\"></div></td></tr>";
	 } 
	buffer[_i++] = "</table></td></tr>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "abook.Contacts#ZmEditContactView_header"
}, false);

AjxTemplate.register("abook.Contacts#ZmEditContactView_headerName", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_NAME_row\" style='padding-bottom:4;'><table border=0 cellpadding=0 cellspacing=0>";
	 if (appCtxt.get(ZmSetting.PHONETIC_CONTACT_FIELDS)) { 
	buffer[_i++] = "<tr id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_PHONETIC_row\"><td id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_PHONETIC_PREFIX\"></td><td id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_PHONETIC_FIRST_row\"><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_PHONETIC_FIRST\" tabindex=\"190\"></div></td><td id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_PHONETIC_MIDDLE\"></td><td id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_PHONETIC_MAIDEN\"></td><td id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_PHONETIC_LAST_row\"><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_PHONETIC_LAST\" tabindex=\"191\"></div></td><td id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_PHONETIC_SUFFIX\"></td></tr>";
	 } 
	buffer[_i++] = "<tr><td id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_PREFIX_row\" style='padding-right:4;'><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_PREFIX\" tabindex=\"200\"></div></td><td style='padding-right:4;'><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_FIRST\" tabindex=\"201\"></div></td><td id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_MIDDLE_row\" style='padding-right:4;'><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_MIDDLE\" tabindex=\"202\"></div></td><td id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_MAIDEN_row\" style='padding-right:4;'><table border=0 cellspacing=0 cellpadding=0><td style='padding:0 2;'>(</td><td><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_MAIDEN\" tabindex=\"203\"></div></td><td style='padding:0 2;'>)</td></table></td><td style='padding-right:4;'><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_LAST\" tabindex=\"204\"></div></td><td id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_SUFFIX_row\" style='padding-right:4;'><table border=0 cellspacing=0 cellpadding=0><td style='padding-right:4;'>,</td><td><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_SUFFIX\" tabindex=\"205\"></div></td></table></td><td><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_DETAILS\" tabindex=\"206\"></div></td></tr></table></div><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_NICKNAME_row\" style='padding-bottom:4;'><table border=0 cellpadding=0 cellspacing=0><tr><td>&ldquo;</td><td><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_NICKNAME\" tabindex=\"250\"></div></td><td>&rdquo;</td></tr></table></div>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "abook.Contacts#ZmEditContactView_headerName"
}, false);

AjxTemplate.register("abook.Contacts#ZmEditContactView_headerJobInfo", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_JOB_row\" style='padding-bottom:4;'><table border=0 cellpadding=0 cellspacing=0><tr><td id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_TITLE_row\"><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_TITLE\" tabindex=\"301\"></div></td><td id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_TITLE_DEPARTMENT_SEP\">&nbsp;-&nbsp;</td><td id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_DEPARTMENT_row\"><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_DEPARTMENT\" tabindex=\"302\"></div></td></tr></table></div><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_COMPANY_row\" style='padding-bottom:4;'><table border=0 cellpadding=0 cellspacing=0>";
	 if (appCtxt.get(ZmSetting.PHONETIC_CONTACT_FIELDS)) { 
	buffer[_i++] = "<tr id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_PHONETIC_COMPANY_row\"><td><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_PHONETIC_COMPANY\" tabindex=\"304\"></div></td></tr>";
	 } 
	buffer[_i++] = "<tr><td><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_COMPANY\" tabindex=\"305\"></div></td></tr></table></div>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "abook.Contacts#ZmEditContactView_headerJobInfo"
}, false);

AjxTemplate.register("abook.Contacts#ZmEditContactView_body", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<tr valign=top><td class=rowLabel>";
	buffer[_i++] = ZmMsg.emailLabel;
	buffer[_i++] = "</td><td class=rowValue><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_EMAIL\" tabindex=\"500\"></div></td></tr><tr valign=top><td class=rowLabel>";
	buffer[_i++] = ZmMsg.phoneLabel;
	buffer[_i++] = "</td><td class=rowValue><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_PHONE\" tabindex=\"600\"></div></td></tr><tr valign=top><td class=rowLabel>";
	buffer[_i++] = ZmMsg.imLabel;
	buffer[_i++] = "</td><td class=rowValue><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_IM\" tabindex=\"700\"></div></td></tr><tr valign=top><td class=rowLabel>";
	buffer[_i++] = ZmMsg.addressLabel;
	buffer[_i++] = "</td><td class=rowValue><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_ADDRESS\" tabindex=\"800\"></div></td></tr><tr valign=top><td class=rowLabel>";
	buffer[_i++] = ZmMsg.urlLabel;
	buffer[_i++] = "</td><td class=rowValue><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_URL\" tabindex=\"900\"></div></td></tr><tr valign=top><td class=rowLabel>";
	buffer[_i++] = ZmMsg.otherLabel;
	buffer[_i++] = "</td><td class=rowValue><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_OTHER\" tabindex=\"1000\"></div></td></tr><tr valign=top><td class=rowLabel>";
	buffer[_i++] = ZmMsg.notesLabel;
	buffer[_i++] = "</td><td class=rowValue><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_NOTES\" tabindex=\"1100\"></div></td></tr>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "abook.Contacts#ZmEditContactView_body"
}, false);

AjxTemplate.register("abook.Contacts#ZmEditContactViewRows", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<table border=0 cellspacing=0 cellpadding=0><tbody id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_rows\" tabindex=\"100\"></tbody></table>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "abook.Contacts#ZmEditContactViewRows"
}, false);

AjxTemplate.register("abook.Contacts#ZmEditContactViewDetailsButton", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<span id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_title\" class=\"ZButtonBorder ZWidgetTitle\"></span>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "abook.Contacts#ZmEditContactViewDetailsButton"
}, false);

AjxTemplate.register("abook.Contacts#ZmEditContactViewInputSelect", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<table border=0 cellpadding=0 cellspacing=0><tr><td style='padding-right:3'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_input' tabindex=\"100\"></div></td><td><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_select' tabindex=\"200\"></div></td></tr></table>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "abook.Contacts#ZmEditContactViewInputSelect"
}, false);

AjxTemplate.register("abook.Contacts#ZmEditContactViewInputDoubleSelect", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<table border=0 cellpadding=0 cellspacing=0><tr><td style='padding-right:3'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_input' tabindex=\"100\"></div></td><td><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_select' tabindex=\"200\"></div></td><td><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_select2' tabindex=\"300\"></div></td></tr></table>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "abook.Contacts#ZmEditContactViewInputDoubleSelect"
}, false);

AjxTemplate.register("abook.Contacts#ZmEditContactViewOther", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<table border=0 cellpadding=0 cellspacing=0><tr><td style='padding-right:3'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_input' tabindex=\"100\"></div></td><td style='padding-right:3'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_picker' tabindex=\"200\"></div></td><td style='padding-left:14px'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_select' tabindex=\"300\"></div></td></tr></table>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "abook.Contacts#ZmEditContactViewOther"
}, false);

AjxTemplate.register("abook.Contacts#ZmEditContactViewAddressRow", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<table><tr id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_row\" class=\"DwtFormRow\" valign=top><td><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "\" tabindex=\"100\"></div></td><td><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_add\" class=\"DwtFormRowAdd\" tabindex=\"200\"></div></td><td><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_remove\" class=\"DwtFormRowRemove\" tabindex=\"300\"></div></td></tr></table>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "abook.Contacts#ZmEditContactViewAddressRow"
}, false);

AjxTemplate.register("abook.Contacts#ZmEditContactViewAddressSelect", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<table border=0 cellpadding=0 cellspacing=0><tr valign=top><td style='padding-right:3'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_input' tabindex=\"100\"></div></td><td><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_select' tabindex=\"200\"></div></td></tr></table>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "abook.Contacts#ZmEditContactViewAddressSelect"
}, false);

AjxTemplate.register("abook.Contacts#ZmEditContactViewAddress", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<table style='position:relative;left:-3'><tr><td><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_STREET\" tabindex=\"100\"></div></td></tr></table><table style='position:relative;left:-3'><tr><td><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CITY\" tabindex=\"200\"></div></td><td>,&nbsp;</td><td><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_STATE\" tabindex=\"300\"></div></td><td>&nbsp;&nbsp;</td><td><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_ZIP\" tabindex=\"400\"></div></td></tr></table><table style='position:relative;left:-3'><tr><td><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_COUNTRY\" tabindex=\"500\"></div></td></tr></table>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "abook.Contacts#ZmEditContactViewAddress",
	"class": "DwtForm ZmEditContactViewAddress"
}, false);

AjxTemplate.register("abook.Contacts#SplitView", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<table border=0 cellpadding=0 cellspacing=5 width=100% height=100%><col width=230></col><tr><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_alphabetbar'></td></tr><tr height=100%><td valign=top><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_splitview' style='position:relative;width:100%;height:100%'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_listview' style='position:absolute;width:230px;height:100%;left:0px'></div><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_sash' style='position:absolute;width:8px;height:100%;left:230px'></div><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_contentCell' style='position:absolute;height:100%;left:238px'><div class='ZmContactInfoView' id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_content'></div></div></div></td></tr></table>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "abook.Contacts#SplitView"
}, false);

AjxTemplate.register("abook.Contacts#SplitView_content", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] =  AjxTemplate.expand("abook.Contacts#SplitView_header", data) ;
	buffer[_i++] = "<table id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_details'>";
	buffer[_i++] =  ZmContactSplitView.showContactEmails(data) ;
	buffer[_i++] =  ZmContactSplitView.showContactPhones(data) ;
	buffer[_i++] =  ZmContactSplitView.showContactIMs(data) ;
	buffer[_i++] =  ZmContactSplitView.showContactAddresses(data) ;
	buffer[_i++] =  ZmContactSplitView.showContactUrls(data) ;
	buffer[_i++] =  ZmContactSplitView.showContactOther(data) ;
	buffer[_i++] =  ZmContactSplitView.showContactNotes(data) ;
	buffer[_i++] =  ZmContactSplitView.showContactDLMembers(data) ;
	buffer[_i++] = "</table>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "abook.Contacts#SplitView_content"
}, false);

AjxTemplate.register("abook.Contacts#SplitView_header", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	
			var contact		= data.contact;
			var imageUrl	= contact.getImageUrl();
			var accountName = appCtxt.multiAccounts && contact.account && contact.account.getDisplayName();
			var imgClassName = contact.isGroup() ? "GroupPerson_48" : "Person_48"; 
		
	buffer[_i++] = "<table border=0 cellspacing=2 cellpadding=2 width=100% class='contactHeaderTable ";
	buffer[_i++] =  data.contactHdrClass ;
	buffer[_i++] = "'><tr>";
	 if (imageUrl) { 
	buffer[_i++] = "<td valign=bottom width=48 rowspan=2><img src=\"";
	buffer[_i++] =  imageUrl ;
	buffer[_i++] = "\" \n";
	buffer[_i++] = "\t\t\t\t";
	 if (AjxEnv.isIE) { 
	buffer[_i++] = "\n";
	buffer[_i++] = "\t\t\t\t\theight=48\n";
	buffer[_i++] = "\t\t\t\t";
	 } else { 
	buffer[_i++] = "\n";
	buffer[_i++] = "\t\t\t\t\tstyle=\"max-width:48px;max-height:48px\"\n";
	buffer[_i++] = "\t\t\t\t";
	 } 
	buffer[_i++] = "\n";
	buffer[_i++] = "\t\t\t\t border=0></td>";
	 } else { 
	buffer[_i++] = "<td valign=bottom width=48 rowspan=2>";
	buffer[_i++] =  AjxImg.getImageHtml(imgClassName) ;
	buffer[_i++] = "</td>";
	 } 
	buffer[_i++] = "<td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_tags' align='right' colspan=2></td></tr><tr><td valign=bottom>";
	buffer[_i++] =  AjxTemplate.expand("#SplitView_headerName", data) ;
	buffer[_i++] =  AjxTemplate.expand("#SplitView_headerJobInfo", data) ;
	buffer[_i++] = "</td><td valign=bottom align='right'>";
	 if (accountName) { 
	buffer[_i++] = "<table border=0 cellpadding=0 cellspacing=0 width=1%><tr><td class='contactLocation' nowrap=\"nowrap\">";
	buffer[_i++] =  ZmMsg.accountLabel ;
	buffer[_i++] = "&nbsp;</td><td class='companyFolder' colspan=10>";
	buffer[_i++] =  AjxStringUtil.htmlEncode(accountName) ;
	buffer[_i++] = "</td></tr></table>";
	 } 
	buffer[_i++] = "<table border=0 cellpadding=0 cellspacing=0 width=1%><tr><td class='contactLocation' nowrap=\"nowrap\">";
	buffer[_i++] =  ZmMsg.locationLabel ;
	buffer[_i++] = "&nbsp;</td>";
	 if (data.addrbook) { 
	buffer[_i++] = "<td width=20>";
	buffer[_i++] =  AjxImg.getImageHtml(data.addrbook.getIcon()) ;
	buffer[_i++] = "</td><td>&nbsp;</td><td class='companyFolder'>";
	buffer[_i++] =  data.addrbook.getName() ;
	buffer[_i++] = "</td>";
	 } else if (data.isGal) { 
	buffer[_i++] = "<td width=20>";
	buffer[_i++] =  AjxImg.getImageHtml("GAL") ;
	buffer[_i++] = "</td><td>&nbsp;</td><td class='companyFolder'>";
	buffer[_i++] =  ZmMsg.GAL ;
	buffer[_i++] = "</td>";
	 } 
	buffer[_i++] = "</tr></table></td></tr></table>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "abook.Contacts#SplitView_header"
}, false);

AjxTemplate.register("abook.Contacts#SplitView_headerName", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	
	        var contact		= data.contact;
	        var isPhonetic  = appCtxt.get(ZmSetting.PHONETIC_CONTACT_FIELDS);
	        var fullnameHtml= contact.getFullNameForDisplay(isPhonetic);
	        if (!isPhonetic) {
	            fullnameHtml = AjxStringUtil.htmlEncode(fullnameHtml);
	        }
	        var nickname	= contact.getAttr(ZmContact.F_nickname);
	    
	buffer[_i++] = "<div class='contactHeader ";
	buffer[_i++] =  data.isInTrash ? "Trash" : "" ;
	buffer[_i++] = "'>";
	buffer[_i++] =  fullnameHtml ;
	buffer[_i++] = "</div>";
	 if (nickname) { 
	buffer[_i++] = "<div class='companyName'>&ldquo;";
	buffer[_i++] =  AjxStringUtil.htmlEncode(nickname) ;
	buffer[_i++] = "&rdquo;</div>";
	 } 

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "abook.Contacts#SplitView_headerName"
}, false);

AjxTemplate.register("abook.Contacts#SplitView_headerJobInfo", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	
	        var contact		= data.contact;
	        var isPhonetic  = appCtxt.get(ZmSetting.PHONETIC_CONTACT_FIELDS);
	        var company		= contact.getAttr(ZmContact.F_company);
	        var title		= contact.getAttr(ZmContact.F_jobTitle);
	        var department	= contact.getAttr(ZmContact.F_department);
	    
	 if (title || department) { 
	buffer[_i++] = "<div class='companyName'>";
	 if (title) { 
	buffer[_i++] = "<span>";
	buffer[_i++] =  AjxStringUtil.htmlEncode(title) ;
	buffer[_i++] = "</span>";
	 } 
	 if (department) { 
	 if (title) { 
	buffer[_i++] = "\n";
	buffer[_i++] = "                &nbsp;-&nbsp;\n";
	buffer[_i++] = "            ";
	 } 
	buffer[_i++] = "<span>";
	buffer[_i++] =  AjxStringUtil.htmlEncode(department) ;
	buffer[_i++] = "</span>";
	 } 
	buffer[_i++] = "</div>";
	 } 
	 if (company) {
	        var phoneticCompany = isPhonetic && contact.getAttr(ZmContact.F_phoneticCompany);
	        var companyHtml = AjxStringUtil.htmlRubyEncode(company, phoneticCompany);
	        
	buffer[_i++] = "<div class='companyName'>";
	buffer[_i++] =  companyHtml ;
	buffer[_i++] = "</div>";
	 } 

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "abook.Contacts#SplitView_headerJobInfo"
}, false);

AjxTemplate.register("abook.Contacts#SplitView_dlmember-collapsed", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<td ";
	buffer[_i++] =  data.expandTdText ;
	buffer[_i++] = ">";
	buffer[_i++] =  data.html.join("") ;
	buffer[_i++] = "</td><td class=rowLabel>";
	buffer[_i++] =  data.contact.canExpand ? ZmMsg.membersLabel : "" ;
	buffer[_i++] = "</td><td colspan=2 class=rowValue>";
	buffer[_i++] =  data.value ;
	buffer[_i++] = "</td>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "abook.Contacts#SplitView_dlmember-collapsed"
}, false);

AjxTemplate.register("abook.Contacts#SplitView_dlmember-expanded", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<td ";
	buffer[_i++] =  data.seenone ? "" : data.expandTdText ;
	buffer[_i++] = ">";
	buffer[_i++] =  data.html.join("") ;
	buffer[_i++] = "</td><td class='rowLabel'>";
	buffer[_i++] =  data.seenone ? "" : ZmMsg.membersLabel ;
	buffer[_i++] = "</td><td colspan=2 class='rowValue'>";
	buffer[_i++] =  data.value ;
	buffer[_i++] = "</td>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "abook.Contacts#SplitView_dlmember-expanded"
}, false);

AjxTemplate.register("abook.Contacts#SplitView_address_value", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<tr valign=top><td class=rowLabel>";
	buffer[_i++] =  data.seenone ? "" : data.label ;
	buffer[_i++] = "</td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_";
	buffer[_i++] = data.name;
	buffer[_i++] = "' class=rowValue>";
	 var address = data.address;
				if (address.Street) { 
	buffer[_i++] = "<div>";
	buffer[_i++] = address.Street;
	buffer[_i++] = "</div>";
	 }
				if (address.City || address.State || address.PostalCode) { 
	buffer[_i++] = "<div><span>";
	buffer[_i++] = address.City;
	buffer[_i++] = "</span>";
	 if (address.City && address.State) { 
	buffer[_i++] = ", ";
	 } 
	buffer[_i++] = "<span>";
	buffer[_i++] = address.State;
	buffer[_i++] = "</span>";
	 if (address.City || address.State) { 
	buffer[_i++] = "&nbsp;";
	 } 
	buffer[_i++] = "<span>";
	buffer[_i++] = address.PostalCode;
	buffer[_i++] = "</span></div>";
	 }
				if (address.Country) { 
	buffer[_i++] = "<div>";
	buffer[_i++] = address.Country;
	buffer[_i++] = "</div>";
	 } 
	buffer[_i++] = "</td><td style='color:gray' class='rowType'>&nbsp;";
	buffer[_i++] =  data.type ;
	buffer[_i++] = "</td></tr>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "abook.Contacts#SplitView_address_value"
}, false);

AjxTemplate.register("abook.Contacts#SplitView_list_item", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<tr valign=top>";
	 if (data.isDL) { 
	buffer[_i++] = "<td><div class =\"ImgBlank_16\"></div></td>";
	 } 
	buffer[_i++] = "<td class='rowLabel'>";
	buffer[_i++] =  data.seenone ? "" : data.label ;
	buffer[_i++] = "</td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_";
	buffer[_i++] = data.name1.replace(/['"]/g,'');
	buffer[_i++] = "' ";
	buffer[_i++] =  data.type ? "" : "colspan=2" ;
	buffer[_i++] = " class='rowValue'>";
	buffer[_i++] =  data.value ;
	buffer[_i++] = "</td>";
	 if (data.type) { 
	buffer[_i++] = "<td style='color:gray' class='rowType'>&nbsp;";
	buffer[_i++] =  data.type ;
	buffer[_i++] = "</td>";
	 } 
	buffer[_i++] = "</tr>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "abook.Contacts#SplitView_list_item"
}, false);

AjxTemplate.register("abook.Contacts#SplitViewGroup", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] =  AjxTemplate.expand("abook.Contacts#SplitView_header", data) ;
	buffer[_i++] = "<table>";
	buffer[_i++] =  ZmContactSplitView.showContactGroup(data) ;
	buffer[_i++] = "</table>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "abook.Contacts#SplitViewGroup"
}, false);

AjxTemplate.register("abook.Contacts#SimpleView-NoResults", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<table width='100%' cellspacing=0 cellpadding=1><tr><td class='NoResults' valign='top' ><br/><br/>";
	buffer[_i++] =  AjxMsg.noResults ;
	buffer[_i++] = "</td></tr></table>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "abook.Contacts#SimpleView-NoResults"
}, false);

AjxTemplate.register("abook.Contacts#GroupView", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<!-- title bar --><table cellspacing=0 cellpadding=0 width=100%><tr class='contactHeaderRow' id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_headerRow'><td width=20><center>";
	buffer[_i++] =  AjxImg.getImageHtml("Group") ;
	buffer[_i++] = "</center></td><td><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_title' class='contactHeader'></div></td><td align='right' id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_tags'></td></tr></table><table border=0 cellpadding=5 cellspacing=5 width=100% height=100%><tr><!-- content: left pane --><td width=50% valign=top><table border=0 cellpadding=2 cellspacing=2 width=100%><tr><td nowrap>*&nbsp;";
	buffer[_i++] =  ZmMsg.groupNameLabel ;
	buffer[_i++] = "&nbsp;<input type='text' autocomplete='off' size=18 id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_groupName'></td><td><table border=0 cellpadding=0 cellspacing=0><tr><td nowrap>";
	buffer[_i++] =  ZmMsg.addressBookLabel ;
	buffer[_i++] = "&nbsp;</td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_folderSelect'></td></tr></table></td></tr><tr><td colspan='2' nowrap>*&nbsp;";
	buffer[_i++] =  ZmMsg.groupMembers ;
	buffer[_i++] = "</td></tr></table><div class='groupMembers'  id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_groupMembers'></div><table border=0 cellpadding=3 cellspacing=2 width=100%><tr><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_delButton'></td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_delAllButton'></td><td width=100%></td></tr></table></td><!-- content: right pane --><td width=50% valign=top><fieldset><legend class='groupFieldset'>";
	buffer[_i++] =  ZmMsg.addMembers ;
	buffer[_i++] = "</legend><table border=0>";
	 if (data.detailed) { 
	buffer[_i++] = "<tr id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_searchNameRow'><td align=right>";
	buffer[_i++] =  ZmMsg.nameLabel ;
	buffer[_i++] = "</td><td><input type='text' id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_searchNameField'></td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_searchButton' valign=top rowspan=3></td></tr>";
	 if (appCtxt.get(ZmSetting.PHONETIC_CONTACT_FIELDS)) { 
	buffer[_i++] = "<tr id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_searchPhoneticRow'><td align=right>";
	buffer[_i++] =  ZmMsg.phoneticNameLabel ;
	buffer[_i++] = "</td><td><input type='text' id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_searchPhoneticField'></td></tr>";
	 } 
	buffer[_i++] = "<tr id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_searchEmailRow'><td align=right>";
	buffer[_i++] =  ZmMsg.emailAddrLabel ;
	buffer[_i++] = "</td><td><input type='text' id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_searchEmailField'></td></tr><tr id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_searchDepartmentRow'><td align=right>";
	buffer[_i++] =  ZmMsg.departmentLabel ;
	buffer[_i++] = "</td><td><input type='text' id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_searchDepartmentField'></td></tr>";
	 } else { 
	buffer[_i++] = "<tr id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_searchRow'><td align=right>";
	buffer[_i++] =  ZmMsg.findLabel ;
	buffer[_i++] = "</td><td><input type='text' id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_searchField'></td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_searchButton'></td></tr>";
	 } 
	 if (data.showSearchIn) { 
	buffer[_i++] = "<tr><td align=right>";
	buffer[_i++] =  ZmMsg.searchIn ;
	buffer[_i++] = "</td><td colspan=2 id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_listSelect'></td></tr>";
	 } 
	buffer[_i++] = "</table><div class='groupMembers' id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_listView'></div><table border=0 cellpadding=3 cellspacing=2 width=100%><tr><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_addButton'></td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_addAllButton'></td><td width=100%></td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_prevButton'></td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_nextButton'></td></tr></table><table border=0 cellpadding=3 cellspacing=2 width=100%><tr><td>";
	buffer[_i++] = ZmMsg.enterAddrBelow;
	buffer[_i++] = "</td></tr><tr><td><textarea class='groupMembers' id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_addNewField'></textarea></td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_addNewButton'></td></tr></table></fieldset></td></tr></table>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "abook.Contacts#GroupView"
}, false);

AjxTemplate.register("abook.Contacts#ZmAlphabetBar", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<center><table class='AlphabetBarTable' border=0 cellpadding=0 cellspacing=0 width=80% id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_alphabet'><tr>";
	 for (var i = 0; i < data.numLetters; i++) { 
	buffer[_i++] = "<td _idx='";
	buffer[_i++] =  i ;
	buffer[_i++] = "' onclick='ZmContactAlphabetBar.alphabetClicked(this\n";
	buffer[_i++] = "\t\t\t\t\t\t";
	 if (i > 0) { 
	buffer[_i++] = "\n";
	buffer[_i++] = "\t\t\t\t\t\t\t, \"";
	buffer[_i++] =  data.sortVals[data.alphabet[i]] || data.alphabet[i] ;
	buffer[_i++] = "\"\n";
	buffer[_i++] = "\t\t\t\t\t\t\t";
	 if (i+1 < data.numLetters) { 
	buffer[_i++] = "\n";
	buffer[_i++] = "\t\t\t\t\t\t\t\t, \"";
	buffer[_i++] =  data.endSortVals[data.alphabet[i]] || data.alphabet[i+1] ;
	buffer[_i++] = "\"\n";
	buffer[_i++] = "\t\t\t\t\t\t\t";
	 } 
	 } 
	buffer[_i++] = "\n";
	buffer[_i++] = "\t\t\t\t\t\t); return false;'\n";
	buffer[_i++] = "\t\t\t\t\t\t";
	 if (data.alphabet[i].length > 1) { 
	buffer[_i++] = " colspan='";
	buffer[_i++] =  data.alphabet[i].length/2 + 1 ;
	buffer[_i++] = "' ";
	 } 
	buffer[_i++] = "\n";
	buffer[_i++] = "\t\t\t\t\t\tclass='DwtButton AlphabetBarCell' onmouseover='ZmContactAlphabetBar._onMouseOver(this)' onmouseout='ZmContactAlphabetBar._onMouseOut(this)' ";
	buffer[_i++] =  (i>0) ? " style='border-left-width:0;'>" : ">" ;
	buffer[_i++] =  data.alphabet[i] ;
	buffer[_i++] = "</td>";
	 } 
	buffer[_i++] = "</tr></table></center>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "abook.Contacts#ZmAlphabetBar"
}, false);

AjxTemplate.register("abook.Contacts#Tooltip", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<table cellpadding=0 cellspacing=0 border=0 width=250><tr><td colspan=2 valign=top><div style='border-bottom: 1px solid black; margin-bottom:2px'><table cellpadding=0 cellspacing=0 border=0 width=100%><tr valign='bottom'><td style='font-weight:bold'>";
	buffer[_i++] =  AjxStringUtil.htmlEncode(data.entryTitle); ;
	buffer[_i++] = "</td><td align='right'>";
	buffer[_i++] =  AjxImg.getImageHtml(data.contact.getIcon()); ;
	buffer[_i++] = "</td></tr></table></div></td></tr>";
	 if (data.contact.isGroup()) {
				var members = data.contact.getGroupMembers().good.getArray();
				for (var i = 0; i < members.length; i++) {
			
	buffer[_i++] = "<tr><td width=20>";
	buffer[_i++] =  AjxImg.getImageHtml("Message") ;
	buffer[_i++] = "</td><td>";
	buffer[_i++] =  AjxStringUtil.htmlEncode(members[i].toString()) ;
	buffer[_i++] = "</td></tr>";
		} 
			} else {
			
	buffer[_i++] =  AjxTemplate.expand("abook.Contacts#AddTooltipEntry", {data:data, field:"fullName"}) ;
	buffer[_i++] =  AjxTemplate.expand("abook.Contacts#AddTooltipEntry", {data:data, field:"jobTitle"}) ;
	buffer[_i++] =  AjxTemplate.expand("abook.Contacts#AddTooltipEntry", {data:data, field:"company"}) ;
	buffer[_i++] =  AjxTemplate.expand("abook.Contacts#AddTooltipEntry", {data:data, field:"mobilePhone"}) ;
	buffer[_i++] =  AjxTemplate.expand("abook.Contacts#AddTooltipEntry", {data:data, field:"workPhone"}) ;
	buffer[_i++] =  AjxTemplate.expand("abook.Contacts#AddTooltipEntry", {data:data, field:"homePhone"}) ;
	 if (data.contact.isGal) {
					var emails = data.contact.getEmails();
					for (var i = 0; i < emails.length && emails.length <= 3; i++) {
				
	buffer[_i++] =  AjxTemplate.expand("abook.Contacts#AddTooltipEntry", {data:data, field:"email", val:emails[i]}) ;
		} 
	buffer[_i++] =  AjxTemplate.expand("abook.Contacts#AddTooltipEntry", {data:data, field:"notes", truncate:500}) ;
		} else { 
	buffer[_i++] =  AjxTemplate.expand("abook.Contacts#AddTooltipEntry", {data:data, field:"email", val:data.contact._lookupEmail || data.email}) ;
		} 
		} 
	 if (data.buddy) { 
	buffer[_i++] = "<tr><td colspan=\"2\"><table align=\"center\"><tr><td>";
	buffer[_i++] =  AjxImg.getImageHtml(data.buddy.getPresence().getIcon()) ;
	buffer[_i++] = "</td><td>";
	buffer[_i++] =  AjxStringUtil.htmlEncode(data.buddy.getPresence().getShowText()) ;
	buffer[_i++] = "</td></tr></table></td></tr>";
	 } 
	 if (data.hint) { 
	buffer[_i++] = "<tr><td colspan=\"2\"><div class='TooltipHint'><hr color=black size=1>";
	buffer[_i++] =  data.hint ;
	buffer[_i++] = "</div></td></tr>";
	 } 
	buffer[_i++] = "</table>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "abook.Contacts#Tooltip"
}, false);

AjxTemplate.register("abook.Contacts#TooltipNotInAddrBook", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	 if (data.hint) { 
	buffer[_i++] = "<table cellpadding=0 cellspacing=0 border=0 width=250><tr><td valign=top>";
	 if (appCtxt.get(ZmSetting.CONTACTS_ENABLED)) { 
	buffer[_i++] = "<div style='border-bottom: 1px solid black;'><table cellpadding=0 cellspacing=0 border=0 width=100%><tr valign='bottom'><td style='font-weight:bold'>";
	buffer[_i++] =  ZmMsg.newContact ;
	buffer[_i++] = "</td><td align='right'>";
	buffer[_i++] =  AjxImg.getImageHtml("NewContact"); ;
	buffer[_i++] = "</td></tr></table></div>";
	 } 
	buffer[_i++] = "<div class=\"TooltipNotInAddrBook\">";
	buffer[_i++] =  AjxStringUtil.htmlEncode(data.addrstr) ;
	buffer[_i++] = "</div><hr color=black size=1><div class='TooltipHint'>";
	buffer[_i++] =  data.hint ;
	buffer[_i++] = "</div></td></tr></table>";
	 } else { 
	buffer[_i++] = "<span style='font-weight:bold'>";
	buffer[_i++] =  ZmMsg.emailLabel ;
	buffer[_i++] = " </span>";
	buffer[_i++] =  AjxStringUtil.htmlEncode(data.addrstr) ;
	 } 

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "abook.Contacts#TooltipNotInAddrBook"
}, false);

AjxTemplate.register("abook.Contacts#AddTooltipEntry", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	 if (data.val == null) {
		    var isPhonetic  = appCtxt.get(ZmSetting.PHONETIC_CONTACT_FIELDS);
			data.val = data.field == "fullName" ? data.data.contact.getFullNameForDisplay(isPhonetic) : data.data.contact.getAttr(data.field);
		} 
	 if (data.val != null && data.val != "") { 
	buffer[_i++] = "<tr valign=top><td class='ZmTooltipLabel'>";
	buffer[_i++] =  AjxMessageFormat.format(ZmMsg.makeLabel,
											 AjxStringUtil.htmlEncode(ZmContact._AB_FIELD[data.field])) ;
	buffer[_i++] = "</td>";
	 if (data.truncate) { 
	buffer[_i++] = "<td>";
	buffer[_i++] =  AjxMessageFormat.format(ZmMsg.makeLabel,
											AjxStringUtil.htmlEncode(data.val.substring(0,data.truncate))) ;
	buffer[_i++] = "</td>";
	 } else {
			      var field = "";
			      if (data.field == "fullName" ) {
			        field = isPhonetic ? data.val : AjxStringUtil.htmlEncode(data.val); 
			      }
			      else {
			        field = AjxStringUtil.htmlEncode(data.val);
			      }
	              
	buffer[_i++] = "<td style='white-space:nowrap;'>";
	buffer[_i++] =  field ;
	buffer[_i++] = "</td>";
	 } 
	buffer[_i++] = "</tr>";
	 } 

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "abook.Contacts#AddTooltipEntry"
}, false);

AjxTemplate.register("abook.Contacts#ZmContactPicker", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<div class='ZmContactPicker'><table border=0 cellpadding=1 cellspacing=1 width=100%><tr><td width=600><table border=0 cellpadding=0 cellspacing=0>";
	 if (data.detailed) { 
	buffer[_i++] = "<tr><td valign=middle colspan=3 width=400><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_searchIcon' class=\"ImgSearch\"></div></td></tr><tr id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_searchNameRow'><td align=right>";
	buffer[_i++] =  ZmMsg.nameLabel ;
	buffer[_i++] = "</td><td>&nbsp;<input type='text' autocomplete='off' size=30 nowrap id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_searchNameField'>&nbsp;</td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_searchButton' rowspan=3 valign=top></td></tr>";
	 if (appCtxt.get(ZmSetting.PHONETIC_CONTACT_FIELDS)) { 
	buffer[_i++] = "<tr id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_searchPhoneticRow'><td align=right>";
	buffer[_i++] =  ZmMsg.phoneticNameLabel ;
	buffer[_i++] = "</td><td>&nbsp;<input type='text' autocomplete='off' size=30 nowrap id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_searchPhoneticField'>&nbsp;</td></tr>";
	 } 
	buffer[_i++] = "<tr id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_searchEmailRow'><td align=right>";
	buffer[_i++] =  ZmMsg.emailAddrLabel ;
	buffer[_i++] = "</td><td>&nbsp;<input type='text' autocomplete='off' size=30 nowrap id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_searchEmailField'>&nbsp;</td></tr><tr id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_searchDepartmentRow'><td align=right>";
	buffer[_i++] =  ZmMsg.departmentLabel ;
	buffer[_i++] = "</td><td>&nbsp;<input type='text' autocomplete='off' size=30 nowrap id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_searchDepartmentField'>&nbsp;</td></tr>";
	 } else { 
	buffer[_i++] = "<tr id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_searchRow'><td width=20 valign=middle><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_searchIcon' class=\"ImgSearch\"></div></td><td><input type='text' autocomplete='off' size=30 nowrap id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_searchField'>&nbsp;</td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_searchButton'></td></tr>";
	 } 
	buffer[_i++] = "</table></td><td align=right valign=bottom>";
	 if (data.showSelect) { 
	buffer[_i++] = "<table border=0 cellpadding=0 cellspacing=0><tr><td class='Label nobreak'>";
	buffer[_i++] =  ZmMsg.showNames ;
	buffer[_i++] = "&nbsp;</td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_listSelect'></td></tr></table>";
	 } 
	buffer[_i++] = "</td></tr></table><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_chooser'></div><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_paging'><center><table border=0 cellspacing=0 cellpadding=0 width=100%><tr><td>&nbsp;&nbsp;&nbsp;</td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_pageLeft'></td><td width=100%></td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_pageRight' align=right></td></tr></table></center></div></div>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "abook.Contacts#ZmContactPicker"
}, false);

AjxTemplate.register("abook.Contacts#ZmContactSearch", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_preamble'></div><table border=0 cellpadding=1 cellspacing=1 width=100%><tr><td><table border=0 cellpadding=0 cellspacing=0><tr><td width=20 valign=middle><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_searchIcon' class=\"ImgSearch\"></div></td><td><input type='text' autocomplete='off' size=30 nowrap id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_searchField'>&nbsp;</td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_searchButton'></td></tr></table></td><td align=right>";
	 if (data.showSelect) { 
	buffer[_i++] = "<table border=0 cellpadding=0 cellspacing=0><tr><td class='Label nobreak'>";
	buffer[_i++] =  ZmMsg.showNames ;
	buffer[_i++] = "&nbsp;</td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_folders'></td></tr></table>";
	 } 
	buffer[_i++] = "</td></tr></table><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_results' class='contactSearchResultsDiv'></div>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "abook.Contacts#ZmContactSearch"
}, false);

}
